#ifndef CUIKSYSTEMH
#define CUIKSYSTEMH

#include "list.h"
#include "parameters.h"
#include "boolean.h"
#include "defines.h"

#include <stdlib.h>
#if (SIMPLEX_AVAILABLE)
  #include <coin/Clp_C_Interface.h>
#endif


//************************************************************************************/
/** \brief Work in angular space.

   If set, the box sizes/volumes are computed in angular space.
*/
#define ANGULAR_SPACE 1


//************************************************************************************/
/** 
   \brief One of the possible outcomes when reducing a box along a range.

    One of the possible outcomes when reducing a box along a range.
*/
#define CONTRACTED 0

/** 
   \brief One of the possible outcomes when reducing a box along a range.

    One of the possible outcomes when reducing a box along a range.
*/
#define REDUCED 1

/** 
   \brief One of the possible outcomes when reducing a box along a range.

    One of the possible outcomes when reducing a box along a range.
*/
#define REDUCED_WITH_SOLUTION 2

/** 
   \brief One of the possible outcomes when reducing a box along a range.

    One of the possible outcomes when reducing a box along a range.
*/
#define EMPTY 3

/** 
   \brief One of the possible outcomes when reducing a box along a range.

    One of the possible outcomes when reducing a box along a range.
*/
#define NOT_REDUCED 4

/************************************************************************************/
/** \brief Depth first search.

   In this mode, boxes to be reduced are sorted so that boxes with a
   higher level (i.e., resulting from more bisections) are treated first. 
*/
#define DEPTH_FIRST_SEARCH 1

/************************************************************************************/
/** \brief Breadth first search.

   In this mode, boxes to be reduced are sorted so that boxes with a
   smaller level (i.e., resulting from less bisections) are treated first.
*/
#define BREADTH_FIRST_SEARCH 2

/************************************************************************************/
/** \brief Select the search mode.

   Select the search mode.
*/
#define SEARCH_MODE DEPTH_FIRST_SEARCH

/** \file cuiksystem.h
    \brief Definition of the TCuikSystem type and the associated functions.

    Definition of the TCuikSystem type and the associated functions.

    \sa TCuikSystem, cuiksystem.c, readcuiksystem.l, readcuiksystem.y,
*/
/************************************************************************************/


/************************************************************************************/
/** \brief Point, relative to the size of the selected box side, where we
           split a box.

   Point, relative to the size of the selected box side, where we 
   split a box. 
*/

#if (RANDOMNESS)
  #define CUT_POINT 0.45+0.1*randomDouble()
#else
  #define CUT_POINT 0.5
#endif
/************************************************************************************/
/** 
   \brief Information about the performace of the solver.

   Information about the performace of the solver.
*/
typedef struct {
  unsigned int nBoxes;      /**< Number of processed boxes. */
  unsigned int nReductions; /**< Number of reductions. In some cases boxes are reduced more than once */
  unsigned int nBisected;   /**< Number of reductions boxes. */
  unsigned int nEmpty;      /**< Number of empty boxes. */
  unsigned int nSols;       /**< Number of solution boxes. */
  unsigned int vSols;       /**< Number of verified solution boxes. */
  unsigned int maxLevel;    /**< Maximum level of the processed boxes. */
} TStatistics;

/** 
    \brief Initializes statistics.

    Initializes the statistics. 

    \param nBoxes The number of processed boxes.
    \param nReductions The number of reductions.
    \param nBisected The number of bisected boxes.
    \param nEmpty The number of empty boxes.
    \param nSols The number of solution boxes.
    \param vSols The number of verified solution boxes.
    \param maxLevel Maximum level of the processed boxes.
    
    \param st The statistics initialize.
*/
void InitStatistics(unsigned int nBoxes,unsigned int nReductions,unsigned int nBisected,
		    unsigned int nEmpty,unsigned int nSols,unsigned int vSols,
		    unsigned int maxLevel,TStatistics *st);

/** 
    \brief Resets statistics.

    Sets all the statistics to 0.

    \param st The statistics to initialize.
*/
void ResetStatistics(TStatistics *st);

/** 
   \brief Accumulates statistics.

   
*/
void AccumulateStatistics(TStatistics *st,TStatistics *stg);

/** 
    \brief Prints the statistics.

    Prints the statistics.

    \param f The file where to print the statistics.
    \param title Title of the statistics.
    \param st The statistics to print.
*/
void PrintStatistics(FILE *f,char *title,TStatistics *st);

/** 
    \brief Destructor.

    Releases any memory used by the statistics.

    \param st The statistics to delete.
*/
void DeleteStatistics(TStatistics *st);

/****************************************************************************/
/* Box structure */
/****************************************************************************/


/** 
    \brief A box.

    An orthotope defining a domain of interest and the evaluation of all equations in 
    the corners of this box.

    This structure can not be used without the corresponding \ref TCuikSystem structure.
*/
typedef struct {
  unsigned int level; /**< Box level. */
  unsigned int d;     /**< Dimension. Number of ranges. */
  double *l;          /**< Lower limit for each variable. */
  double *u;          /**< Upper limit for each variable. */
} TCuikBox;


/****************************************************************************/
/* The simplex structure */
/****************************************************************************/

#if (SIMPLEX_AVAILABLE)
  /** 
     \brief The simples tableau type when using 
            <A HREF="http://www.coin-or.org/projects/Clp.xml">Clp</A>.
  */
  #define SimplexType __attribute__ ((aligned(8))) Clp_Simplex

  /** \brief Simplex timeout.

    Maximum time (in seconds) we give the simplex process to return a solution when
    minimizing/mazimizing for a single variable.
  */
  #define SIMPLEX_TIMEOUT  10

  /** 
     \brief The simplex structure.

     The simplex structre. Includes all the information necessary to
     use the simplex.
  */
  typedef struct {
    SimplexType *lp;          /**< The object provided by the library. */ 
    unsigned int nRows;       /**< Maximum number of rows. */ 
    unsigned int nCols;       /**< Maximum number of columns. */
    double *columnLower;      /**< Lower limit for the columns. */ 
    double *columnUpper;      /**< Upper limit for the columns. */
    double *obj;              /**< Coefficients of the objective function. */
    double *rowLower;         /**< Lower limit for the rows. */ 
    double *rowUpper;         /**< Upper limit for the rows. */
    int *rowStarts;           /**< Initial position (in columns and elements) for each row. 
				   The last entry is the last position for last row. */ 
    int *columns;             /**< Indices of the columns used in each row. */ 
    double *elements;         /**< The value for each matrix entry. */
  } TSimplex;

#endif


/****************************************************************************/
/* The problem structure */
/****************************************************************************/

/** 
    \brief A cuiksystem, i.e., a set of variables and equations defining a position analysis problem.

    This structure includes all the information defining a position analysis problem, that is,
    a set of varaibles and a set of equations using these variables.


    \sa cuiksystem.h cuiksystem.c  
*/
typedef struct { 
  unsigned int ne;         /**< Number of equations. */
  unsigned int nv;         /**< Number of variables. */
  unsigned int *nve;       /**< Number of variables per equation. */
  unsigned int **ve;       /**< Variables in each equation. */
  unsigned int *nc;        /**< Number of 'corners' for each equation: 2^nve. */

  boolean *usedVars;       /**< TRUE for variables used in the equations.
			        When eliminating variables some of them are not present in the equations. */
  
  unsigned int *varType;   /**< Variable type: VAR_ROT VAR_TRANS */
  signed int *offset;      /**< Offset for angle conversion.
			        We assume that only two offsets are possible: -1-> -pi/2   1-> pi/2.
			        The offsets only apply to rotational variables and the combination
			        of offsets define a sub-problem. */
  
  double **f0;             /**< Evaluation of the functions in the reference box. One per equation. */
  
  TCuikBox *b0;            /**< Initial ranges for the variables */
  
  /* Auxiliary variables for the trapezoid-based reduction (allocate them once) */
  double **f;              /**< Space for function evaluation. */
  
  boolean *outG;           /**< Reduction status (global) */
  double *boxL;            /**< Lower bounds for witch the functions are computed (just in case they change). */
  double *boxU;            /**< Upper bounds for witch the functions are computed (just in case they cahnge). */
  
  double *veq;             /**< Value of the equations. */

  #if (SIMPLEX_AVAILABLE)
    TSimplex sp;           /**< The simplex, if used. */
  #endif

  /* For simplification. Only used eventually, not even in all simplified systems */
  unsigned int nSimp;      /**< Number of simplications rules. Each simplification is an input
			        output function between variables:
			          xOut = S * xIn 
			        where S is a constant coeficient matrices, and xIn 
				and xOut are the all the multinear monomials of the 
				intput/ouput variables.

				Right now we only define simplification rules for 
				6dof loops where we eliminate 2 variables. Therefore
				the simplification rules have 4 input variables 
				and 2 output ones. In this case, S is a 4x16 matrix.

				We can also define simplifications of 6dof loops where 
				we eliminate 3 variables, but then there is no a one-to-one
				relation between xIn and xOut and, thus, we do not define
				a simplification rule (we do not define an 'S' matrix).
			   */
  unsigned int *nIn;       /**< Number of input variables in each simplification rule. */
  unsigned int **vIn;      /**< Indices of the input variables in each simplification rule. */
  unsigned int *nOut;      /**< Number of output variables in each simplification rule. */
  unsigned int **vOut;     /**< Indices of the output variables in each simplification rule. */
  double **S;              /**< Matrix of coefficients for the input variable monomials. */
  
  /* For Newton */
  unsigned int maxEV;      /**< Maximum number of equations/variables. */
  unsigned int minEV;      /**< Minimum number of equations/variables. */
  double *J;               /**< Jacobian. */
  int *perm;               /**< Permutation (for square systems, we solve a LS using LU-decompisition with partial pivoting). */
  double *sing;            /**< Singular values. Used for non-square system (solved via SVD). */
  int lwork;               /**< Size of the work space. Used for non-square system (solved via SVD). */
  double *work;            /**< Work space. Used for non-square system (solved via SVD). */

} TCuikSystem;


/****************************************************************************/
/* Auxiliary methods                                                        */
/****************************************************************************/


/** 
    \brief Initializes a cuik box.

    Initializes a cuik box.

    \param l The lower limits.
    \param u The upper limits.
    \param d Dimension. Number of ranges in the box.
    \param b The box.
*/
void InitCuikBox(double *l,double *u,unsigned int d,TCuikBox *b);


/** 
    \brief Prints a box.

    Prints the limits of a box.

    \param f The file.
    \param b The box.
*/
void PrintCuikBox(FILE *f,TCuikBox *b);


/** 
    \brief Destructor.

    Releases memory.

    \param b The box.
*/
void DeleteCuikBox(TCuikBox *b);

/** 
   \brief Reads a cuikbox list from a file.

   Reads a cuikbox list from a file.

   \param f The file.
   \param l The list.
*/
void ReadCuikBoxList(FILE *f,Tlist *l);

/** 
   \brief Saves a cuikbox list to a file.

   Saves a cuikbox list to a file.

   \param f The file.
   \param k Number solutions from this value.
   \param angles TRUE if we want to save in angular space.
   \param offset TRUE if the box has to be offset.
   \param error TRUE if we have to print the error.
   \param verified TRUE if the list includes verified solutions.
   \param cs The cuiksystem.
   \param l The list.
*/
void SaveCuikBoxList(FILE *f,unsigned int k,
		     boolean angles,boolean offset,boolean error,boolean verified,
		     TCuikSystem *cs,Tlist *l);

/** 
    \brief Verifies solutions.

    Tries to verify the existence of a solution in the input boxes.

    \param p The set of parameters.
    \param angles TRUE if the boxes contain angles (not tangent parameters).
    \param f_out The file where to store the remaining solutions.
    \param ns Number of cuiksystems.
    \param cs The cuiksystems.
    \param s The list of temptative solutions.
*/
void VerifyCuikBoxes(Tparameters *p,boolean angles,FILE *f_out,
		     unsigned int ns,TCuikSystem *cs,Tlist *s);
/** 
    \brief Removes spourious solutions.

    Uses the simplex procedure to clean a list of solutions obtained with the trapezoid
    reduction method.

    \param p The set of parameters.
    \param angles TRUE if the boxes contain angles (not tangent parameters).
    \param f_out The file where to store the remaining solutions.
    \param ns Number of cuiksystems.
    \param cs The cuiksystems.
    \param s The list of temptative solutions.
*/
void CleanCuikBoxes(Tparameters *p,boolean angles,FILE *f_out,
		    unsigned int ns,TCuikSystem *cs,Tlist *s);

/** 
    \brief Groups boxes that are in contact.

    Groups boxes that are in contact.

    NOTE: The boxes resulting from the clustering are not updated (the evaluation
    of the equations in the corners are not up to date).

    \param c The clustered boxes.
    \param tol Tolerance for the intersection.
    \param angles TRUE if the input boxes are angles.
    \param s The boxes to cluster.
*/
void ClusterCuikBoxes(Tlist *c,double tol,boolean angles,Tlist *s);

/** 
    \brief Converts boxes in angular space.

    Converts boxes in angular space.

    Note that we convert the boxes "in place", without copying them.
    If you need the original boxes, better copy them before the conversion.

    Moreover, the operation does not take into account the offsets in
    the angles. We assume that the input box is already offeset in tangent
    space (\ref CuikBoxOffset).

    \param usedVars The used variables.
    \param varType The variable types.
    \param s The boxes to cluster.
*/
void CuikBoxes2Angles(boolean *usedVars,unsigned int *varType,Tlist *s);

/** 
   \brief Volume of a list of boxes.

   Adds the sum of the volumes of all the boxes in the list.

   \param used Used variables.
   \param l The list.

   \return The volume. 
*/
double CuikBoxListVolume(boolean *used,Tlist *l);

/** 
   \brief Deletes a cuikbox list from a file.

   Deletes a cuikbox list from a file.

   \param l The list.
*/
void DeleteCuikBoxList(Tlist *l);

/****************************************************************************/
/* Methods to initialize/define/re-define CuikSystems (constructors et al.) */
/****************************************************************************/

/** 
    \brief Constructor.

    Creates an empty cuiksystem.

    \param cs The cuiksystem to initialize.
*/
void InitCuikSystem(TCuikSystem *cs);

/** 
    \brief Constructor from a file.

    Creates one or more cuiksystems from the information on a file. 

    The input file can be either a 6R description (following the convention
    by Manocha and Canny), a maf file (encoding a multi-affine system) 
    or a cuik file (explicitly giving the variables and equations). In the
    later case, the maf file is generated on the fly.

    \param p The set of parameters.
    \param filename The name of the file from which to read the variables and equations.
    \param csa The array with the read cuiksytems.

    \return The number of cuiksystems actually read.
*/
unsigned int ReadProblemFromFile(Tparameters *p,char *filename,TCuikSystem **csa);

/** 
    \brief Constructor from a file.

    Creates one or more cuiksystems from the information on a 6R file defined
    uisng the Manocha convention:
      - a_1 ... a_6
      - d_1 ... d_6
      - alpha_1 ... alpha_6
      - R
      - t
      .

    \param p The set of parameters.
    \param filename The name of the file from which to read the variables and equations.
    \param csa The array with the read cuiksytems.

    \return The number of cuiksystems actually created. Typically 64 or 0 if there is an
            error reading the file.
*/
unsigned int Read6RFile(Tparameters *p,char *filename,TCuikSystem **csa);

/** 
   \brief Converts from loops to a cuik file.

   Reads a file of loops and writes a file of equations (cuik file).

   In this file we do not take into account offsets. Note however that
   they can be given explicitly as variables in the loop file.

   \param p The parameters.
   \param filename The name of the file.
*/
void Loops2Cuik(Tparameters *p,char *filename);

/** 
    \brief Read a problem from a loops file.

    Generate the cuik data (num. variables, num equations, evaluation on a reference
    box, etc). from a file with the loop equations.

    \param p A set of parameters.
    \param filename The name of the file from which to read the variables and equations.
    \param csa The array of cuiksystems to define.

    \return The number of sub-problems generated. O if the file can not be found.
*/
unsigned int ReadLoopsFile(Tparameters *p,char *filename,TCuikSystem **csa);

/** 
    \brief Generate the cuik data from  a file with equations/variables.

    Generate the cuik data (num. variables, num equations, evaluation on a reference
    box, etc). from a file with the variables and equations.

    \param p A set of parameters.
    \param filename The name of the file from which to read the variables and equations.
    \param csa The array of cuiksystems to define.

    \return The number of sub-problems generated. O if the file can not be found.
*/
unsigned int ReadCuikFile(Tparameters *p,char *filename,TCuikSystem **csa);

/** 
    \brief Read a problem from a MAF file.

    Generate the cuik data (num. variables, num equations, evaluation on a reference
    box, etc). from a file in MAF format.

    \param p A set of parameters.
    \param filename The name of the file from which to read the variables and equations.
    \param csa The array of cuiksystems to define.

    \return The number of sub-problems generated. O if the file can not be found/parsed.
*/
unsigned int ReadMAFFile(Tparameters *p,char *filename,TCuikSystem **csa);

/** 
   \brief Saves a cuiksystem to a MAF file.

   MAF files include several problems, each with a different offset.
   The advantatge of MAF files is that there is no need to parse them and
   they already include the evaluation of the functions in the corners
   of the boxes to considere.

   This function stores a set of cuiksystems into a MAF file.

   \param filename The name of the output file.
   \param ns The number of systems to save.
   \param csa The array of cuiksystems.
*/
void SaveCuikSystemToMAF(char *filename,unsigned int ns,TCuikSystem *csa);


/** 
    \brief Copy constructor.

    Creates an cuiksystem from another cuiksystem.

    \param cs_dst The cuiksystem to create via copy.
    \param cs_src The cuiksystem from where to copy.
*/
void CopyCuikSystem(TCuikSystem *cs_dst,TCuikSystem *cs_src);

/** 
    \brief Gets the number of variables already in the cuiksystem.

    Returns the number of variables already in the cuiksystem.

    \param cs The cuiksystem to query.

    \return The number of variables already in the cuiksystem.
*/
unsigned int GetCSNumVariables(TCuikSystem *cs);

/** 
    \brief Gets the number of equations already in the cuiksystem.

    Returns the number of equations already in the cuiksystem.

    \param cs The cuiksystem to query.

    \return The number of equations already in the cuiksystem.
*/
unsigned int GetCSNumEquations(TCuikSystem *cs);
  
/****************************************************************************/
/*                    Methods to solve the CuikSystem.                      */
/****************************************************************************/

/** 
    \brief Determines the solution set for a cuiksystem.

    This is the entry function for the Cuik solver. It takes as input
    a cuiksystem and returns a set of boxes bounding the solution set.
    If the solutions are isolated (zero-dimensional) the output is
    a set of isolated boxes. If the solution is continuous the output
    is a set of boxes bounding the solution space. Be aware that
    isolating solution sets with high dimensionality can be expensive.
    To this end it is recommended to use the parallel version of cuik

    \param s Number of sub-problem
    \param p A set of parameters.
    \param searchSpace Boxes defining the search space. NULL if we have to use
                       the box defined from the ranges of the input variables.
    \param f_out File where to store the solution boxes. If NULL
		 the boxes are not stored in the file.
    \param solutions A list where to store the non-verified solutions, If NULL
                     the solutions are not added to the list.
    \param verifiedS A list where to store the verified solutions, This
                     can be NULL we do not want to classify the solutions.
    \param st Structure where to accumulate the statistics.  Can be NULL.
    \param cs The cuiksystem to solve.

    \sa MPI_SolveCuikSystem
*/
void SolveCuikSystem(unsigned int s,Tparameters *p,Tlist *searchSpace,
		     FILE *f_out,Tlist *solutions,Tlist *verifiedS,
		     TStatistics *st,TCuikSystem *cs);

/** 
   \brief Defines ranges for the unused variables for a list of solutions.

   Consideres each solution box (as identified by \ref SolveCuikSystem) and 
   bounds the ranges of the unsused variables from the used ones using
   \ref PropagateRanges

   Unused variables are those eliminated in the simplification process.

   \param simplify Number of removed variables in the simplification (2 or 3).s
   \param epsilon Numerical accuracy.
   \param solutions The solutions identified with the reduced set of variables.
   \param cs The cuiksystem.
*/
void PropagateSolutions(unsigned int simplify,double epsilon,Tlist *solutions,TCuikSystem *cs);

/** 
   \brief Defines ranges for the unused variables in a box.

   Bound the ranges of the unsused variables from the used ones.

   Unused variables are those eliminated in the simplification process.

   Rigth now this function is only defined for loop equations involving
   6 variables from where 2 of them are removed (put in terms of the
   remaning 4).

   \param epsilon Numerical accuracy.
   \param b The box.
   \param cs The cuiksystem.

   \return TRUE if the box is not empty.
*/
boolean PropagateRanges(double epsilon,TCuikBox *b,TCuikSystem *cs);

/** 
   \brief Intersect the boxes in a list with a given (sub-)problem.

   Check the boxes in a list and keep those which intersect with the
   ranges in a given sub-problem (the ranges given by the offsets
   defining the sub-problem). We can either discard a box or keep
   the part of the box which intersect with the sub-problem.

   \param p The set of parameters.
   \param listIn Input list of boxes.
   \param listOut Output list of boxes.
   \param cs The sub-problem.
*/
void IntersectWithCuikSystem(Tparameters *p,Tlist *listIn,Tlist *listOut,TCuikSystem *cs);

/** 
    \brief Determines the best dimensions to split a given box.

    Determines the dimension for which it is more advisable to
    split a box. The criterion to determine the split dimension
    can  be selected through the parameters.

    \param p A set of parameters.
    \param b The box to be split.
    \param cs The cuiksystem.

    \return The dimension along to which the box is to be bisected.
*/
unsigned int ComputeSplitDim(Tparameters *p,TCuikBox *b,TCuikSystem *cs);

/** 
  \brief Evaluates the equation set on a point.

  Evaluates the equation set on a point. 

  \param p The point.
  \param r The array with the evaluation of each equation. The space for
           this array should be allocated externally.
  \param cs The cuiksystem to query.
*/
void EvaluateCSEquations(double *p,double *r,TCuikSystem *cs);

/****************************************************************************/
/*               Output the information in the CuikSystem                   */
/****************************************************************************/

/** 
    \brief Prints a cuiksystem.

    Writes the cuiksystem in a file that can be <em>stdout</em>.

    \param p A set of parameters.
    \param f The stream where to write the cuiksystem.
    \param cs The cuiksystem.
*/
void PrintCuikSystem(Tparameters *p,FILE *f,TCuikSystem *cs);


/****************************************************************************/
/*                                 Destructor                               */
/****************************************************************************/

/** 
    \brief Destructor.

    Deletes a cuiksystem.

    \param cs The cuiksystem to be deleted.
*/
void DeleteCuikSystem(TCuikSystem *cs);

#endif
