#ifndef DEFINESH
#define DEFINESH

#include "error.h" /*The NEW macros call error()*/
#include <stdlib.h> /*The NEW macros call malloc,..*/
#include <fenv.h> /*FPU control*/

/** 
   \file defines.h 

   \brief Definitions of constants and macros used in several parts of the cuik library.

   Definitions of constants and macros used in several parts of the cuik library.

   \sa geom.h
*/

/************************************************************************************/
/************************************************************************************/
/** 
   \brief Floating point operations giving a value below this constant (in absolute value)
   are considered 0.

   Floating point operations giving a value below this constant (in absolute value)
   are considered 0. 

   We use 1e-11 because trigonometric operations (sin/cos/...) give an error typically
   around 1e-12. Moreover, we print doubles with 12 decimals and, thus the last one
   typically varies when performing operations. Note that 1e-11 is far below the typical
   precission used inside the simplex algorithms (1e-6).

   No to be confused with parameter \e EPSILON. ZERO is used as a general
   threshold to round towards 0 and Epsilon is used only when passing parameters
   to the simplex. For a super-numerically-stable simplex \e EPSILON and ZERO
   would be the same.
*/
#define ZERO 1e-12

/** 
   \brief If 0 we avoid any randomness in the cuiksuite behavior.

    If set to 0 we avoid any randomness in the cuiksuite behavior. This is
    basically used for debug. In norma mode, the applications in the cuiksuite
    rely on some randomness (to select the variable to split a box, to select
    the split point,...). This is usefull to improve the performance in 
    degenerate situations and it does not hinder the convergence in the
    usual cases.
    For debug purposes, though, this randomness can be quite annoying and can
    be deactivated here.

    Right now we only use random behavior when in optimized mode.

    \todo Remove this in the final version. This is for debug only
*/
#define RANDOMNESS 0

//(_DEBUG<2)

/************************************************************************************/
/************************************************************************************/
    
/** \def INF 
    \brief Infinite.
    
    Infinite.
*/
#define INF 1e10

#ifndef M_PI
  /** \brief Pi.

      Pi
  */
  #define M_PI 3.14159265358979323846
#endif


#ifndef M_PI_2
  /** \brief Pi/2.

      Pi/2
  */
  #define M_PI_2 1.57079632679489661923
#endif

#ifndef M_2PI
  /** \brief 2*Pi.

      2*Pi
  */
  #define M_2PI (2.0*M_PI)
#endif

#ifndef M_E
  /** \brief e.

      Base of the natural log.
  */
  #define M_E 2.718281828459045
#endif

#ifndef M_C
  /** 
      \brief Speed of light in meter per second.
  */
  #define M_C 299792458
#endif

#ifndef M_G
  /** 
      \brief Gravity acceleration.
  */
  #define M_G 9.80665 
#endif

/************************************************************************************/
/************************************************************************************/
/** 
   \brief Maximum number of nested included files.

   A maximum of recursively included files to avoid infinite loops.
*/
#define MAX_INCLUDES 100

/************************************************************************************/
/************************************************************************************/
/** 
   \brief Swaps two values.

   Swaps two values using an intermediate storage.

   \param a First value to swap.
   \param b Second value to swap.
   \param c Intermediate storage for the swap.
*/
#define SWAP(a,b,c) {c=a;a=b;b=c;}

/************************************************************************************/
/************************************************************************************/
/** \brief Constant to transform from degrees to radiants.

    Constant to transform from degrees to radiants.
*/
#define DEG2RAD (M_PI/180.0)

/** \brief Constant to transform from radiants to degrees.

    Constant to transform from radiants to degrees.
*/
#define RAD2DEG (180.0/M_PI)

/************************************************************************************/
/************************************************************************************/
/** \brief Returns the sign of a floating point.

     Returns the sign of a floating point: -1 for negative and 1 for possitive or zero.

     \param a The number to operate.
*/
#define SGN(a) (a>=0?1:-1)

/************************************************************************************/
/************************************************************************************/
/** \brief Adjust a real to 0,1,-1, if possible.

    Adjust a real to 0,1,-1, if possible.

    \param a The number to adjust.
*/
#define ADJUST_REAL(a) (fabs(a)<ZERO?0.0:(fabs(a-1.0)<ZERO?+1.0:(fabs(a+1.0)<ZERO?-1.0:a)))

/************************************************************************************/
/************************************************************************************/
/** \brief Forces an angle go be in [-pi,pi].

    Forces an angle go be in [-pi,pi].
    This is intended to be used with a variable as a parameter
    (using an expression would be ineficient).
*/
#define PI2PI(a) {if ((a<-M_PI-ZERO)||(a>M_PI+ZERO)) {a=fmod(a,M_2PI); while(a>M_PI) a-=M_2PI; while (a<-M_PI) a+=M_2PI;}}

/************************************************************************************/
/************************************************************************************/
/** \brief Sets the floating point operations in rounding up mode.

    Sets the floating point operations in rounding up mode.
*/
#define ROUNDUP   fesetround(FE_UPWARD)

/** \brief Sets the floating point operations in rounding down mode.

    Sets the floating point operations in rounding down mode.
*/
#define ROUNDDOWN fesetround(FE_DOWNWARD) 

/** \brief Sets the floating point operations in round near mode.

    Sets the floating point operations in round near mode. This is the default mode.
*/
#define ROUNDNEAR fesetround(FE_TONEAREST)

/************************************************************************************/
/************************************************************************************/

/** 
   \brief Accuracy for the rounding to +/-PI, +/-PI/2.

   Accuracy used when rounding values to +/-PI and +/-PI/2
*/
#define ANGLE_ACCURACY 1e-4

/** 
   \brief Round to +/-PI, +/-PI/2.

   When reading values from equation files, we round them to +/-PI or to +/-PI/2 to gain accuracy.
   The rounding is only applied for variables that are known to be angles.
   
   \param a The angular value to round.
*/
#define ROUND2SYMBOL(a) (fabs((a)+M_PI)<ANGLE_ACCURACY?-M_PI:(fabs((a)-M_PI)<ANGLE_ACCURACY?M_PI:(fabs((a)+M_PI_2)<ANGLE_ACCURACY?-M_PI_2:(fabs((a)-M_PI_2)<ANGLE_ACCURACY?M_PI_2:a))))

/** 
   \brief Prints a number (possibly +/-PI, +/-PI/2, or +/-inf) to a file.

   Prints a number (possibly +/-PI, +/-PI/2, or +/-inf) to a file.
   Numbers close (up to 1e-4 to +/-PI or +/-PI/2 are printed symbolically). This increases accuracy when
   reading these values.

   This type of printing is only used for variables that are known to be angles.
   
   \param f The file.
   \param a The angular value to print.
*/
#define SYMBOL_PRINT(f,a) (fabs((a)+M_PI)<ANGLE_ACCURACY?fprintf(f,"-PI"):(fabs((a)-M_PI)<ANGLE_ACCURACY?fprintf(f,"PI"):(fabs((a)+M_PI_2)<ANGLE_ACCURACY?fprintf(f,"-PI/2"):(fabs((a)-M_PI_2)<ANGLE_ACCURACY?fprintf(f,"PI/2"):INF_PRINT(f,a)))))

/** 
   \brief Prints a number (possibly +/-inf) to a file.

   Prints a number (possibly +/-inf) to a file.
   
   \param f The file.
   \param a The number.
*/
#define INF_PRINT(f,a) (a==INF?fprintf(f,"inf"):(a==-INF?fprintf(f,"-inf"):fprintf(f,"%.12g",a)))

/************************************************************************************/
/************************************************************************************/
/** \brief Allocates memory space.

    Allocates memory space for \e _n elements of type \e _type and assigns the pointer to \e _var.
    If the memory can not be allocated, triggers an error.
*/
#define NEW(_var,_n,_type) {(_var)=(_type *)malloc(sizeof(_type)*(_n)); \
                            if (!(_var)) Error("Memory allocation error");}

/** \brief Allocates and cleans memory space.

    Allocates memory space for \e _n elements of type \e _type and assigns the pointer to \e _var
    and initializes the space to zero.
    If the memory can not be allocated, triggers an error.
*/
#define NEWZ(_var,_n,_type) {(_var)=(_type *)calloc(_n,sizeof(_type)); \
                             if (!(_var)) Error("Memory allocation error");}

/** \brief Expands a previously allocated memory space.

    Expands a previously reserved memory space
    so that it can accomodate \e _n elements of type \e _type  assigns the pointer to \e _var.
    The previously allocated space is pointed by \e _var.
    If the memory can not be expanded, triggers an error.
*/
#define MEM_EXPAND(_var,_n,_type) {(_var)=(_type *)realloc((void *)(_var),sizeof(_type)*(_n)); \
                                   if (!(_var)) Error("Memory reallocation error");}
/** \brief Duplicates a previously allocated memory space.

    Dublicates a previously reserved memory space
    that accomodates \e _n elements of type \e _type.
    After the \e duplication, the pointer \e _var takes to a memory space
    that can accomodate 2*\e _n elements and the value of variable \e _n 
    is also doubled.
*/
#define MEM_DUP(_var,_n,_type) {(_n)*=2;MEM_EXPAND(_var,_n,_type);}

/************************************************************************************/
/************************************************************************************/
/** \brief Prints a variable name into a file.

    Automatically generated variables have a name that starts with a '_'. This is done
    to avoid the generation of a variable with the same name of a variable defined by
    the user (the syntax does not allow for variables starting with a '_').<br>
    When printing the name of the variable into a file we remove the starting '_' to
    allow the input parser to read the output file. If this results in a duplicated
    variable the parser will detect it.
*/
#define PRINT_VARIABLE_NAME(f,name) if (name[0]=='_') fprintf(f,"%s",&(name[1])); else  fprintf(f,"%s",name)

/************************************************************************************/
/************************************************************************************/
/** \brief Used to denote an identifier that has not been initialized.

    Used to denote an identifier that has not been initialized.
*/
#define NO_UINT ((unsigned int)(-1))

/************************************************************************************/
/************************************************************************************/


/** \brief Tangent half-angle subtitution.
    
    Tangent half-angle subtitution

    \param a The angle to convert to the tangent space.
*/
#define A2T(a) (tan((a)/2.0))

/** \brief Converts from paramter to angular space.
    
    Undoes the tangent-half angle substitution to recover the angle.

    \param t The half tangent to convert to angle.
*/
#define T2A(t) (2.0*atan(t))

/** \brief Tangent angular difference.

    Computes the angular difference between two angles from their representation
    in tangent space.

    This is equivalent to: A2T(u)-A2T(l) but with only
    one atan thanks to the use of trigonometric identities

    A2T(u)-A2T(l) = 2*atan(u) - 2*atan(l) = 2*[atan(u)-atan(l)] = 2*atan[(u-l)/(1+u*l)]
*/
#define TAD(u,l) 2*atan2(((u)-(l)),(1+(u)*(l)))

/** \brief Applies the angular offset.

    Adds the offset to an angle.
    
    \param a The angle.
    \param o The offset (in pi/2 units). This works for any offset althogh in general we 
             assume that only 3 offsets are possible: 0, -1->-pi/2 and 1->+pi/2.
*/
#define OA(a,o) ((a)+(o)*M_PI_2)

/** \brief Applies the angular offset to a parameter in half tangent space.

    Adds the offset to an angle expressed in tangent space.

    We have to compute t' from t with the following steps:
       - a=T2A(t)=2*atan(t) --> a/2=atan(t)
       - a'=OA(a,o)=a+o*pi/2 , with o=+/-1
       - t'=A2T(a')=tan(a'/2)
       .
    
    Thus, we have 
       - t'=tan(a'/2)=tan((a+o*pi/2)/2)=tan(a/2+o*pi/4)=tan(atan(t)+o*pi/4)
       .

    We use the identiy tan(a1+a2)=(tan(a1)+tan(a2))/(1-tan(a1)*tan(a2))

    In our case a1=atan(t) and a2=o*pi/4 and, thus, tan(a1)=t and, with o=+/-1, tan(a2)=o

    Thus t'=(t+o)/(1-t*o)
    
    If o=+1 then we have: (t+1)/(1-t)
       With t close to 1 (angle close to pi/2) this give almost infinite.
    
    If o=-1 then we have: (t-1)/(1+t)
       With t close to -1 (angle close to -pi/2) this give almost infinite.

    If o=0 then we have: t
    
    
    \param t The angle in half tangent space.
    \param o The offset. We assume that only 3 offsets are possible: 0, -1->-pi/2 and 1->+pi/2.
*/
// #define OT(t,o) A2T(OA(T2A(t),o))
#define OT(t,o) (((t)+(o))/(1.0-((t)*(o))))

#endif
