#ifndef EQUATIONH
#define EQUATIONH 

#include "boolean.h"
#include "interval.h"
#include "variables.h"
#include "variable_set.h"

#include <stdlib.h>


/** \file equation.h
    \brief Definition of the Tequation type and the associated functions.

    Definition of the Tequation type and the associated functions.

    \sa Tequation, equation.c.
*/

/************************************************************************************/
/** \brief Initial room for terms.

    Initial room for terms in an equation. It will be enlarged if necessary.

    \sa Tequation
*/
#define INIT_NUM_TERMS 100

/************************************************************************************/
/** \brief Initial room for items.

    Initial room for items in each term. It will be enlarged if necessary.

    \sa Tequation
*/
#define INIT_NUM_ITEMS 10

/************************************************************************************/
/************************************************************************************/

/** \brief In a Tequation, the equation relational operator is great equal.

    In a Tequation, the equation relational operator is great equal.
 */
#define GEQ    0

/** \brief In a Tequation, the equation relational operator is less equal.

    In a Tequation, the equation relational operator is less equal.
 */
#define LEQ    1

/** \brief In a Tequation, the equation relational operator is equal.

    In a Tequation, the equation relational operator is equal.
 */
#define EQU    2

/** \brief In a Tequation, the equation relational operator is not defined yet.

    In a Tequation, the equation relational operator is not definet yet.
    This is used for sub-expressions.
 */
#define NOCMP  3

/************************************************************************************/

/** 
   \brief A term: Product of items. 

   Each item in the term can be a variable or a full equation.
*/
typedef struct {
  unsigned int nItems;    /**< Number of variables indexes (or sub-equations) in the set. */
  unsigned int maxItems;  /**< Maximum number of the variables (or sub-equations) in the set. */
  unsigned int *id;       /**< Variable identifier. NO_UINT for sub-expressions (stored in eq). */
  void **eq;              /**< Equations may include other equations. */
} Tterm;

/** 
  \brief An equation.
 
  An equation with linear terms.
*/
typedef struct {
  unsigned int cmp;     /**< Relational operator in the equation LEQ, GEQ, EQU. */
  double value;         /**< Right-hand value of the equation.*/

  unsigned int maxTerms;/**< Maximum number of terms in the equation.*/
  unsigned int nTerms;  /**< Current number of terms in the equation.*/

  double *scale;        /**< Scale factor for each term. */
  Tterm **terms;        /**< Array with pointers to the terms of the equation.*/

  Tvariable_set vars;   /**< Variables involved in the equation terms.*/
} Tequation;

/** 
  \brief Constructor.
 
  Creates an empty equation.

  \param eq The equation to create.
*/
void InitEquation(Tequation *eq);

/** 
  \brief Copy constructor.
 
  Creates one equation copying the information from another equation.

  \param eq_dst The equation to create via copy.
  \param eq_orig The equation from where to copy.
*/
void CopyEquation(Tequation *eq_dst,Tequation *eq_orig);

/** 
  \brief Adds a scaled equation to another equation.
 
  Adds a scaled equation to another equation.

  \param eqn The equation to scale and add.
  \param ct The scale factor.
  \param eq The equation where to add the result.
*/
void AccumulateEquations(Tequation *eqn,double ct,Tequation *eq);

/** 
   \brief Product of two equations.

   Generates an equations that is the product of two given equations.
   
  \param eq1 The first equation to operate.
  \param eq2 The second equation to operate.
  \param eqOut The resulting equation.
*/
void ProductEquations(Tequation *eq1,Tequation *eq2,Tequation *eqOut);

/** 
   \brief Resets the equation information.

   Removes the information stored in the equation, but without releasing
   the allocated memory.

   \param eq The equation.
*/
void ResetEquation(Tequation *eq);

/** 
  \brief Changes the relational operator (LEQ, GEQ, EQU) of the equation.

  Sets a new relational operator for the equation.

  \param cmp The new relational operator.
  \param eq The equation.
*/
void SetEquationCmp(unsigned int cmp,Tequation *eq);

/** 
  \brief Changes the type of the equation (SYSTEM_EQ, CARTESIAN_EQ, DUMMY_EQ, DERIVED_EQ).

  Sets a new type for the equation.

  \param type The new type.
  \param eq The equation.
*/
void SetEquationType(unsigned int type,Tequation *eq);

/** 
  \brief Changes the right-hand value of the equation.

  Sets a new rigth-hand value for the equation.

  \param v The new right-hand value.
  \param eq The equation.
*/
void SetEquationValue(double v,Tequation *eq);

/** 
  \brief Gets the type of relational operator of the equation.

   Gets the type of relational operator of the equation.

  \param eq The equation.

  \return The type of relational operator of the equation.
*/
unsigned int GetEquationCmp(Tequation *eq);

/** 
  \brief Gets the right-hand value of the equation.

  Gets the right-hand value of the equation.

  \param eq The equation.

  \return The right-hand value of the equation.
*/
double GetEquationValue(Tequation *eq);

/** 
  \brief Gets the set of variables  equation used in the equation.

  Gets the set of variables  equation used in the equation.
  Note, we return a pointer to the interval equation structures so be
  careful what you do with the pointer.

  \param eq The equation.

  \return The equation variable set.
*/
Tvariable_set *GetEquationVariables(Tequation *eq);

/** 
  \brief Gets the number of variables  equation used in the equation.

  Gets the number of variables  equation used in the equation.

  \param eq The equation.

  \return The number of variables in the equation.
*/
unsigned int GetEquationNumVariables(Tequation *eq);

/** 
  \brief Initializes an equation from a variable.

  Define an equation with a single term with a single variable.

  \param i Index of the variable.
  \param eq The equation.

  \sa CmpTerm
*/
void EquationFromVariable(unsigned int i,Tequation *eq);

/** 
  \brief Initializes an equation from an equation.

  Define an equation with a single term a single sub-equation.

  \param seq The sub-equation.
  \param eq The equation.

  \sa CmpTerm
*/
void EquationFromSubExpression(Tequation *seq,Tequation *eq);

/** 
  \brief Evaluates an equation in a given point.

  Evaluates an equation in a given point including its right-hand side.
  This is like a combination of \ref EvaluateEquation and \ref GetEquationValue
  into a single function.

  \param varValues Values defining the point where to evaluate the equation. The
                   array must include values for \b all the variables in the system 
		   (from where the function selects the values for the variables 
		   included in the equation).
  \param eq The equation.

  \return The value of the equation including the right-hand side.  
*/
double EvaluateEquation(double *varValues,Tequation *eq);

/** 
  \brief Prints an equation.

  Writes an equation to a given stream.

  \param f The stream where to print the equation.
  \param varNames The name of the variables.
	          If this parameter is NULL the printed
	          names for the variables are <em>v_X</em> with \e X the index
	          of the variable.
  \param eq The equation to print.

  \sa PrintTerm PrintVarSet
*/
void PrintEquation(FILE *f,char **varNames,Tequation *eq);

/** 
  \brief Destructor.

  Deletes the information stored in the equation and frees the allocated memory space.
  
  \param eq The equation to delete.
*/
void DeleteEquation(Tequation *eq);

#endif
