#ifndef EQUATIONSH
#define EQUATIONSH

#include "boolean.h"
#include "variables.h"
#include "equation.h"
#include "defines.h"

#include <stdio.h>

/** \file equations.h
    \brief Definition of the Tequations type and the associated functions.

    Definition of the Tequations type and the associated functions.

    \sa Tequations, equations.c.
*/
/************************************************************************************/

/************************************************************************************/

/** \brief Initial room for equations.

    Initial room for equations in an equation set. It will be enlarged if necessary.

    \sa Tequations
*/
#define INIT_NUM_EQUATIONS 10
/************************************************************************************/

/** 
   \brief Set of equations.

   Set of equations including scalar and matrix equations.

   Note that the matrix equations are added recently so in most of the comments in the code
   "eauations" is used  instead of "scalar equations"

   The kino-equations are equations on the velocity. Thus, they do not play any relevant role
   in the normal evaluation of the equations, but they are used when computing the Jacobian
   of an equation set (they are directly transferred to the Jacobian),

   \see Tequation, TequationInfo, Tvector 
 */
typedef struct {
  unsigned int neq;         /**< Current number of equations. */

  unsigned int m;           /**< Max number of equations. */
  Tequation **equation;      /**< The original equation*/
  
} Tequations;

/** 
  \brief Constructor.
 
  Initializes a set of equations.

  \param eqs The equation set to be initialized.
*/
void InitEquations(Tequations *eqs);

/** 
  \brief Copy constructor.
 
  Initializes a set of equations from another set.

  \param eqs_dst The equation set to be initialized.
  \param eqs_src The equation set from where to get the information.
*/
void CopyEquations(Tequations *eqs_dst,Tequations *eqs_src);

/** 
  \brief Number of equations in the set.

  Number of equations in the set.

  \param eqs The equation set.

  \return The number of equations in the set.
*/
unsigned int NEquations(Tequations *eqs);

/** 
  \brief Adds an equation to the set.

  Adds an equation to the set and computes its associated information.
  Empty equations (equations with no monomials) or repeated equations
  are not added to the set.
  Moreover, equations are sorted from  simplest to more complex.

  \param equation The equation to be added.
  \param eqs The equation set.

  \sa TequationInfo.
*/
void AddEquation(Tequation *equation,Tequations *eqs);

/** 
  \brief Gets an equation from the set.

  Returns a pointer to the \e n-th equation stored in the set.

  \param n The number of equation to get.
  \param eqs The equation query.
*/
Tequation *GetEquation(unsigned int n,Tequations *eqs);

/**  
   \brief Evaluates all equality equations in the set.

   Evaluates all equality equations in the set. This includes the equality
   scalar equations and all matrix equations.

   \param v The value for the variables.
   \param r the array where to store the result.
   \param eqs The equations to evaluate.
*/
void EvaluateEqualityEquations(double *v,double *r,Tequations *eqs);

/** 
   \brief Prints a set of equations.

   Writes a set of equations into a stream.

   \param f The stream where to write the equations.
   \param varNames The name of the variables.
	          If this parameter is NULL the printed
	          names for the variables are <em>v_X</em> with \e X the index
	          of the variable.
   \param eqs The equation set to print.

*/
void PrintEquations(FILE *f,char **varNames,Tequations *eqs);

/** 
  \brief Destructor
  
  Deletes the information stored in the set of equations and 
  frees the allocated memory space.
  
  \param eqs The set of equations to delete.
 */
void DeleteEquations(Tequations *eqs);

#endif
