#ifndef FILENAMEH
#define FILENAMEH

/** 
   \file filename.h

   \brief  Definition of the Tfilename type and the associated functions.

   Functions to deal with file names.

   The main files used in the CuikSuite have the following extensions:

    
      - <b>.cuik</b> Files including variables and equations. The solution set
        of a cuik file can be obtained using \ref cuik.c "cuik". This will produce
	a .sol file. 
      - <b>.param</b> File with the parameters. The default set of parameters
        is in share/default.param
      - <b>.sol</b> Solution file resulting from \ref cuik.c "cuik". 
      .

   \sa filename.c Tfilename 
*/

/************************************************************************************/
/** \brief File extension for loops files.

    File extension for loops files.
*/
#define LOOPS_EXT              "loops"

/** \brief File extension for equation files.

    File extension for equation files.
*/
#define CUIK_EXT              "cuik"

/** \brief File extension for parameter files.

    File extension for parameter files. 
*/
#define PARAM_EXT             "param"

/** \brief File extension for solution files.

    File extension for solution files. 
*/
#define SOL_EXT               "sol"

/** \brief File extension for 2D plot files. 

    File extension for 2D plot files. 
*/
#define PLOT2D_EXT            "fig"

/** \brief File extension for multi-affine equations. 

    File extension files summaring problems involving multi-affine
    equations. Such files include the number of equations and variables,
    the variables in each equation, and the evaluation of the
    equations on a unit box.
*/
#define MAF_EXT            "maf"


/************************************************************************************/

/** 
   \brief Data structure to hold the information about the name of a file.

   Data structure to hold the information about the name of a file.
   
   \sa filename.h filename.c
 */
typedef struct {
  char *path;      /**< The path of the file. */
  char *name;      /**< The name of the file. */
  char *ext;       /**< The extension of the file. */
  char *baseName;  /**< The path plus the name. */
  char *fullName;  /**< The path plus the name plus the extension. */
} Tfilename;

/** 
   \brief Constructor

   Generates a Tfilename structure.

   \param path The path. If not given we get it from the system and is
               the path from of the directory from where the application is exeucted.
   \param name The name of the file. If the given name includes part of a path, it is
               removed from the name and added to the path data field.
   \param suffix A suffix to add to the name. Possibly NULL.
   \param ext The extension for the file.
   \param fn The filename structure to create.
*/
void CreateFileName(char *path,char *name,char *suffix,char *ext,Tfilename *fn);

/** 
   \brief Gets the file full name (paht+name+extension).

   Gets the file full name (path+name+extension).

   \param fn The filename structure.

   \return Returns the full name for the file.
*/
char *GetFileFullName(Tfilename *fn);

/** 
   \brief Gets the file path.

   Gets the file path.

   \param fn The filename structure.

   \return Returns the path for the file.
*/
char *GetFilePath(Tfilename *fn);

/** 
   \brief Gets the file base name (paht+name) .

   Gets the file base name (paht+name).

   \param fn The filename structure.

   \return Returns the base name (path+name) for the file.
*/
char *GetFileBaseName(Tfilename *fn);

/** 
   \brief Gets the file name.

   Gets the file name.

   \param fn The filename structure.

   \return Returns the name for the file.
*/
char *GetFileName(Tfilename *fn);

/** 
   \brief Gets the file extension.

   Gets the file extension.

   \param fn The filename structure.

   \return Returns the extension for the file.
*/
char *GetFileExtension(Tfilename *fn);

/** 
    \brief Creates a link to a given filename.

    Creates a link to a given filename with a given extension.

    \param fn1 File to link.
    \param ext Extension of the files.
    \param fn2 File name of the link.
*/
void LinkFileNameWithExtension(char *fn1,char *ext,Tfilename *fn2);


/** 
   \brief Destructor.

   Deleltes the Tfilename structure.

   \param fn The filename structure to delete.
*/
void DeleteFileName(Tfilename *fn);

#endif
