#ifndef LOOPSH
#define LOOPSH 

#include "transforms.h"


/** \file loops.h
    \brief Definition of the Tloops type and the associated functions.

    Definition of the Tloops type and the associated functions.

    \sa Tloops, loops.c.
*/

/** \brief Initial room for loops in a sequence.

    Initial room for loops in a sequence. It will be enlarged if necessary.

    \sa Tequations
*/
#define INIT_NUM_LOOPS 12

/** \brief Maximum number of equations per loop.
    
     Maximum number of equations per loop. We typically define 6 equations for
     non-simplified loops and 3 or 4 for simplified ones. However, if necessary
     we can generate up to 8 equations.

     This value is only used to allocate space for equation evaluation.
*/
#define MAX_EQS_LOOP 8


/** 
  \brief A sequence of loops.
 
  A sequence of spatial transform.
*/
typedef struct {
  Ttransforms **ts;    /**< The transform sequences defining each loop. */

  unsigned int neq;   /**< Number of equations. */
  Tvariable_set vars; /**< Variables used in the loops. */
  
  unsigned int n;     /**< Current number of loops. */
  unsigned int m;     /**< Maximum number of loops. */
} Tloops;

/** 
   \brief Constructor.

   Initializes the set of loops.

   \param lps The set of loops to initialize.
*/
void InitLoops(Tloops *lps);

/** 
   \brief Copy constructor.

   Copies a set of loop equations.

   \param lps_dst The loops where to copy.
   \param lps_src The loops rom where to copy.

 */
void CopyLoops(Tloops *lps_dst,Tloops *lps_src);

/** 
   \brief Adds a loops to a set.

    Adds a loops to a set.
    
    \param ts The transform sequence defining the loop.
    \param lps The set of loops.
*/
void AddLoop(Ttransforms *ts,Tloops *lps);

/** 
   \brief Number of loops.

   Number of loops.

   \param lps The set of loops to query.

   \return The number of loops in the set.
*/
unsigned int NLoops(Tloops *lps);

/** 
   \brief Variables used in the loop.

   Returns the set of variables used in the loops.

   \param lps The set of loops to query.

   \return The variables used in the loops.
 */
Tvariable_set *GetVariablesInLoops(Tloops *lps);

/** 
   \brief Number of equations in the set.

   Number of equations in the set.

   \param lps The set of loops to query.

   \return The number of equations in the set.
*/
unsigned int NEquationsInLoops(Tloops *lps);

/** 
   \brief Number of variables in the set.

   Number of variables in the set.

   \param lps The set of loops to query.

   \return The number of variables in the set.
*/
unsigned int NVariablesInLoops(Tloops *lps);

/** 
   \brief Gets a loop.

   Extracts a loop from a set.

   \param nl Loop identifier.
   \param lps The set of loops.
   
   \return The transforms sequence defining the loop.
 */
Ttransforms *GetLoop(unsigned int nl,Tloops *lps);

/** 
   \brief Evaluates the equations in the loops.

   Evaluates the equations in the loops at a given point.

   \param val The evaluation point. 
   \param offs The offsets for each variables.
   \param l The space for the evaluation. Must have \ref NEquationsInLoops entries.
   \param lps The loops
 */
void EvaluateLoops(double *val,signed int *offs,double *l,Tloops *lps);


/** 
   \brief Simplify the loop equations.

   Simplify the loop equations.
   
   Up to now, we only eliminate the last two variables in each loop equation
   but more complex simplifications are possible: eliminate 3 variables per 
   loop or generate different variants of each loop equation with different
   variable eliminations.

   \param simplify Simplification level. Number of variables to eliminate.
   \param offs The offsets defining the sub-problem of intersets. NULL 
               if no offset is required.
   \param lps_simp The simplified loops.
   \param lps The loop equations to simplify.
 */
void SimplifyLoops(unsigned int simplify,
		   signed int *offs,Tloops *lps_simp,Tloops *lps);
 
/** 
   \brief Update the simplification.

   Updates a simplification for a new set of offsets.

   \param offs The offsets defining the sub-problem of intersets. NULL 
               if no offset is required.
   \param lps_simp The simplified loops.
   \param lps The loop equations to simplify.
*/
void ReSimplifyLoops(signed int *offs,Tloops *lps_simp,Tloops *lps);

/** 
   \brief Prints the loops.

   Prints the loops.

   \param f The file where to print.
   \param varnames The name of the variables.
   \param lps The loops to print.
 */
void PrintLoops(FILE *f,char **varnames,Tloops *lps);

/** 
   \brief Prints the loops.

   Prints the loops in the form of equations.

   \param f The file where to print.
   \param varnames The name of the variables.
   \param lps The loops to print.
 */
void PrintLoopsEquations(FILE *f,char **varnames,Tloops *lps);

/** 
   \brief Prints the loops.

   Prints the loops in the form of C*T where C is a constant
   matrix and T is a vector of monomials.

   \param f The file where to print.
   \param varnames The name of the variables.
   \param lps The loops to print.
*/
void PrintLoopsMatrixVector(FILE *f,char **varnames,Tloops *lps);

/** 
   \brief Destructor.

   Releases the memory in a set of loops.

   \param lps The set of loops to destroy.
*/
void DeleteLoops(Tloops *lps);

#endif
