#ifndef PARAMETERSH
#define PARAMETERSH 

#include "boolean.h"

#include <stdio.h>

/** 
   \file parameters.h

   \brief  Definition of the Tparameters type and the associated functions. 

   Definition of the Tparameters type and the associated functions. 

   \sa Tparameters, parameters.c, readparameters.l readparameters.y.
*/


/** \brief One of the two possible reduction procedures.

    Reduction using the trapezoid method.
*/ 
#define RED_TRAPEZOID 0

/** \brief One of the two possible reduction procedures.

    Reduction using the simplex method.
*/ 
#define RED_SIMPLEX   1


/** 
   \brief No clean.

   Solutions obtained with the trapezoid method are not clean.
*/
#define NO_CLEAN      0

/** 
   \brief Clean with the simplex method.

   Solutions obtained with the trapezoid method are clean with the
   simplex method.
*/
#define CLEAN_SIMPLEX 1


/** 
   \brief Verify only with Miranda's theorem.

   Solutions obtained with the trapezoid method are verified with the
   Miranda's theorem. 

   This only works for 0-dimensional solution sets (points)
*/
#define VERIFY_MIRANDA 0

/** 
   \brief Verify with the newton method.

   Solutions obtained with the trapezoid method are verified with the
   Newton method. 
*/
#define VERIFY_NEWTON  1

/** 
   \brief Verify with the simplex method with replacemnet.

   This is the same as \ref VERIFY_NEWTON but the boxes containing a solution
   are replaced by tiny boxes around this solution. This should only be used
   if the problem has isolated solutions. For problems with continuous
   solutions sets this will break the full coverage of the solution set.
*/
#define VERIFY_NEWTON_REPLACE  2

/************************************************************************************/
/** \brief Default file of parameters 

    Default file of parameters. This is the file of parameters read in the first
    place, before reading the file specific for the problem at hand.
    In this way, the problem specific parameter need only to include values
    for the parameters whose value is not to be the default value.
*/
#define DEFAULT_PARAMS "share/CuikTrapezoid/default.param"
/************************************************************************************/

/** 
   \brief Numerical tolerance.
    
   Threshold to turn any number into 0 before passing it to the simplex.
   This should be larger than 1e-6, the threshold hard-coded in many simplex packages.
*/
#define CT_EPSILON                   0

/** 
   \brief Reduction threshold.
    
   If boxes reduce more that \e RHO the reduction process is repeated.
*/
#define CT_RHO                       1

/** 
   \brief Box size threshold.
    
   Size of the boxes bounding continuous solution sets.
*/
#define CT_SIGMA                     2

/** 
   \brief Reduction procedure.
    
   Reduction procedure: trapezoid or simplex
*/
#define CT_REDUCTION                  3

/** 
   \brief Clean solutions.
    
   If set, the solutions returned with the trapezoid method
   are filtered with the simplex method.
*/
#define CT_CLEAN                      4

/** 
   \brief Verify solutions.
    
   If set, the solutions returned with the trapezoid method
   are verified with an additional method (Newton, etc).
*/
#define CT_VERIFY                     5

/** 
   \brief Simplify equations.
    
   If set, the the number of variables in each
   equation is reduced. This speed ups the
   solution process, but it typically causes
   more clustering.

   We may need to introduce redundant equations
   to compensate.
*/
#define CT_SIMPLIFY                   6

/** 
   \brief Rotate loops.
    
   If set, we repeat each loop as many
   times as variables involved in it and
   with each repetition starting by a different
   variable.
*/
#define CT_ROTATE_LOOPS               7

/** 
   \brief Save in angular space.
    
   If set, the solutions are saved in angular space (not in
   tangent space). 
*/
#define CT_SAVE_ANGLES                8

/** 
   \brief Offset.
    
   Offset when defining the two halves of a revolute range.
   We typically split the range for revolute joints in two halves [0,pi] [-pi,0]
   In tangent space this becomes [-1,1] [-1,1] when the ranges are defined from
   pi/2, -pi/2.

   With this parameter we can change these halves to [offset,pi+offset] [-pi+offset,0-offset]
   This is used to capture solutions at 0/pi only once (not once for each half).
   This value should be small (if different from 0).
*/
#define CT_OFFSET                      9

/** 
   \brief  Total number of parameters.

   Total number of parameters.
*/
#define NPARAMETERS (CT_OFFSET+1)

/** 
   \brief Information about a single parameter.
   
   Information about a single parameter (name and value).
*/
typedef struct
 {
   char *name;            /**< Name of the parameter. */
   double value;          /**< Value of the parameter. */
   unsigned int timesSet; /**< Times this parameter has been set. */
} Tparameter;

/** 
   \struct Tparameters 
   \brief A table of parameters.

   A set of parameters. Each parameter is identified with a constant
   defined in file parameters.h (check the source code).

   \sa parameters.h, parameters.c
*/

/** 
   \brief A table of parameters.

   A set of parameters. Each parameter is identified with a constant
   defined in file parameters.h.

   This is basically a set of constants, but only a small, predefined
   set of constants are allowed. This is why we do not re-use
   \ref Tconstants but provide a different implementation.
   
   \sa parameters.h, parameters.c
*/
typedef Tparameter Tparameters[NPARAMETERS];

/** 
   \brief Constructor.

   Defines an empty set of parameters.

   \param p The set to be initialized.
*/
void InitParameters(Tparameters *p);

/** 
   \brief Constructor from a file.

   Defines an set of parameters from a file. It first read the default
   set of parameters and then the given set of parameters.

   \param file Name of the file with the set of parameters particular
               for the problem at hand.
   \param p Set of parameters to define.

   \sa ReadParameters, defines.h
*/
void InitParametersFromFile(char *file,Tparameters *p);

/** 
   \brief Gets the value for a particular parameter.
   
   Gets the value for a particular parameter.

   \param n Number of parameter.
   \param p Set of parameters.

   \return The value of the parameter. If there is no parameter with the
           given number this functions triggers an error.
*/
double GetParameter(unsigned int n,Tparameters *p);

/** 
  \brief Returns the parameter identifier given the paramete name.

  Determines the parameter identifier for a given parameter name.

  \param name The paremeter name.
  \param p The set of parameteres.

  \return The paremeter identifier or NO_UINT if there is not a parameter with the given name.
*/
unsigned int GetParameterID(char *name,Tparameters *p);

/** 
  \brief Checks if a parameter is already defined.

  Determines is a parameter is already defined. The default initialization (the one from
  the default set of parameters) is not taken into account. Only explicit initializations
  from the users are considered.
  
  Only parameters alrady defined can be used in the definition of other parameters.

  \param n The identifier of the parameter.
  \param p The parameter set.
  
  \return TRUE if the parameter is already defined.
*/
boolean ParameterSet(unsigned int n,Tparameters *p);

/** 
   \brief Sets the name and value for a particular parameter.
   
   Sets the name and value for a particular parameter.

   \param n Number of parameter.
   \param name The name for the parameter. This is a string that is used
               when printing the parameter.
   \param v The value.
   \param p Set of parameters.
*/
void SetParameter(unsigned int n,char *name,double v,Tparameters *p);

/** 
   \brief Sets the value for a particular parameter.
   
   Sets the value for a particular parameter.

   \param n Number of parameter.
   \param v The new value for the parameter.
   \param p Set of parameters.
*/
void ChangeParameter(unsigned int n,double v,Tparameters *p);

/** 
   \brief Prints a parameter set.
   
   Writes a parameter set to a stream, that can be \e stdout.

   In  principle the output of this function can be used as a parameter file
   (i.e., it can be parsed correctly with \ref InitParametersFromFile).

   \param f The stream
   \param p Set of parameters to print. 
*/
void PrintParameters(FILE *f,Tparameters *p);

/** 
   \brief Destructor.
   
   Deletes a set of parameters and frees the allocated memory.

   \param p Set of parameters to be deleted. 
*/
void DeleteParameters(Tparameters *p);

#endif
