#ifndef TRANSFORMH
#define TRANSFORMH 

#include "boolean.h"

#include <stdio.h>
#include <string.h>

/** \file transform.h
    \brief Definition of the Ttransform type and the associated functions.

    Definition of the Ttransform type and the associated functions.

    Note that we internally represent the transforms with a non-normalized dual
    quaternion. Thus problems will arise for rotations of pi radiants. 

    \sa Ttransform, transform.c.
*/

/************************************************************************************/
/** \brief A constant transform.

    One type of transform.

    \sa Ttransform
*/
#define TR_CT 0

/** \brief Rotation in X.

    One type of transform.

    \sa Ttransform
*/
#define TR_RX 1

/** \brief Rotation in Y.

    One type of transform.

    \sa Ttransform
*/
#define TR_RY 2

/** \brief Rotation in Z.

    One type of transform.

    \sa Ttransform
*/
#define TR_RZ 3

/** \brief Translation in X.

    One type of transform.

    \sa Ttransform
*/
#define TR_TX 4

/** \brief Translation in Y.

    One type of transform.

    \sa Ttransform
*/
#define TR_TY 5

/** \brief Translation in Z.

    One type of transform.

    \sa Ttransform
*/
#define TR_TZ 6

/** \brief DH rotation transform.

    One type of transform.

    \sa Ttransform
*/
#define TR_DHR 7

/************************************************************************************/


/** 
  \brief A transform.
 
  A spatial transform.
*/
typedef struct {
  double q[8];          /**< Constant transform in dual quaternion form. */

  unsigned int type;    /**< Type of transform. Only used for variable transforms. */
  unsigned int var;     /**< Variable in the transform. NO_ID if the transform is constant. */
  boolean inv;          /**< TRUE if the variable transform is inverted. */
} Ttransform;

/** 
  \brief Constructor.
 
  Creates an identity transform.

  \param tr The transform to create.
*/
void NewIDTransform(Ttransform *tr);


/** 
   \brief Copy constructor.

   Copies a transform.
   
   \param tr_dst The transform to create.
   \param tr_src The source transform.
*/
void CopyTransform(Ttransform *tr_dst,Ttransform *tr_src);

/** 
   \brief Determines if the transform is variable.

   Identifies variable transforms.
   
   \param tr The transform to check
   
   \return TRUE if the transform is variable.
 */
boolean VariableTransform(Ttransform *tr);


/** 
   \brief Determines if the transform is constant.

   Identifies constant transforms.
   
   \param tr The transform to check
   
   \return TRUE if the transform is constant.
 */
boolean CtTransform(Ttransform *tr);

/** 
   \brief Identifies identity transforms.

   Identifies constant identity transforms.

   \param tr The transform to check.

   \return TRUE if it is the identity (no rotation, no translation).
 */
boolean IsIdentity(Ttransform *tr);


/** 
    \brief Returns the variable in the transform.

     Returns the variable in the transform, if any.

     \param tr The transform to check.

     \return The variable in the transform. NO_UINT if 
             the transform is constant.
 */
unsigned int GetTransformVariable(Ttransform *tr);

/** 
  \brief Inversion.
 
  Inverts a transform.

  \param tr The transform to invert.
*/
void InvertTransform(Ttransform *tr);

/** 
  \brief Constructor.
 
  Creates a transform.

  \param tt Type of transform: RX, RY, RZ, TX, TY, TZ
  \param value Value of the transform.
  \param inv TRUE if the transform is inverted
  \param tr The transform to create.
*/
void NewCtTransform(unsigned int tt,double value,boolean inv,Ttransform *tr);

/** 
  \brief Constructor.
 
  Creates a transform.

  \param tt Type of transform: RX, RY, RZ, TX, TY, TZ
  \param id Variable in the transform.
  \param inv TRUE if the transform is inverted
  \param tr The transform to create.
*/
void NewVarTransform(unsigned int tt,unsigned int id,boolean inv,Ttransform *tr);

/** 
  \brief Constructor.
 
  Creates a rotation DH transform (a rotation about Z)

  \param d Constant 'd' DH parameter.
  \param theta Variable representing the theta DH parameter.
  \param a Constant 'a' DH parameter.
  \param alpha Constant 'alpha' DH parameter.
  \param inv TRUE if the transform is inverted
  \param tr The transform to create.
*/
void NewDHRTransform(double d,unsigned int theta,double a,double alpha,boolean inv,Ttransform *tr);

/** 
   \brief Defines a general transform from a transform matrix.

   Defines a general constant transform from the parameters in an 4x4 homogenous matrix.
   The last row of the matrix is assumed to be[0 0 0 1].
   No check is performed to verify whether the matrix is correct (orthonormal etc.)!!

   \param m11 Entry 1,1 of the 4x4 homogeneous matrix.
   \param m12 Entry 1,2 of the 4x4 homogeneous matrix.
   \param m13 Entry 1,3 of the 4x4 homogeneous matrix.
   \param m14 Entry 1,4 of the 4x4 homogeneous matrix.
   \param m21 Entry 2,1 of the 4x4 homogeneous matrix.
   \param m22 Entry 2,2 of the 4x4 homogeneous matrix.
   \param m23 Entry 2,3 of the 4x4 homogeneous matrix.
   \param m24 Entry 2,4 of the 4x4 homogeneous matrix.
   \param m31 Entry 3,1 of the 4x4 homogeneous matrix.
   \param m32 Entry 3,2 of the 4x4 homogeneous matrix.
   \param m33 Entry 3,3 of the 4x4 homogeneous matrix.
   \param m34 Entry 3,4 of the 4x4 homogeneous matrix.
   \param inv TRUE if the transform must be inverted.
   \param tr The transform to create.
*/
void NewTransformFromMatrix(double m11,double m12,double m13,double m14,
			    double m21,double m22,double m23,double m24,
			    double m31,double m32,double m33,double m34,boolean inv,Ttransform *tr);

/** 
   \brief Defines a general transform from a dual quaternion.

   Defines a general constant transform from a dual quaternion. The
   quaternion is supposed to be correctly defined (no check is performed).

   \param q1 First parameter of the dual quaternion.
   \param q2 Second parameter of the dual quaternion.
   \param q3 Third parameter of the dual quaternion.
   \param q4 Fourth parameter of the dual quaternion.
   \param q5 Fifth parameter of the dual quaternion.
   \param q6 Sixth parameter of the dual quaternion.
   \param q7 Seventh parameter of the dual quaternion.
   \param q8 Eighth parameter of the dual quaternion.
   \param inv TRUE if we have to invert the transform given by q1,...,q8.
   \param tr The transform to create.
*/
void NewTransformFromDualQuat(double q1,double q2,double q3,double q4,
			      double q5,double q6,double q7,double q8,boolean inv,Ttransform *tr);

/** 
   \brief Dual quaternion representing a transform.

   Generates the dual quaternion representing a transform.

   \param val Values for the variables in the problem.
   \param offs The offset for each variable, if any.
   \param q The output dual quaternion (8 doubles).
   \param tr The transform.
*/
void Transform2DualQuad(double *val,signed int *offs,double *q,Ttransform *tr); 

/** 
   \brief Prints a transform.

   Prints a transform.
   
   \param f File where to print the transform.
   \param varname The names of the variables.
   \param tr The transform to print.
 */
void PrintTransform(FILE *f,char **varname,Ttransform *tr);

/** 
  \brief Destructor.

  Deletes the information stored in the transform and frees the allocated memory space.
  
  \param tr The transform to delete.
*/
void DeleteTransform(Ttransform *tr);

#endif
