#ifndef TRANSFORMSH
#define TRANSFORMSH 

#include "transform.h"
#include "variable_set.h"
#include "boolean.h"
#include "defines.h"
#include "basic_algebra.h"

/** \file transforms.h
    \brief Definition of the Ttransforms type and the associated functions.

    Definition of the Ttransforms type and the associated functions.

    \sa Ttransforms, transforms.c.
*/


/** \brief Initial room for transforms in a sequence.

    Initial room for transforms in a sequence. It will be enlarged if necessary.

    \sa Tequations
*/
#define INIT_NUM_TRANSFORMS 12

/** 
   \brief Compact transform sequence operation constant transforms.

   If set, we compact transform sequence operation constant consecutive transforms.
*/
#define COMPACT_TRANSFORMS 1

/** 
   \brief Use redundant equations per loop.

   If set, we generate 7 equations per loop, i.e., we also use the first
   element of the dual part (it is also multilineal and equal to 0, but
   redundant).

   \todo Define parameter for the redundancy conditions.
*/
#define REDUNDANT_EQ_IN_LOOP 0

/** 
   \brief Use redundant equations in simplification.

   If set, the simplification includes redundant equations.

   This only applies to the case where we remove 3 variables
   from each loop. 
*/
#define REDUNDANT_SIMP 1

/** 
  \brief A sequence of transforms.
 
  A sequence of spatial transform.

  We can have normal or simplified sequences. In the later we remove
  some of the variables, expressing them in terms of the remaining ones.

  Once simplified, the transform is expressed in matrix form: a matrix
  of coefficients which multiplied by a vector of monomials give the
  equation values. The matrix form is necessary for the variable elimination.
  
  See \ref SimplifyTransforms for details on the simplification process

  Caution: The simplification can not be undo.

*/
typedef struct {
  
  Tvariable_set vars;   /**< The variables in the sequence. */
  Ttransform **tr;      /**< The transforms in the sequence. */

  unsigned int n;       /**< Current number of transforms in the sequence. */
  unsigned int m;       /**< Maximum number of transforms in the sequence. */

  unsigned int neq;     /**< Number of equations defined by the sequence. */
  unsigned int ndx[8];  /**< Indices of the dual quaternion vector defining
			     the equations. At most 8. */

  unsigned int simp;    /**< Simplification level for this transform (0,2,3). */
  Tvariable_set rvars;  /**< The set of removed variables. */
  unsigned int nr;      /**< Number of rows of the matrix of coefficients. */
  unsigned int nc;      /**< Number of columns of the matrix of coefficients. */
  
  double *M;            /**< Matrix of coefficients. Defines the equations such as
			        M * T_k = 0 
			     with T_k the monomials of the kept variables. */
  double *x;            /**< Space for the vector of monomials of the kept variables. 
			     Used when evaluation equations in matrix form, i.e. when 
			     computing M*x. */
  double *S;            /**< Simplification matrix, i.e., a matrix such that
			        T_r = S * T_k
			     with T_r the monomials of the removed variables
			     and T_k the monomials of the kepth variables.
			     This can be NULL for some simplified sequences
			     (in some cases it is not possible to define this
			     one-to-one relation). */
  void *ts1;             /**< LHS transform sequence in the simplification. */
  TQR qr;                /**< QR decomposition used in the simplification.
			      Stored to speed up re-simplifications. */
  
} Ttransforms;

/** 
   \brief Initializes a sequence of transforms.
   
   Initializes a sequence of transforms.

   \param ts The sequence to initialize.
*/
void InitTransforms(Ttransforms *ts);

/** 
   \brief Copies a sequence of transforms.

   Copy of a sequence of transforms.

   \param ts_dst The transform sequence to create.
   \param ts_src The transform sequence from where to copy.
*/
void CopyTransforms(Ttransforms *ts_dst,Ttransforms *ts_src);

/** 
   \brief The set of variables in a transforms sequence.

   The set of variables in a transforms sequence.

   \return The set of variables in a transforms sequence.
 */
Tvariable_set *GetTransformsVariables(Ttransforms *ts);

/** 
   \brief Variables eliminated in the simplification.

   Returns the set of variables eliminated in the simplification.

   \param ts The transforms sequence to query.

   \return The removed variables.
 */
Tvariable_set *GetTransformsRemovedVariables(Ttransforms *ts);

/** 
   \brief Identifies simplified transfom sequences.

   Identifies simplified transfom sequences.

   \param ts The transforms sequence to query.

   \return TRUE if the transforms sequence is simplifed.
*/
boolean SimplifiedTransforms(Ttransforms *ts);

/** 
   \brief Number of equations defined by the transform sequence.

   Number of equations defined by the transform sequence. Typically 6 but
   they can be less for simplified sequences.

   \param ts The transform sequence to query.

   \return  Number of equations defined by the transform sequence.
*/
unsigned int NTransformsEquations(Ttransforms *ts);

/** 
   \brief Number of transforms in a transform sequence.

   Number of transforms in the sequence. 

   Notice that this remains the same, even after simplification.

   \param ts The transform sequence to query.

   \return  Number of transforms.
*/
unsigned int NTransforms(Ttransforms *ts);

/** 
   \brief Number of variables used in the transform sequence.

   Number of variables used in the transform sequence.

   \param ts The transform sequence to query.

   \return  Number of variables used in the transform sequence.
*/
unsigned int NTransformsVariables(Ttransforms *ts);

/** 
   \brief Rotates a transform sequence.

   Assuming that a transform sequence T_1*T_2*...*T_n is equal to the
   identity (i.e., it represents a closed loop), the same
   loop can be represented as T_i*...*T_n*T_1*...T_(i-1).
   Different loop "rotations" lead to different variable eliminations.
   
   Since rotations are related to variable eliminations, it makes sense
   to define the rotations variable-wise, but in for experimental
   purposes we also define the cut transform-wise (i.e., after the
   rotation the variables in the transform may be the same as before 
   since only constant transforms may be rotated.

   \param nv When operating variable-wise this indicates 
             the variable where to cut the loop (indicates the
	     variable to move to the end of the loop). 
	     When operating transform-wise, this indicates
	     the transform where to cut (indicates the last
	     transform from the start one to move to the end).
   \param var TRUE if the rotation has to be done variable-wise.
              If FALSE, we operate transform-wise
   \param ts The transform to rotate. It will be modified!
 */
void RotateTransforms(unsigned int nv,boolean var,Ttransforms *ts); 

/** 
   \brief Splits a transform sequence in two.

    Splits a transform sequence in two.

    If we have a loop T_1*...*T_n and assuming that it is
    equal to the identity (a closed loop) we can move part
    of the loop the right-hand side of the equation

       T_1*...*T_k-1 = T_n^-1....T_k^-1

    This function computes the left-hand side (in ts1) and
    the right-hand side (in ts2).

    \param nv Number of variables to keep in the left (in ts1). 
    \param ts1 The first sequence. 
    \param ts2 The second sequence. 
    \param ts The original sequence.

    \return The cut point. The transform below the returned index form ts1.
 */
unsigned int CutTransforms(unsigned int nv,Ttransforms *ts1,Ttransforms *ts2,Ttransforms *ts);

/** 
   \brief Pre-multiplies a sequence of transforms with a transform.

   Pre-multiplies a sequence of transforms with a transform.

   \param tr The transform.
   \param ts1 The sequence of transforms to modify.
   \param ts The resulting sequence.
*/
void PreMultiplyTransforms(Ttransform *tr,Ttransforms *ts1,Ttransforms *ts);

/** 
   \brief Removes the first transform in a sequence.

   Removes the first transform in a sequence. This is typically used to
   remove constant transforms and move them to the other side of an
   equation.
   
   \param ts The sequence to reduce.  
 */
void RemoveFirstTransform(Ttransforms *ts);

/** 
  \brief Post-multiply a sequence of transforms with another sequence.

  Post-multiply a sequence of transforms with another sequence.
  
  \param ts1 The sequence to add.
  \param inv Whether we have to invert tr1 before adding it.
  \param compact It true, and the first and last transforms in the resulting
                 transform sequence are constant, they are operated.
  \param ts The transform sequence to extend.
*/
void PostMultiplyTransforms(Ttransforms *ts1,boolean inv,boolean compact,Ttransforms *ts);

/** 
   \brief Evaluates a sequence of transforms.

   Evaluates a sequence of transforms.

   \param val The evaluation point.
   \param offs The offset of each variable.
   \param q The result of the evaluation.
   \param ts The sequence of transforms to evaluate.

   \return The number of equations. The same as \ref NTransformsEquations
           This is the size required for q.
*/
unsigned int EvaluateTransforms(double *val,signed int *offs,double *q,Ttransforms *ts);

/** 
   \brief Simplify transforms based on QR decomposition.

   Simplify transforms based on QR decomposition.

   This simplification works as follows. Given a loop
   
       T(t1)*T(t2)*T(t3)*T(t4)*T(t5)*T(t6) = H

    we can write

       T(t1)*T(t2)*T(t3)*T(t4) = H*T(t6)^-1*T(t5)^-1

   which can be expressed as

       A * X1 = B * X2

   where A and B are constant coefficient matrices and X1 and X2 are
   vectors of monomials in t1,...,t4 and t5,t6, respectively. 

   Matrix A is 8 x 16 (we use 8 equations in the simplification).
   Matrix B is 8 x 4

   Matrix B can be decomposed as

       B = Q * R * P

   With Q a 8x8 orthonormal matrix, R a 8x4 diagonal matrix and P a 
   permutation matrix. Matrix R can be decomposed in two blocks, 
   a 4x4 triangular matrix and a 4x4 null matrix. Thus, we have

      Q^t * A * X1 = R * P * X2

      Q^t * A * X1 = [R1;0] * P * X2

      [Q1,Q2]^t * A * X1 = [R1;0] * P * X2
   
   where Q1 and Q2 include the first and last 4 columns of Q.
   Finally, we have

       Q2^t * A * X1 = 0

   which define 4 equations in 4 variables.

   We also have 

       Q1^t * A * X1 = R1 * P * X2

   Thus, given X1, X2 (actually P * X2) can be recovered solving a 
   triangular system.

   \param simplify Simplification level. Number of variables to eliminate.
   \param offs The offsets defining the sub-problem, possibly NULL.
   \param ts The sequence to simplify. 

   \return TRUE if the simplifications was successful
*/
boolean SimplifyTransforms(unsigned int simplify,signed int *offs,Ttransforms *ts);


/** 
   \brief Updates the simplification.

   Updates the simplification taking into account a new set of offsets.

   Simplification has to be done for each independent problem (for each set
   of offsets). Repeating it from scratch is slow. This function allows
   for a fast computation of the simplification for a new set of 
   offsets. It should be applied to previously simplified
   transforms.

   \param offs The offsets defining the sub-problem, possibly NULL.
   \param ts The sequence to re-simplify. 
*/
void ReSimplifyTransforms(signed int *offs,Ttransforms *ts);
  
/** 
   \brief Gets the simplification matrix.

   Once we simplify a transforms sequence, we define a matrix S such that
      - T_r = S * T_k 
      .
    where T_r and T_k are the monomials of the removed and kept variables,
    repectively.

    \param S The matrix. The space for the matrix is allocated inside this
             function (and must be de-allocated by the caller). If matrix
	     S is only define for 4-2 simplifications (loops with 6 dof
	     where 2 variables are removed and 4 are kept). 
    \param ts The transform sequence to query.
*/
void GetTransformSimplification(double **S,Ttransforms *ts);

/** 
   \brief Evalutes the transform sequence from the coefficient matrix.

   Evalutes the transform sequence from the coefficient matrix obtained with \ref CoefficientMatrix

   \param val The values of the variables.
   \param m The matrix of coefficients.
   \param q The result of the evaluation.
   \param ts The transform sequence.

   \return The number of equations. The same as \ref NTransformsEquations
           This is the size required for q.
*/
unsigned int EvaluateTransformsFromCoefficients(double *val,double *m,double *q,Ttransforms *ts);

/** 
   \brief Defines a matrix of coefficients.

   Defines a matrix of coefficients, i.e., a matrix so that we can
   represent the multilinear polynomial encoded in the sequence of 
   transforms as:

   M*C

   where C is a vector of monimials x_1^{0,1}*....*x_n^{0,1}, i.e.,
   all the multilinear monomials with the variables in the sequence.
   M is the matrix of constant coefficients computed in this function.
   
   \param offs The offsets. Each set of offsets define a different
               problem and, thus, a different matrix.
   \param m The output matrix. The matrix is allocated in this function 
            and stored columwise.
   \param normalize TRUE if the equation coefficients have to be normalized.
   \param ts The transforms sequence to evaluate.
 */
void CoefficientMatrix(signed int *offs,double **m,boolean normalize,Ttransforms *ts);

/** 
   \brief Prints a sequence of transforms.

   Prints a sequence of transforms.

   \param f File where to print the transforms.
   \param varname The names of the variables.
   \param ts The transform sequence to print.
*/
void PrintTransforms(FILE *f,char **varname,Ttransforms *ts);

/** 
   \brief Prints the transforms as an equation.

   Prints the transforms as an equation that can be parsed, e.g., by the \ref cuikmaf.c "cuikmaf" application.

   \param f File where to print the transforms.
   \param varname The names of the variables.
   \param ts The transform sequence to print.
 */
void PrintTransformsEquations(FILE *f,char **varname,Ttransforms *ts);

/** 
   \brief Prints the transforms.

   Prints the transforms in the form of C*T where C is a constant
   matrix and T is a vector of monomials.

   \param f The file where to print.
   \param varname The name of the variables.
   \param ts The loops to print.
 */
void PrintTransformsMatrixVector(FILE *f,char **varname,Ttransforms *ts);

/** 
   \brief Destructor

   Deletes a sequence of transforms.

   \param ts The sequence to delete. 
*/
void DeleteTransforms(Ttransforms *ts);

#endif
