#ifndef VARIABLEH
#define VARIABLEH

#include "interval.h"
#include "boolean.h"

/** 
   \file variable.h

   \brief Definition of the Tvariable type and the associated functions. 

   Definition of the Tvariable type and the associated functions. 

   \sa Tvariable, variable.c.
*/

/** 
   \brief One of the variable types.

   Rotational variables.
*/
#define VAR_ROT   0 

/** 
   \brief One of the variable types.

   Translational variables.
*/
#define VAR_TRANS 1

/** 
   \brief One of the variable types.

   Offset variables.
*/
#define VAR_OFFSET 2

/************************************************************************************/

/** 
   \brief Data associated with each variable in the problem. 

   Data associated with each variable in the problem. 

   \sa variable.h, variable.c
*/
typedef struct
{
  char *name;            /**< Variable name. */
  unsigned int varType;  /**< Variable type. */
  unsigned int sv;       /**< The variable shifted (or affected) by the offset, if any. */
  Tinterval is;          /**< Valid range for this variable. */
} Tvariable;

/** 
   \brief Constructor.

   Creates a new variable.

   \param name Name for the new variable.
   \param varType The variable type. 
   \param v The variable to create.
*/
void NewVariable(char *name,unsigned int varType,Tvariable *v);

/** 
   \brief Constructor.

   Creates an offset new variable.

   \param name Name for the new variable.
   \param sv The shifted variable (the variable affected by the offset).
   \param v The variable to create.
*/
void NewOffsetVariable(char *name,unsigned int sv,Tvariable *v);

/** 
   \brief Copy constructor.
   
   Creates a new variable from another one.
   
   \param v_dst The variable to create.
   \param v_src The variable from where to copy.
*/
void CopyVariable(Tvariable *v_dst,Tvariable *v_src);

/** 
   \brief Gets the variable name.

   Gets the variable name.

   \param v The variable to query.

   \return The a pointer to the variable name.
 */
char *GetVariableName(Tvariable *v);

/** 
   \brief Gets the variable type.

   Gets the variable type.

   \param v The variable to query.

   \return The variable type.
 */
unsigned int GetVariableType(Tvariable *v);

/** 
   \brief Identifies offset variables.

   Identifies offset variables.

   \param v The variable.

   \return TRUE for offset variables.
*/
boolean IsOffsetVariable(Tvariable *v);

/** 
   \brief Shifted variable.

   The variable affected by the offset, if any.

   \param v The variable.

   \return The index of the variable affected by the
           offset of NO_UINT if the input 
	   is not an offset variable.
*/
unsigned int ShiftedVariable(Tvariable *v);

/** 
   \brief Sets the new range for the variable.

   Sets the new range for the variable.

   \param i The new range for the variable.
   \param v The variable to query.

*/
void SetVariableInterval(Tinterval *i,Tvariable *v);

/** 
   \brief Gets the  range of valid values for the variable.

   Gets the  range of valid values for the variable.

   \param v The variable to query.
   
   \return A pointer to the range of valid values for the variable.
*/
Tinterval *GetVariableInterval(Tvariable *v);

/** 
   \brief Prints a variable.

   Prints a variable (name plus valid range) to a stream that can be \e stdout.

   \param f The file where to print.
   \param v The variable to print.
*/
void PrintVariable(FILE *f,Tvariable *v);

/** 
   \brief Destructor.

   Deletes the Tvariable structure and frees the allocated memory.

   \param v The variable to delete.
*/
void DeleteVariable(Tvariable *v);

#endif
