#ifndef VARIABLESH
#define VARIABLESH

#include "variable.h"
#include "variable_set.h"
#include "boolean.h"

#include <stdio.h>

/** 
   \file variables.h

   \brief Definition of the Tvariables type and the associated functions. 

   Definition of the Tvariables type and the associated functions. 

   We have four types of variables: 
    - system variables: the variables of the mechanisms.
    - dummy variables: introduced when linearizing the problem. Dummy variables are only
      reduced using equation-wise constraints and are never bisected.
    - cartesian varibles: they refer to the cartesian position of point in the mechanisms. Thes
      variables are reduced but not bisected.
    .

   The position of a variable in the set is taken as its identifier and used to refer to
   this variable from many other structures (i.e., Tvariable_set).

   \sa Tvariables, variables.c.
*/

/** 
   \brief Set of variables of a cuiksystem. 

   Set of variables  involved in a cuiksystem.

   \sa variables.h, variables.c.
*/
typedef struct {
  unsigned int max_variables; /**< Maximum number of variables in the set.*/
  unsigned int n;             /**< Current number of variables in the set.*/
  unsigned int no;            /**< Number of offset variables in the set. */
  Tvariable **vs;             /**< The information for each variable.*/
} Tvariables;

/** 
   \brief Constructor.

   Defines an empty set of variables.

   \param vs The variable set to initalize.
*/
void InitVariables(Tvariables *vs);

/** 
   \brief Copy onstructor.

   Defines a set of variables from another set

   \param vs_dst The variable set to initalize.
   \param vs_src The variable set from where to copy.
*/
void CopyVariables(Tvariables *vs_dst,Tvariables *vs_src);

/** 
   \brief Gets the number of variables in a set.

   Returns the number of variables in a variable set.

   \param vs The variable set to query.

   \return The number of variables in the set.
*/
unsigned int NVariables(Tvariables *vs);

/** 
   \brief Gets the number of offset variables in a set.

   Returns the number of offset variables in a variable set.

   \param vs The variable set to query.

   \return The number of offset variables in the set.
*/
unsigned int NOffsetVariables(Tvariables *vs);

/** 
   \brief Gets the number of offset variables in a set.

   Returns the number of offset variables in a variable set.

   \param vs The variable set to query.

   \return The number of offset variables in the set.
*/
unsigned int NOffsetVariables(Tvariables *vs);

/** 
   \brief Adds a variable to the set.

   Adds a variable to the set.

   \param v The variable to add.
   \param vs The variable set to update.
*/
unsigned int AddVariable(Tvariable *v,Tvariables *vs);

/** 
   \brief Gets the information of a variable given the name of this variable.

   Returns a pointer to the variable information given the name of the variable 
   to query.

   \param name The name of the variable to retrive.
   \param vs The variable set to query. 

   \return A pointer to the information relative to the variable with the given name.
           NULL if there is no variable with the given name.
*/
Tvariable *GetVariableWithName(char *name,Tvariables *vs);

/** 
   \brief Gets the information of a variable given its number in the set (i.e., its
          identifier).

   Returns a pointer to the variable information given its position in the set (i.e.,
   its identifier).
   If the set has less than \e n variables, this function triggers an error.

   \param n The number of the variable to retrive.
   \param vs The variable set to query. 

   \return A pointer to the information relative to the variable with the given name.
*/
Tvariable *GetVariable(unsigned int n,Tvariables *vs);

/** 
   \brief Gets the variable identifier (i.e., its position in the set) given a variable name.

   Returns  the variable identifier (i.e., its position in the set) given a variable name. 

   \param name The name of the variable whose identifier we want to retrive.
   \param vs The variable set to query. 

   \return The variable indentifier or NO_UINT if there is not a variable with the given
           name in the set.
*/
unsigned int GetVariableID(char *name,Tvariables *vs);

/** 
   \brief Gets the name for all the variables in the set.

   Gets a pointer to the name for all the variables in the set.

   \param varNames Space for the pointers to the variable names.
   \param vs The variable set to query. 
*/
void GetVariableNames(char **varNames,Tvariables *vs);

/** 
   \brief Gets the name for a particular variable.

   Gets a pointer to the name if a particular variable.

   \param i The identifier of the variable.
   \param vs The variable set to query. 

   \return The pointer to the variable name or NULL if the identifier
           does not correspond to a valid variable.
*/
char *VariableName(unsigned int i,Tvariables *vs);

/** 
    \brief Creates a point from the center of the ranges of a set of variables.

    Creates a point from the center of the ranges of a set of variables.

    \param p The point to be created. The space for this point is allocated
             inside this function but must be deallocated externally.
    \param vs The variable set.
*/
void PointFromVariables(double **p,Tvariables *vs);

/** 
   \brief Prints a set of variables to an stream.

   Prints a set of variables to an stream, that can be \e stdout.
   Variables are printed in a format that can be properly parsed
   by the \e .cuik file parser.

   \param f The stream where to print.
   \param vs The variable set to print. 

   \sa readcuiksystem.l, readcuiksystem.y.
*/
void PrintVariables(FILE *f,Tvariables *vs);

/** 
   \brief Destructor.

   Deletes the Tvariables structure and frees the allocated memory.

   \param vs The variable set to delete. 
*/
void DeleteVariables(Tvariables *vs);

#endif
