
#include "cuiksystem.h"
#include "parameters.h"

#include "defines.h"
#include "error.h"
#include "filename.h"
#include "random.h"

#include "basic_algebra.h"

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>

/** \file cuik.c
    \brief Main body of the \ref cuik.c "cuik" application.
 
    <center>
    \dot
    digraph applications {
      size="7,15"
      node [shape=record, style=filled, fontname=Helvetica, fontsize=12];

      cuik    [label="cuik"    color="#99ff99" shape=ellipse ];
      
      param   [label="param"   color="#ff9999"];
      kin     [label="maf"   color="#ff9999"];
      sol     [label="sol"     color="#ff9999"];

      param -> cuik
      kin -> cuik
      cuik -> sol
    }
    \enddot
    </center>

    This is the main application for position analysis. 
    <br>

    \sa cuiksystem.h cuiksystem.c
*/

/** \def PARALLEL

    \brief Parallel execution.

    If set, the problems with different offsets are solved in parallel.
*/
#ifdef _OPENMP
  #include <omp.h>
  #define PARALLEL 1
#else
  #define PARALLEL 0
#endif

/** 
   \brief Save the solutions as found.

   If set, the solutions are saved as found. This is advisable for large 
   problems.
*/
#define SAVE_ON_LINE 0


/** 
   \brief Main body of the \b cuik application.

   Main body of the \b cuik application.

   Use:
      - \b cuik \e problem_name.cuik
      .
   where
      - \e problem_name contains the kinematic equations
      .

   The \e .cuik extension is optional.
   
   \param argc Number of elements in the command line that starts the application (i.e., the
               \b cuik command itself and its arguments).
   \param arg Strings with the arguments.
   
   \return EXIT_SUCCESS (0) if the execution worked out fine and EXIT_FAILURE if not.
*/
int main(int argc, char **arg)
{
  FILE *f_out;             /* File where the solutions are printed as they are found */
   
  Tfilename fparam;
  Tfilename fsols;

  unsigned int i;
  unsigned int ns;         /* Number of cuiksystems */
  TCuikSystem *cuiksystem; /* The set of variables and equations */  
  Tparameters parameters;  /* Parameters used in the Cuik process */
  Tlist **boxes;           /* Initial set of boxes for each sub-problem (if given) */ 
  #if (!SAVE_ON_LINE)
    Tlist *verifiedS;        /* The list of verified solutions (not used for simplified systems) */
    Tlist *solutions;        /* The list of other solutions */
    double vs;
    unsigned int k;
  #endif
  boolean saveAngles;      /* TRUE when saving in angular space */
  TStatistics *st;         /* Statistics for each sub-problem */
  TStatistics stg;         /* Global statistics */
  
  unsigned int ri;
  
  struct timespec t0,t1,t2,t3;

  if (argc>1)
    {      
      /* Set the random seed */
      ri=(unsigned int)time(NULL);
      //ri=1465223267;   // We use this to fix the random seed when debugging
      randomSet(ri);

      /*Init parameters*/
      CreateFileName(NULL,arg[1],NULL,PARAM_EXT,&fparam);
      InitParametersFromFile(GetFileFullName(&fparam),&parameters);
	  
      /*Create the output file */
      saveAngles=(GetParameter(CT_SAVE_ANGLES,&parameters)>0);
      
      clock_gettime(CLOCK_REALTIME,&t0);
      
      /* Read the problem file */
      ns=ReadProblemFromFile(&parameters,arg[1],&cuiksystem);
      
      clock_gettime(CLOCK_REALTIME,&t1);
	  
      /* Read the input files */
      NEW(boxes,ns,Tlist*);
      if (argc>2)
	{
	  FILE *f_in;              /* File from where to read the input boxes, if necessary. */
	  Tfilename fboxes;
	  Tlist boxesIn;

	  /* If we save boxes in angles, we assume we have to read boxes in angles too */
	  CreateFileName(NULL,arg[2],NULL,SOL_EXT,&fboxes);

	  f_in=fopen(GetFileFullName(&fboxes),"r");
	  fprintf(stderr,"Reading boxes from: %s\n",GetFileFullName(&fboxes));
	  
	  if (!f_in)
	    Error("Could not open input file");
	  
	  ReadCuikBoxList(f_in,&boxesIn);
	  
          #pragma omp parallel for private(i) if ((!_DEBUG)&&(PARALLEL)&&(ns>1))
	  for(i=0;i<ns;i++)
	    {
	      NEW(boxes[i],1,Tlist);
	      IntersectWithCuikSystem(&parameters,&boxesIn,boxes[i],&(cuiksystem[i]));
	    }

	  DeleteFileName(&fboxes);
	}
      else
	{
	  for(i=0;i<ns;i++)
	    boxes[i]=NULL;
	}
      
      /* Allocate space for the solutions */
      #if (!SAVE_ON_LINE)
        /* ns is the number of sub-problems to solve */
        NEW(solutions,ns,Tlist);
	NEW(verifiedS,ns,Tlist);
      #endif
      NEW(st,ns,TStatistics);

      /* Open the output file */
      if (saveAngles)
	CreateFileName(NULL,arg[1],"_angles",SOL_EXT,&fsols);
      else
	CreateFileName(NULL,arg[1],NULL,SOL_EXT,&fsols);
      
      f_out=fopen(GetFileFullName(&fsols),"w"); /*start a brand new output file*/
      
      if (!f_out)
	Error("Could not open the output file");
      
      fprintf(f_out,"\n==========================================================================\n");
      fprintf(f_out,"Equations from file: %s\n",arg[1]); /* The name of the problem that was solved*/
      fprintf(f_out,"==========================================================================\n");
      fprintf(f_out,"Random seed: %u\n",ri);
      PrintParameters(f_out,&parameters);
      fprintf(f_out,"==========================================================================\n");      
      fprintf(f_out,"SOLUTIONS:\n\n"); 
      fflush(f_out);
      
      clock_gettime(CLOCK_REALTIME,&t2);

      #pragma omp parallel for private(i) if ((!_DEBUG)&&(PARALLEL)&&(ns>1))
      for(i=0;i<ns;i++)
	{
          #if (SAVE_ON_LINE)
	    SolveCuikSystem(i,&parameters,boxes[i],f_out,NULL,NULL,&(st[i]),&(cuiksystem[i]));
	  #else
	    SolveCuikSystem(i,&parameters,boxes[i],NULL,&(solutions[i]),&(verifiedS[i]),&(st[i]),&(cuiksystem[i]));
	  #endif
	}
   
      clock_gettime(CLOCK_REALTIME,&t3);

      #if (!SAVE_ON_LINE)
        vs=0.0;    
	k=1;

	for(i=0;i<ns;i++)
	  {
	    SaveCuikBoxList(f_out,k,saveAngles,TRUE,TRUE,TRUE,&(cuiksystem[i]),&(verifiedS[i]));
	    vs+=CuikBoxListVolume(NULL,&(verifiedS[i]));
	    k+=ListSize(&(verifiedS[i]));
	    DeleteCuikBoxList(&(verifiedS[i]));
	  }
	free(verifiedS);
	
	for(i=0;i<ns;i++)
	  {
	    SaveCuikBoxList(f_out,k,saveAngles,TRUE,TRUE,FALSE,&(cuiksystem[i]),&(solutions[i]));
	    vs+=CuikBoxListVolume(NULL,&(solutions[i]));
	    k+=ListSize(&(solutions[i]));
	    DeleteCuikBoxList(&(solutions[i]));
	  }
	free(solutions);
      #endif
            
      /* Define the global statistics from the execution statistics of each sub-problem */
      ResetStatistics(&stg);
      for(i=0;i<ns;i++)
	{
	  AccumulateStatistics(&(st[i]),&stg);
	  DeleteStatistics(&(st[i]));
	}
      free(st);

      /* Print information on screen */
      PrintParameters(stderr,&parameters);
      PrintStatistics(stderr,"Statistics",&stg);
      fprintf(stderr,"Num. problems  : %u\n",ns);
      #ifdef _OPENMP
        fprintf(stderr,"Num threads    : %u\n",omp_get_max_threads());
      #endif
      fprintf(stderr,"Reading time   : %f seconds\n",(t1.tv_sec+t1.tv_nsec*1e-9)-(t0.tv_sec+t0.tv_nsec*1e-9));
      fprintf(stderr,"Execution time : %f seconds\n",(t3.tv_sec+t3.tv_nsec*1e-9)-(t2.tv_sec+t2.tv_nsec*1e-9));
      fprintf(stderr,"Total time     : %f seconds\n",(t1.tv_sec+t1.tv_nsec*1e-9)-(t0.tv_sec+t0.tv_nsec*1e-9)+(t3.tv_sec+t3.tv_nsec*1e-9)-(t2.tv_sec+t2.tv_nsec*1e-9));
      
      fprintf(f_out,"\n==========================================================================\n");
      PrintStatistics(f_out,"Statistics",&stg);
      #if (!SAVE_ON_LINE)
        fprintf(f_out,"Solution volume: %g\n",vs);
      #endif

      fclose(f_out);

      /* Delete the input boxes, if any */
      if (boxes[0]!=NULL)
	{
	  for(i=0;i<ns;i++)
	    {
	      DeleteCuikBoxList(boxes[i]);
	      free(boxes[i]);
	    }
	}
      free(boxes);
	      
      /* Delete the execution statistics */
      DeleteStatistics(&stg);
      
      /*Delete  the parameters and the equations*/
      DeleteParameters(&parameters);

      for(i=0;i<ns;i++)
	DeleteCuikSystem(&(cuiksystem[i]));
      free(cuiksystem);
 
      DeleteFileName(&fparam);
      DeleteFileName(&fsols);
    }
  else
    {
      fprintf(stderr,"  Wrong number of parameters.\n");
      fprintf(stderr,"  Use:\n");   
      fprintf(stderr,"      cuik <problem file name> \n");
      fprintf(stderr,"  where <problem file name> is a 6R/loops/cuik/maf file\n");
    }

  return(EXIT_SUCCESS);
}

