
#include "cuiksystem.h"
#include "parameters.h"

#include "defines.h"
#include "error.h"
#include "filename.h"

#include <string.h>
#include <stdlib.h>
#include <time.h>

#ifdef _OPENMP
  #include <omp.h>
#endif

/** \file cuikcluster.c
    \brief Main body of the \ref cuikcluster.c "cuikcluter" application.
 
    <center>
    \dot
    digraph applications {
      size="7,15"
      node [shape=record, style=filled, fontname=Helvetica, fontsize=12];

      cluster    [label="_cluster.sol"    color="#99ff99" shape=ellipse ];
      
      kin     [label="cuik"   color="#ff9999"];
      sol     [label="sol"     color="#ff9999"];

      sol -> cuik
      kin -> cluster
    }
    \enddot
    </center>

    This application cluster lists of boxes. 
    <br>

    \sa cuiksystem.h cuiksystem.c
*/

/** 
   \brief Main body of the \b cuikcluster application.

   Main body of the \b cuikcluster application.

   Use:
      - \b cuikcluster \e problem_name.sol \e tolerance \e angles
      .
   where
      - \e problem_name contains the solutions to cluster
      - \e tolerance [optional] Tolerance for box intersection. Default is 0.
      - \e angles [optional] Set to 1 if the boxes should be considered angles. Default 1.
      .

   The \e .sol extension is optional.
   
   \param argc Number of elements in the command line that starts the application (i.e., the
               \b cuik command itself and its arguments).
   \param arg Strings with the arguments.
   
   \return EXIT_SUCCESS (0) if the execution worked out fine and EXIT_FAILURE if not.
*/
int main(int argc, char **arg)
{
  FILE *f_out;             /* File where the solutions are printed as they 
			      are found */
 
  FILE *f_in;            /* Used to read-and-print the equation input 
			    file in the output file */  
  Tlist solutions,clusters;

  Tfilename fsols;
  Tfilename fclusters;

  boolean angles;
  double tol;

  struct timespec t0,t1;
  
  if (argc>1)
    {      
      /* Read the list of boxes */
      CreateFileName(NULL,arg[1],NULL,SOL_EXT,&fsols);
      f_in=fopen(GetFileFullName(&fsols),"r");
      if (!f_in)
	Error("Can not open the file with the solutions");

      if (argc>2)
	tol=atof(arg[2]);
      else
	tol=0.0;
      
      if (argc>3)
	angles=(boolean)atoi(arg[3]);
      else
	angles=TRUE;
      
      ReadCuikBoxList(f_in,&solutions);
      
      clock_gettime(CLOCK_REALTIME,&t0);
      
      ClusterCuikBoxes(&clusters,tol,angles,&solutions);

      clock_gettime(CLOCK_REALTIME,&t1);
      fprintf(stderr,"Execution time: %f seconds\n",(t1.tv_sec+t1.tv_nsec*1e-9)-(t0.tv_sec+t0.tv_nsec*1e-9));
      
      /* Save the list of boxes */
      CreateFileName(NULL,arg[1],"_cluster",SOL_EXT,&fclusters);
      f_out=fopen(GetFileFullName(&fclusters),"w");
      if (!f_out)
	Error("Can not open the file for the clusters");
      SaveCuikBoxList(f_out,1,FALSE,FALSE,FALSE,FALSE,NULL,&clusters);

      /* Release memory */
      DeleteCuikBoxList(&solutions);
      DeleteCuikBoxList(&clusters);
      
      DeleteFileName(&fsols);
      DeleteFileName(&fclusters);
    }
  else
    {
      fprintf(stderr,"  Wrong number of parameters.\n");
      fprintf(stderr,"  Use:\n");   
      fprintf(stderr,"      cuikcluster <problem file name>.sol \etolerance \e angles\n");
      fprintf(stderr,"  where <problem file name> contains the solutions\n");
      fprintf(stderr,"            (the '.sol' extension is not required).\n");
      fprintf(stderr,"        <tolerance> Tolerance for box intersection.\n");
      fprintf(stderr,"        <angles> is set to 1 if the boxes contain angles [optional with default=1].\n");
    }

  return(EXIT_SUCCESS);
}

