#include "cuiksystem.h"

#include "error.h"
#include "boolean.h"
#include "defines.h"
#include "variable.h"
#include "random.h"
#include "filename.h"
#include "basic_algebra.h"


#include <stdlib.h>
#include <sys/stat.h>

#include <math.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <signal.h>
#ifdef _OPENMP
  #include <omp.h>
#endif
#ifdef _LAPACKE
  #include <lapacke.h>
#else
  /* assuming clapack */
  #include <clapack.h>
#endif


/** 
   \brief If set the 6R are defined with an extra equation.

    If set the 6R are defined with an extra equation.
    Experimentally this seems not to pay off and, so, it may be better
    to set it to 0 by default.
*/
#define REDUNDANT_6R 0


/** 
   \brief Define a 6R as a single problem.

   If set, the 6R is defined as a single problem, with no offsets and with large
   ranges for the variable (see \ref LARGE_RANGE_6R).

   This is used only for experimental purposes and should be 0 normally.
*/
#define SINGLE_6R 0

/** 
   \brief Range for variables when \ref SINGLE_6R is set.

   Range for variables when \ref SINGLE_6R is set.
   IF  \ref SINGLE_6R is not set (the default), this is not used.

   Value larger than 100 are problematic in general. With 100 the explored
   range is [-178.85,+178.85] (just less than 3 degrees are out).
*/
#define LARGE_RANGE_6R 100

/** \file cuiksystem.c
    \brief Implementation of the functions operating on TCuikSystem.

    Implementation of the functions operating on TCuikSystem.

    \sa TCuikSystem, cuiksystem.h.
*/

inline void InitStatistics(unsigned int nBoxes,unsigned int nReductions,unsigned int nBisected,
			   unsigned int nEmpty,unsigned int nSols,unsigned int vSols,
			   unsigned int maxLevel,TStatistics *st)
{
  if (st!=NULL)
    {
      st->nBoxes=nBoxes;
      st->nReductions=nReductions;
      st->nBisected=nBisected;
      st->nEmpty=nEmpty;
      st->nSols=nSols;
      st->vSols=vSols;
      st->maxLevel=maxLevel;
    }
}

void ResetStatistics(TStatistics *st)
{
  if (st!=NULL)
    {
      st->nBoxes=0;
      st->nReductions=0;
      st->nBisected=0;
      st->nEmpty=0;
      st->nSols=0;
      st->vSols=0;
      st->maxLevel=0;
    }
}

void AccumulateStatistics(TStatistics *st,TStatistics *stg)
{
  if ((st!=NULL)&&(stg!=NULL))
    {
      stg->nBoxes+=st->nBoxes;
      stg->nReductions+=st->nReductions;
      stg->nBisected+=st->nBisected;
      stg->nEmpty+=st->nEmpty;
      stg->nSols+=st->nSols;
      stg->vSols+=st->vSols;
      if (st->maxLevel>stg->maxLevel)
	stg->maxLevel=st->maxLevel;
    }
}

void PrintStatistics(FILE *f,char *title,TStatistics *st)
{
  if (st!=NULL)
    {
      fprintf(f,"%%   REDUNDANT_6R : %u\n",REDUNDANT_6R);
      if (title!=NULL) fprintf(f,"%s\n",title);
      fprintf(f,"  Processed boxes     : %u\n",st->nBoxes);
      fprintf(f,"  Number of reductions: %u\n",st->nReductions);
      fprintf(f,"  Bisected boxes      : %u\n",st->nBisected);
      fprintf(f,"  Empty boxes         : %u\n",st->nEmpty);
      fprintf(f,"  Solution boxes      : %u\n",st->nSols);
      fprintf(f,"  Verified solutions  : %u\n",st->vSols);
      fprintf(f,"  Max level           : %u\n",st->maxLevel);
    }
}

inline void DeleteStatistics(TStatistics *st)
{}


/** 
    \brief Initializes a cuik box.

    Initializes a cuik box with all ranges set to [-1,1]-

    \param d Dimension.
    \param b The box.
*/
void UnitCuikBox(unsigned int d,TCuikBox *b);

/** 
    \brief Copies a box.

    Copies a box.

    \param b_dst The box where to copy.
    \param b_src The original box. 
*/
void CopyCuikBox(TCuikBox *b_dst,TCuikBox *b_src);

/** 
   \brief Center of a box.

   Gets the point at the center of a box.

   \param x The center to define.
   \param b The input box
*/
void CuikBoxCenter(double *x,TCuikBox *b);

/** 
   \brief Determines if a point is inside a box.

   Determines if a point is inside a box.

   \param x The point to define.
   \param b The input box

   \return TRUE if the point is in the box.
*/
boolean PointInCuikBox(double *x,TCuikBox *b);

/** 
    \brief Checks if two boxes intersect.

    Checks if two boxes intersect.

    \param tol Tolerance. Boxes are enlarged by this size  
               to detect the intersection even in the presence
	       of numerical errors. Can be set to 0 if not
	       tolerance is required.
    \param angles TRUE if the boxes contain angles.
    \param b1 The first box.
    \param b2 The second box. 

    \return TRUE if the two boxes intersect.
*/
boolean CuikBoxesIntersect(double tol,boolean angles,TCuikBox *b1,TCuikBox *b2);

/** 
    \brief Union of two boxes.

    Expands the second box so that it fully includes the first one.

    \param angles TRUE if the boxes include angles.
    \param b1 The first box.
    \param b The second box. 
*/
void CuikBoxUnion(boolean angles,TCuikBox *b1,TCuikBox *b);

/** 
   \brief Volume of the box.

   Volume of the box.

   \param used The used variables.
   \param b The box.

   \return The volume of the box.
*/
double CuikBoxVolume(boolean *used,TCuikBox *b);

/** 
   \brief Size of the box.

   Size of the box.

   \param used The used variables.
   \param b The box.

   \return The size of the box.
*/
double CuikBoxSize(boolean *used,TCuikBox *b);

/** 
    \brief Splits a box in two sub-boxes.

    Splits a box in two sub-boxes.
    
    \param nv The range to split
    \param alpha The split point
    \param b1 The first resulting sub-box
    \param b The input box and the second box resulting from the split.
*/
void SplitCuikBox(unsigned int nv,double alpha,TCuikBox *b1,TCuikBox *b);

/** 
   \brief Offsets a cuik box in tangent space.

   Takes a box in tangent space and applies the offset (basically,
   we undo the offset to put a box in a common reference).
   
   \param usedVars Used variables. Can be NULL.
   \param varType The variables types.
   \param offset The offsets (one per dimension).
   \param b The box to transform.
*/
void CuikBoxOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b);

/** 
   \brief Unoffsets a cuik box in tangent space.

   Takes a box in tangent space and removes the offset (basically,
   we undo the offset to put a box in a common reference).
   
   \param usedVars Used variables. Can be NULL.
   \param varType The variables types.
   \param offset The offsets (one per dimension).
   \param b The box to transform.
*/
void CuikBoxUnOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b);

/** 
   \brief Determines the offset of a box.

   Takes a box and determines the offset applied
   to generate the box. This works independently if
   the box containts angles or tangent parameters.

   \param usedVars Used variables. Can be NULL.
   \param varType The variables types.
   \param b The box.

   \return A code giving the sub-problem that generated the box.
           Each sub-problem is defined with a particular offset.
*/
unsigned int GetCuikBoxOffset(boolean *usedVars,unsigned int *varType,TCuikBox *b);

/** 
   \brief Converts a box in tangent space to the angular space.

   Takes a box in tangent space and converts it to angular space.
   This function assumes that there is no offset in any variable.
   
   \param usedVars Used variables. Can be NULL.
   \param varType The variables types.
   \param b The box to transform.
*/
void CuikBox2Angles(boolean *usedVars,unsigned int *varType,TCuikBox *b);

/** 
   \brief Converts a box in angular space to the tangnet space.

   Takes a box in angular space and converts it to tangent space.
   This function assumes that there is no offset in any variable.
   
   \param usedVars Used variables. Can be NULL.
   \param varType The variables types.
   \param b The box to transform.
*/
void CuikBox2Tangent(boolean *usedVars,unsigned int *varType,TCuikBox *b);

/** 
   \brief Converts a box in tangent space to the angular space taking into acoount the offsets.

   Takes a box in tangent space and converts it to angular space taking into acoount the offsets.
   The offset is added after converting from tangent to angle.
    
   \param usedVars Used variables. Can be NULL.
   \param varType The variables types.
   \param offset The offsets (one per dimension).
   \param b The box to transform.
*/
void CuikBox2AnglesWithOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b);

/** 
   \brief Converts a box in angular space to the tangent space taking into acoount the offsets.

   Takes a box in angular space removes the offsets and converts it to tangent space.
   The offset is substracted from the angle and then the tangent space parameter is computed.
   
   \param usedVars Used variables. Can be NULL.
   \param varType The variables types.
   \param offset The offsets (one per dimension).
   \param b The box to transform.
*/
void CuikBox2TangentWithOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b);

/** 
   \brief Read a box from a file.

   Read a box from a file.

   \param f The file.
   \param b The box.
   
   \return TRUE if the read can be correctly read.
*/
boolean ReadCuikBox(FILE *f,TCuikBox *b);

/** 
   \brief Copies a list of boxes.

    Copies a list of boxes

    \param l_dst List to create.
    \param l_src List from where to copy.
*/
void CopyCuikBoxList(Tlist *l_dst,Tlist *l_src);

#if (SIMPLEX_AVAILABLE)
  /** 
     \brief Initializes the simplex.
     
      Initializes the simplex.
      This function must be called after initializing the cuiksystem.

      \param epsilon
      \param cs The cuiksystem structure.
  */
  void InitSimplex(double epsilon,TCuikSystem *cs);

  /** 
     \brief Sets the simplex matrix of a given box.
     
      Sets the simplex matrix of a given box.
      This is the function to call when we have a new box to reduce.

      \param epsilon Numerical accuracy.
      \param b The box.
      \param cs The cuiksystem structure.

      \return TRUE if the simplex was set correctly.
  */
  boolean SetSimplex(double epsilon,TCuikBox *b,TCuikSystem *cs);

  /** 
      \brief Deletes the symplex.

      Deletes the symplex.
      
      \param cs The cuiksystem structure.
  */
  void DeleteSimplex(TCuikSystem *cs);

#endif


/** 
   \brief Defines a cuiksystem for a 6R problem.

   Defines a cuiksystem for a 6R problem from the DH-parameters
   given in vectors a, d, and alpha znd the end-effector
   constraint (given in R and t).

   \param p The set of parameters.
   \param np Encodes the sub-problems (i.e., the offsets).
   \param tp Total of sub-problems. If 1, a single problem is defined, with large
             variable ranges and no offsets.
   \param a Vector of a's
   \param d Vector of d's
   \param alpha Vector of alpha's
   \param R The rotation matrix (4x4 homogeneous matrix) given the inverse (i.e., the
            transpose) of the end-effector rotation constraints.
   \param t The end-effector translation constraint. 
   \param cs The cuiksystem to define.
*/
void Define6RProblem(Tparameters *p,unsigned int np,unsigned int tp,
		     double *a,double *d,double *alpha,double *R,double *t,TCuikSystem *cs);


/** 
    \brief Checks a solution.

    Verfies if a box may include a solution: that all the functions
    have positive/negative values along all variables.

    This is mostly necessary when reducing the ranges one
    equation and variable at a time: The last equation may reduce
    on range and make the first equation unfeasible. Thus, before
    accepting a solution we have to perform a global test 

    \param p The set of parameters.
    \param b The box to be reduced.
    \param cs The cuiksystem with the equations to use.
    
    \return EMPTY if the box can not include a solution, REDUCED if it may
            include a solution, and REDUCED_WITH_SOLUTION if it for sure
	    includes a solution according to Miranda's test).

*/
unsigned int CheckSolution(Tparameters *p,TCuikBox *b,TCuikSystem *cs);

/** 
   \brief Examines a box trying to determine if it includes a solution point.

    Examines a box trying to determine if it includes a solution point.
    Different methods can be used (the one to use is fixed in the paremeters).
       - Miranda's theorem
       - Newton (possibly with replacement)
       - Face (using trapezoid or simplex)
       .
    
    \param p The set of parameters (including the selection of the verification 
             method to use).
    \param x Space for a point. Only used if Newton-based methods are selected.
             This space must be allocated/deallocated externally (it is more
	     efficient this way since we typically verify many boxes in sequence).
    \param b The box to verify.
    \param cs The cuiksystem. Important: If face-simplex is selected, the
              simplex structure in the cuiksystem must be initialized before
	      calling this function.

    \return \ref REDUCED or \ref REDUCED_WITH_SOLUTION
*/
unsigned int VerifyBox(Tparameters *p,double *x,TCuikBox *b,TCuikSystem *cs);

/** 
    \brief Reduces a box as much as possible.

    Reduces the ranges of a box as much as possible applying first an equation-wise
    crop and then a simplex-based reduction. If the reduction is significative (i.e.,
    the reduction ratio is above paramter RHO) and the box is not smaller than SMALL_SIGMA,
    the reduction process is iterated.
    <br>
    Observe that SMALL_SIGMA can be much smaller that SIGMA (the parameter to accept a box
    as a solution). The idea here is that if a box contains a punctual solution it will
    be shrink till it has a size below SMALL_SIGMA and, if the box bounds a patch of
    a continuous solution set, the box will not be smaller more than SIGMA.<br>
    If the reduced box is fully included in the original box, then we can certify that
    the box has a solution.

    \param p The set of parameters.
    \param b The box to be reduced.
    \param nRed [output] Number of reductions applied to the box.
    \param cs The cuiksystem with the equations to use.
    
    \return The status of the box after the reduction (CONTRACTED, REDUCED, REDUCED_WITH_SOLUTION, NOT_REDUCED, EMPTY).

*/
unsigned int ReduceBox(Tparameters *p,TCuikBox *b,unsigned int *nRed,TCuikSystem *cs);

#if (SIMPLEX_AVAILABLE)
  /** 
      \brief Reduces a box as much as possible.
      
      Reduces the ranges of a box as much as possible applying first an equation-wise
      crop and then a simplex-based reduction. If the reduction is significative (i.e.,
      the reduction ratio is above paramter RHO) and the box is not smaller than SMALL_SIGMA,
      the reduction process is iterated.
      <br>
      Observe that SMALL_SIGMA can be much smaller that SIGMA (the parameter to accept a box
      as a solution). The idea here is that if a box contains a punctual solution it will
      be shrink till it has a size below SMALL_SIGMA and, if the box bounds a patch of
      a continuous solution set, the box will not be smaller more than SIGMA.<br>
      If the reduced box is fully included in the original box, then we can certify that
      the box has a solution.
      
      \param p The set of parameters. 
      \param b The box to be reduced.
      \param nRed [output] Number of reductions applied to the box.
      \param cs The cuiksystem with the equations to use.
      
      \return The status of the box after the reduction (CONTRACTED, REDUCED, NOT_REDUCED, EMPTY).
      
  */
  unsigned int ReduceBoxSimplex(Tparameters *p,TCuikBox *b,unsigned int *nRed,TCuikSystem *cs);
#endif

/** 
   \brief Computes the Jacobian of the system of equations at a given point.

   Computes the Jacobian of the system of equations at a given point.

   \param x The point.
   \param veq The evaluation of the equations at x. If null the value is
              computed inside the funtion.
   \param J Space for the Jacobian matrix (a matrix of size ne x nv).
   \param cs The system.
*/
void ComputeJacobian(double *x,double *veq,double *J,TCuikSystem *cs);

/** 
   \brief Refines a solution using the Newton method.

   Applies the Newton interation until the error in a solution is below
   \ref CT_EPSILON "EPSILON". The error is measured as the norm of the
   equation evaluation at the given point.

   \param p The set of parameters.
   \param b The box where we are looking for a solution. Can be NULL.
   \param x The point.
   \param cs The cuiksystem.
*/
double Newton(Tparameters *p,TCuikBox *b,double *x,TCuikSystem *cs);

/** 
   \brief Evaluates an equation.

   Evaluates an equation interpolating from the unit box.

   \param i The equation to evaluate.
   \param x The point where to evaluate.
   \param cs The cuiksystem.
*/
double EvaluateCSEquation(unsigned int i,double *x,TCuikSystem *cs);


/** 
    \brief Modifies the evaluation of a function on a given dimension.

    Interpolates the value of a function on a new interval for a given
    variable, assuming that the limits for the rest of variables remains
    the same.

    This is the function to use when we change a box along one dimesion.

    \param nv Index of the variable (in the equation).
    \param l Lower limits for the variables when computing the function values.
    \param u Upper limits for the variables when computing the function values.
    \param newL New lower value for which we want to re-compute the functions. 
    \param newU New upper value for which we want to re-compute the functions. 
    \param nc Numbe of corners. Number of elements in 'f'.
    \param f The computed values. At input, the values of the equation
    at the box corners before changing the range. At output
    the new values after the modification of the range.
    \param cs The cuiksystem with the equations and variables.
*/
void InterpolateFunction(unsigned int nv,
			 double l,double u,double newL,double newU,
			 unsigned int nc,double *f,TCuikSystem *cs);
/** 
    \brief Evaluates an equation from an evaluation on the unit box.

    Uses an evaluation on a unit box ([0,1]^n) of an equation and the multilinearity 
    to get the values of the equation into the corners of a new box.

    \param ne The equation.
    \param l Lower limits for variables.
    \param u Upper limits for variables.
    \param f The computed values.
    \param cs The cuiksystem with the equations and variables.
*/
void InterpolateFunctionFromInitialBox(unsigned int ne,double *l,double *u,double *f,TCuikSystem *cs);

/**  
   \brief Determines the trapezoid for a given action.


   Projects the values for a given funcition (equation) on the lower and upper extremes of the
   range of a given variable. This defines a trapezoid that will be later clipped with the
   line f=0.

   \param epsilon Numerical accuracy.
   \param nf Number of elements in 'f'. This is 2^n with 'n' the number of variables in 
             the equation used to evaluate 'f'
   \param f The function (equation) values.
   \param v The variable. Note: this is the index of the variable in the equation, not
            the global index of the variable in the problem!
   \param ll Output. Lower bound of the trapezoid at the lower limit of the variable.
   \param lu Output. Upper bound of the trapezoid at the lower limit of the variable.
   \param ul Output. Lower bound of the trapezoid at the upper limit of the variable.
   \param uu Output. Upper bound of the trapezoid at the upper limit of the variable.
   \param cs The cuik system.
 */
void ComputeTrapezoid(double epsilon,unsigned int nf,double *f,unsigned int v,
		      double *ll,double *lu,double *ul,double *uu,
		      TCuikSystem *cs);


/** 
   \brief Trapezoid clipping.

   Cuts a trapezoid with the zero level.

   NOTE: newL and newU must be initialized externally to the variable ranges
   used to computed the trapezoid.

   \param epsilon Numerical interval.
   \param l The lower limit used to compute the trapezoid.
   \param u The upper limit used to compute the trapezoid.
   \param newL The new lower limit. As input, it must be l or any number in [l,u]
               out of a previous reducion process.
   \param newU The new upper limit. As input, it must be u or any number in [l,u]
               out of a previous reducion process.
   \param ll Lower bound of the trapezoid at the lower limit of the variable.
   \param lu Upper bound of the trapezoid at the lower limit of the variable.
   \param ul Lower bound of the trapezoid at the upper limit of the variable.
   \param uu Upper bound of the trapezoid at the upper limit of the variable.

   \return Either \ref EMPTY, \ref CONTRACTED, \ref REDUCED or \ref NOT_REDUCED. 
*/
unsigned int TrapezoidClipping(double epsilon,
			       double l,double u,double *newL,double *newU,
			       double ll,double lu,double ul,double uu);


/** 
    \brief Maximum error for a box.

    Computes the error for the solution represented by a box. The error
    is computed taking the central point of the box and replacing the
    values in the equation set.
    
    Only system equality equations are taken into account in the error.

    \param b The solution box.
    \param cs The cuiksystem.

    \return Maximum error for the equations for the central point of the
            box.
*/
double ErrorInSolution(TCuikBox *b,TCuikSystem *cs);

  
/************************************************************************/
/************************************************************************/
/************************************************************************/

void InitCuikBox(double *l,double *u,unsigned int d,TCuikBox *b)
{
  b->level=1;
  b->d=d;
  
  NEW(b->l,2*d,double);
  b->u=&(b->l[d]);

  memcpy(b->l,l,sizeof(double)*d);
  memcpy(b->u,u,sizeof(double)*d);
}

void UnitCuikBox(unsigned int d,TCuikBox *b)
{
  unsigned int i;
  
  b->level=1;
  b->d=d;
  
  NEW(b->l,2*d,double);
  b->u=&(b->l[d]);

  for (i=0;i<d;i++)
    {
      b->l[i]=-1;
      b->u[i]=+1;
    }
}

void CopyCuikBox(TCuikBox *b_dst,TCuikBox *b_src)
{ 
  b_dst->level=b_src->level;
  b_dst->d=b_src->d;
  
  NEW(b_dst->l,2*b_dst->d,double);
  b_dst->u=&(b_dst->l[b_dst->d]);

  memcpy(b_dst->l,b_src->l,2*sizeof(double)*b_dst->d);

}
 
void CuikBoxCenter(double *x,TCuikBox *b)
{
  unsigned int i;
  
  for(i=0;i<b->d;i++)
    x[i]=(b->u[i]+b->l[i])/2.0;
}

boolean PointInCuikBox(double *x,TCuikBox *b)
{
  boolean in;
  unsigned int i;

  in=TRUE;
  for(i=0;((i<b->d)&&(in));i++)
    in=((b->l[i]<=x[i])&&(x[i]<=b->u[i]));

  return(in);
}

boolean CuikBoxesIntersect(double tol,boolean angles,TCuikBox *b1,TCuikBox *b2)
{
  boolean intersect;
  unsigned int i;
  
  intersect=(b1->d==b2->d);
  for(i=0;((intersect)&&(i<b1->d));i++)
    {

      if (angles)
	{
	  intersect=((((b1->l[i]<=b2->l[i]-tol)&&(b2->l[i]-tol<=b1->u[i]))|| // No shift
		      ((b1->l[i]<=b2->u[i]+tol)&&(b2->u[i]+tol<=b1->u[i]))||
		      ((b2->l[i]-tol<=b1->l[i])&&(b1->l[i]<=b2->u[i]+tol))||
		      ((b2->l[i]-tol<=b1->u[i])&&(b1->u[i]<=b2->u[i]+tol)))||
		     
		     (((b1->l[i]-M_2PI<=b2->l[i]-tol)&&(b2->l[i]-tol<=b1->u[i]-M_2PI))|| // Shift b1 -2*PI
		      ((b1->l[i]-M_2PI<=b2->u[i]+tol)&&(b2->u[i]+tol<=b1->u[i]-M_2PI))||
		      ((b2->l[i]-tol<=b1->l[i]-M_2PI)&&(b1->l[i]-M_2PI<=b2->u[i]+tol))||
		      ((b2->l[i]-tol<=b1->u[i]-M_2PI)&&(b1->u[i]-M_2PI<=b2->u[i]+tol)))||
		     
		     (((b1->l[i]+M_2PI<=b2->l[i]-tol)&&(b2->l[i]-tol<=b1->u[i]+M_2PI))|| // Shift b1 +2*PI
		      ((b1->l[i]+M_2PI<=b2->u[i]+tol)&&(b2->u[i]+tol<=b1->u[i]+M_2PI))||
		      ((b2->l[i]-tol<=b1->l[i]+M_2PI)&&(b1->l[i]+M_2PI<=b2->u[i]+tol))||
		      ((b2->l[i]-tol<=b1->u[i]+M_2PI)&&(b1->u[i]+M_2PI<=b2->u[i]+tol))));
	}
      else
	intersect=(((b1->l[i]<=b2->l[i]-tol)&&(b2->l[i]-tol<=b1->u[i]))||
		   ((b1->l[i]<=b2->u[i]+tol)&&(b2->u[i]+tol<=b1->u[i]))||
		   ((b2->l[i]-tol<=b1->l[i])&&(b1->l[i]<=b2->u[i]+tol))||
		   ((b2->l[i]-tol<=b1->u[i])&&(b1->u[i]<=b2->u[i]+tol)));
    }
  return(intersect);
}

void CuikBoxUnion(boolean angles,TCuikBox *b1,TCuikBox *b)
{
  unsigned int i;
  double l1,l2,l3;
  double u1,u2,u3;
  double s1,s2,s3;

  for(i=0;i<b->d;i++)
    {
      if (angles)
	{
	  /* Union without any shift */
	  if (b1->l[i]<b->l[i]) l1=b1->l[i]; else l1=b->l[i];
	  if (b1->u[i]>b->u[i]) u1=b1->u[i]; else u1=b->u[i];
	  s1=u1-l1;

	  /* Union shifting b1 -2*pi */
	  if (b1->l[i]-M_2PI<b->l[i]) l2=b1->l[i]-M_2PI; else l2=b->l[i];
	  if (b1->u[i]-M_2PI>b->u[i]) u2=b1->u[i]-M_2PI; else u2=b->u[i];
	  s2=u2-l2;
	  
	  /* Union shifting b1 +2*pi */
	  if (b1->l[i]+M_2PI<b->l[i]) l3=b1->l[i]+M_2PI; else l3=b->l[i];
	  if (b1->u[i]+M_2PI>b->u[i]) u3=b1->u[i]+M_2PI; else u3=b->u[i];
	  s3=u3-l3;

	  if ((s1<s2)&&(s1<s3))
	    {
	      b->l[i]=l1;
	      b->u[i]=u1;
	    }
	  else
	    {
	      if ((s2<s1)&&(s2<s3))
		{
		  b->l[i]=l2;
		  b->u[i]=u2;
		}
	      else
		{
		  b->l[i]=l3;
		  b->u[i]=u3;
		}
	    }
	}
      else
	{
	  if (b1->l[i]<b->l[i])
	    b->l[i]=b1->l[i];
	  
	  if (b1->u[i]>b->u[i])
	    b->u[i]=b1->u[i];
	}
    }
}
  
double CuikBoxVolume(boolean *used,TCuikBox *b)
{
  unsigned int i;
  double v;

  v=1.0;
  for(i=0;i<b->d;i++)
    {
      if ((used==NULL)||(used[i]))
	{
          #if (ANGULAR_SPACE)
            //v*=(T2A(b->u[i])-T2A(b->l[i]));
	    v*=TAD(b->u[i],b->l[i]);
           #else
	    v*=(b->u[i]-b->l[i]);
           #endif
	}
    }
  
  return(v);
}

double CuikBoxSize(boolean *used,TCuikBox *b)
{
  unsigned int i;
  double s,sd;

  s=0.0;
  for(i=0;i<b->d;i++)
    {
      if ((used==NULL)||(used[i]))
	{
          #if (ANGULAR_SPACE)
            //sd=T2A(b->u[i])-T2A(b->l[i]);
            sd=TAD(b->u[i],b->l[i]);
          #else 
	    sd=b->u[i]-b->l[i];
          #endif	
	  if (sd>s) s=sd;
	}
    }
      
  return(s);
}

void SplitCuikBox(unsigned int nv,double alpha,TCuikBox *b1,TCuikBox *b)
{
  double c;
  #if (ANGULAR_SPACE)
    double la,ua,ca;
  #endif

  /* Copy b into b1 */
  NEW(b1->l,2*b->d,double);
  b1->u=&(b1->l[b->d]);

  memcpy(b1->l,b->l,2*sizeof(double)*b->d);
  
  /* interpolation points */
  #if (ANGULAR_SPACE)
    la=T2A(b->l[nv]);
    ua=T2A(b->u[nv]);
    ca=la+(ua-la)*alpha;
    c=A2T(ca);
  #else
    c=b->l[nv]+(b->u[nv]-b->l[nv])*alpha;
  #endif
  
  /* Split the range */
  b1->u[nv]=c;
  b->l[nv]=c;
  
  b->level++;
  b1->level=b->level;
  b1->d=b->d;
  
  #if (_DEBUG>1)
    fprintf(stderr,"--------------------------------------\n");
    fprintf(stderr,"Splitting box [%u]: %f %f \n",nv,alpha,c);
    fprintf(stderr,"  b1: ");
    PrintCuikBox(stderr,b1);
    fprintf(stderr,"  b2: ");
    PrintCuikBox(stderr,b);
  #endif
}

void CuikBoxOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b)
{
  unsigned int i;
  
  for(i=0;i<b->d;i++)
    {
      if (((usedVars==NULL)||(usedVars[i]))&&(varType[i]==VAR_ROT))
	{
	  b->l[i]=OT(b->l[i],offset[i]);
	  b->u[i]=OT(b->u[i],offset[i]);
	}
    }
}

void CuikBoxUnOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b)
{
  unsigned int i;
  
  for(i=0;i<b->d;i++)
    {
      if (((usedVars==NULL)||(usedVars[i]))&&(varType[i]==VAR_ROT))
	{
	  b->l[i]=OT(b->l[i],-offset[i]);
	  b->u[i]=OT(b->u[i],-offset[i]);
	}
    }
}

unsigned int GetCuikBoxOffset(boolean *usedVars,unsigned int *varType,TCuikBox *b)
{
  unsigned int i,k,np;

  np=0;
  k=0;
  for(i=0;i<b->d;i++)
    {
      if ((usedVars==NULL)||(usedVars[i]))
	{
	  if (((b->l[i]<-1e-6)&&(b->u[i]>+1e-6))||
	      ((b->l[i]<+M_PI-1e-6)&&(b->u[i]>+M_PI+1e-6))||
	      ((b->l[i]<-M_PI-1e-6)&&(b->u[i]>-M_PI-1e-6)))
	    {
	      fprintf(stderr,"[%f %f]\n",b->l[i],b->u[i]);
	      Error("A box from two subproblems. A box resulting from clustering?");
	    }
	  else
	    {
	      if (varType[i]==VAR_ROT)
		{
		  if (b->l[i]>=0)
		    np+=(1<<k);
		  k++;
		}
	    }
	}
    }
  return(np);
}

void CuikBox2Angles(boolean *usedVars,unsigned int *varType,TCuikBox *b)
{
  unsigned int i;
  
  for(i=0;i<b->d;i++)
    {
      if (((usedVars==NULL)||(usedVars[i]))&&(varType[i]==VAR_ROT))
	{
	  b->l[i]=T2A(b->l[i]);
	  if (b->l[i]<-M_PI) b->l[i]=-M_PI;
	  
	  b->u[i]=T2A(b->u[i]);
	  if (b->u[i]>+M_PI) b->u[i]=+M_PI;
	}
    }
}

void CuikBox2Tangent(boolean *usedVars,unsigned int *varType,TCuikBox *b)
{
  unsigned int i;
  
  for(i=0;i<b->d;i++)
    {
      if (((usedVars==NULL)||(usedVars[i]))&&(varType[i]==VAR_ROT))
	{
	  b->l[i]=A2T(b->l[i]);
	  b->u[i]=A2T(b->u[i]);
	}
    }
}

void CuikBox2AnglesWithOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b)
{
  unsigned int i;
  
  for(i=0;i<b->d;i++)
    {
      if (((usedVars==NULL)||(usedVars[i]))&&(varType[i]==VAR_ROT))
	{
	  b->l[i]=OA(T2A(b->l[i]),offset[i]);
	  //if (b->l[i]<-M_PI) b->l[i]=-M_PI;
      
	  b->u[i]=OA(T2A(b->u[i]),offset[i]);
	  //if (b->u[i]>+M_PI) b->u[i]=+M_PI;
	}
    }
}

void CuikBox2TangentWithOffset(boolean *usedVars,unsigned int *varType,signed int *offset,TCuikBox *b)
{
  unsigned int i;
  
  for(i=0;i<b->d;i++)
    {
      if (((usedVars==NULL)||(usedVars[i]))&&(varType[i]==VAR_ROT))
	{
	  b->l[i]=A2T(OA(b->l[i],-offset[i]));
	  b->u[i]=A2T(OA(b->u[i],-offset[i]));
	}
    }
}

void PrintCuikBox(FILE *f,TCuikBox *b)
{
  unsigned int i;

  fprintf(f,"{ %u ",b->d);
  for(i=0;i<b->d;i++)
    fprintf(f,"[%.12f,%.12f] ",b->l[i],b->u[i]);
  fprintf(f,"}\n");
}

boolean ReadCuikBox(FILE *f,TCuikBox *b)
{
  int token=~EOF;
  unsigned int i;

  do token=fgetc(f); while ((token!=EOF)&&(token!='{')); /* skip everything till the beginning of the box */

  if (token!=EOF)
    {
      if (fscanf(f,"%u",&(b->d))!=1)
	token=EOF;
      else
	{
	  NEW(b->l,2*b->d,double);
	  b->u=&(b->l[b->d]);
	}
      
      for(i=0;((token!=EOF)&&(i<b->d));i++)
	{
	  do token=fgetc(f); while ((token!=EOF)&&(token!='['));
	  if (fscanf(f,"%lf",&(b->l[i]))!=1)
	    token=EOF;
	  else
	    {
	      /* We compensate for rounding error when saving/reading box bounds */
	      if (fabs(b->l[i]-M_PI)<1e-6) b->l[i]=+M_PI;
	      if (fabs(b->l[i]+M_PI)<1e-6) b->l[i]=-M_PI;
	      
	      do token=fgetc(f); while ((token!=EOF)&&(token!=','));
	      if (fscanf(f,"%lf",&(b->u[i]))!=1)
		token=EOF;
	      else
		{
		  /* We compensate for rounding error when saving/reading box bounds */
		  if (fabs(b->u[i]-M_PI)<1e-6) b->u[i]=+M_PI;
		  if (fabs(b->u[i]+M_PI)<1e-6) b->u[i]=-M_PI;
		}
	    }

	  //if (token==EOF)
	  //  Error("Unexpected EOF while reading a box");
	}

      if (token==EOF)
	{
	  free(b->l);
	  return(FALSE);
	}
      else
	{
	  b->level=1;
	  return(TRUE);
	}
    }
  else
    return(FALSE);
}

void DeleteCuikBox(TCuikBox *b)
{ 
  free(b->l);
}

/************************************************************************/
/************************************************************************/
/************************************************************************/
void ReadCuikBoxList(FILE *f,Tlist *l)
{
  boolean readOK;
  TCuikBox *b;
  
  InitList(sizeof(TCuikBox*),l);
  do {
    NEW(b,1,TCuikBox);
    readOK=ReadCuikBox(f,b);
    if (readOK)
      AddLastElement(&b,l);  
  } while(readOK);
}

void CopyCuikBoxList(Tlist *l_dst,Tlist *l_src)
{
  Titerator it;
  TCuikBox *b_dst,*b_src;
  
  InitList(sizeof(TCuikBox*),l_dst);
  
  InitIterator(&it,l_src);
  First(&it);
  while(!EndOfList(&it))
    {
      b_src=*(TCuikBox**)GetCurrent(&it);
      
      NEW(b_dst,1,TCuikBox);
      CopyCuikBox(b_dst,b_src);
      AddLastElement(&b_dst,l_dst); 
      
      Advance(&it);
    }
}

void SaveCuikBoxList(FILE *f,unsigned int k,
		     boolean angles,boolean offset,boolean error,boolean verified,
		     TCuikSystem *cs,Tlist *l)
{
  unsigned int i;
  Titerator it;
  TCuikBox *b;
  
  InitIterator(&it,l);
  First(&it);
  i=0;
  while(!EndOfList(&it))
    {
      b=*(TCuikBox**)GetCurrent(&it);
      if ((error)&&(cs!=NULL))
	fprintf(f,"%u err:%g ",k+i,ErrorInSolution(b,cs));
      else
	fprintf(f,"%u ",k+i);
      if (verified)
	fprintf(f,"<S> ");
      
      if ((offset)&&(cs!=NULL))
	{
	  if (angles)
	    CuikBox2AnglesWithOffset(cs->usedVars,cs->varType,cs->offset,b);
	  else
	    CuikBoxOffset(cs->usedVars,cs->varType,cs->offset,b);
	}
      else
	{
	  if ((angles)&&(cs!=NULL))
	    CuikBox2Angles(cs->usedVars,cs->varType,b);
	}
      
      PrintCuikBox(f,b);
      Advance(&it);
      i++;
    }
}

double CuikBoxListVolume(boolean *used,Tlist *l)
{
  double v;
  Titerator it;
  TCuikBox *b;

  v=0.0;
  InitIterator(&it,l);
  First(&it);
  while(!EndOfList(&it))
    {
      b=*(TCuikBox**)GetCurrent(&it);
      v+=CuikBoxVolume(used,b);
      Advance(&it);
    }
  return(v);
}

void DeleteCuikBoxList(Tlist *l)
{
  TCuikBox *b;
  
  while(!ListEmpty(l))
    {
      ExtractFirst(&b,l);
      DeleteCuikBox(b);
      free(b);
    }
}

void VerifyCuikBoxes(Tparameters *p,boolean angles,FILE *f_out,unsigned int ns,TCuikSystem *cs,Tlist *s)
{
  unsigned int verify;
  Titerator i;
  unsigned int c,nSols,np;
  TCuikBox *b;
  double *x;
      
  verify=(unsigned int)GetParameter(CT_VERIFY,p);
  
  if ((verify==VERIFY_NEWTON)||(verify==VERIFY_NEWTON_REPLACE))
    {
      NEW(x,cs->nv,double);
    }
	
  InitIterator(&i,s);
  First(&i);
  nSols=1;
  while(!EndOfList(&i))
    {
      b=*(TCuikBox**)GetCurrent(&i);
	
      np=GetCuikBoxOffset(cs[np].usedVars,cs[0].varType,b); 
      if (np>=ns)
	Error("Wrong offset in cuikverify");
	
      if (angles)
	CuikBox2TangentWithOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);
      else
	CuikBoxUnOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);

      c=VerifyBox(p,x,b,&(cs[np]));

      fprintf(f_out," %3u err:%g ",nSols,ErrorInSolution(b,&(cs[np])));
      if (c==REDUCED_WITH_SOLUTION)
	fprintf(f_out,"<S> ");

      if (angles)
	CuikBox2AnglesWithOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);
      else
	CuikBoxOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);
	    
      PrintCuikBox(f_out,b);
      nSols++;
	
      Advance(&i);
    }
    
  if ((verify==VERIFY_NEWTON)||(verify==VERIFY_NEWTON_REPLACE))
    free(x);
}

void CleanCuikBoxes(Tparameters *p,boolean angles,FILE *f_out,unsigned int ns,TCuikSystem *cs,Tlist *s)
{
  Titerator i;
  unsigned int c,nSols,nr,np,clean;
  TCuikBox *b;
  double epsilon;

  clean=(unsigned int)GetParameter(CT_CLEAN,p);
  
  if (clean==CLEAN_SIMPLEX)
    {
      #if (SIMPLEX_AVAILABLE)
        epsilon=GetParameter(CT_EPSILON,p);
	for(c=0;c<ns;c++)
	  InitSimplex(epsilon,&(cs[c]));
      #else	
	Error("CleanCuikBoxes requires the simplex");
      #endif
    	
      InitIterator(&i,s);
      First(&i);
      nSols=1;
      while(!EndOfList(&i))
	{
	  b=*(TCuikBox**)GetCurrent(&i);
	
	  np=GetCuikBoxOffset(cs[np].usedVars,cs[np].varType,b);
	  if (np>=ns)
	    Error("Wrong offset in cuikclean");
	
	  if (angles)
	    CuikBox2TangentWithOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);
	  else
	    CuikBoxUnOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);

	  c=ReduceBoxSimplex(p,b,&nr,&(cs[np]));

	  if (c!=EMPTY)
	    {
	      fprintf(f_out," %3u err:%g ",nSols,ErrorInSolution(b,&(cs[np])));
	      if (c==REDUCED_WITH_SOLUTION)
		fprintf(f_out,"<S> ");

	      if (angles)
		CuikBox2AnglesWithOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);
	      else
		CuikBoxOffset(cs[np].usedVars,cs[np].varType,cs[np].offset,b);
	    
	      PrintCuikBox(f_out,b);
	      nSols++;
	    }
	
	  Advance(&i);
	}
    
      for(c=0;c<ns;c++)
	DeleteSimplex(&(cs[c]));
    }
  else
    Error("Please select a clean method");
}

void ClusterCuikBoxes(Tlist *c,double tol,boolean angles,Tlist *s)
{
  unsigned int k,n,l,t,nb;
  Titerator i;
  Titerator j;
  Titerator ci;
  TCuikBox *b2,*b3,*cb;
  boolean modified;
  unsigned int *cluster,nc;

  /*Number of boxes to be clustered*/
  n=ListSize(s);

  fprintf(stderr,"Boxes to cluster: %u\n",n);
  
 /*Initially we have no cluster*/
  NEW(cluster,n,unsigned int);
  nc=0;
  for(k=0;k<n;k++)
    cluster[k]=(unsigned int)(-1);

  /*Initially the output is empty*/
  InitList(sizeof(TCuikBox*),c);
  
  /*Proceed for all boxes in the input list not yet clustered*/
  InitIterator(&i,s);
  First(&i);
  k=0;
  while(!EndOfList(&i))
    {
      if (cluster[k]==(unsigned int)(-1)) /*= not clustered*/
	{
	  /*Start a new cluster*/
	  nc++; /* Identifier of the current cluster */
	  nb=1; /* Number of boxes in the current cluster: just the one initiating the cluster */
	  cluster[k]=nc;

	  NEW(cb,1,TCuikBox);
	  CopyCuikBox(cb,*(TCuikBox**)GetCurrent(&i));
	  
	  do {

	    /*For all boxes still to be clustered*/
	    modified=FALSE; /*Initally we assume the cluster to remain as it is now*/

	    /*and we check for the rest of boxes in the input list whether or not they
	      can be included in the current cluster*/
	    InitIterator(&j,s);
	    First(&j);
	    l=0;
	    while(!EndOfList(&j))
	      {
		if (cluster[l]==(unsigned int)(-1)) /*the box in not yet assigned to any cluster*/
		  {
		    b2=*(TCuikBox **)GetCurrent(&j); /*this is the box candidate to joint the current cluster*/

		    /*Check if the non-clustered box has contact with one of the
		      boxes already in cluster 'nc'=cluster[k]*/
		    InitIterator(&ci,s);
		    First(&ci);
		    t=0;
		    while((cluster[l]!=nc)&&(!EndOfList(&ci)))
		      {
			if (cluster[t]==cluster[k])
			  {
			    b3=*(TCuikBox **)GetCurrent(&ci); /*This is a box already in cluster 'nc'*/

			    if (CuikBoxesIntersect(tol,angles,b2,b3))
			      {
				nb++;
				cluster[l]=nc; /*box b2 (number 'l') is included in the current cluster*/
				CuikBoxUnion(angles,b2,cb); /*Enlarge the cluster bounding box*/

				modified=TRUE; /*Indicate the current cluster has been modified*/
			      }
			  }
			Advance(&ci);
			t++;
		      }
		  }
		Advance(&j);
		l++;
	      }
	  } while(modified); /* If we added one or more boxes to the cluster we have to check whether
			        or not the cluster can be still extended*/

	  fprintf(stderr,"  Cluster %u: %u boxes\n",nc,nb);
	  AddLastElement(&cb,c); /*Add the bounding box of the current cluster to the output list*/
	}
      Advance(&i);
      k++;
    }

  free(cluster);
}

void CuikBoxes2Angles(boolean *usedVars,unsigned int *varType,Tlist *s)
{
  Titerator i;
  TCuikBox *b;
  
  InitIterator(&i,s);
  First(&i);
  while(!EndOfList(&i))
    {
      b=*(TCuikBox**)GetCurrent(&i);
      CuikBox2Angles(usedVars,varType,b); 
      Advance(&i);
    }
}

/************************************************************************/
/************************************************************************/
/************************************************************************/
#if (SIMPLEX_AVAILABLE)
void InitSimplex(double epsilon,TCuikSystem *cs)
{
  TSimplex *s;
  unsigned int i,nn;
    
  /*
    For each equation, the constraints to add to the simplex are
    
         \sum \lambda_i e_i = 0
      -x+\sum \lambda_i c_i = 0
         \sum lamda_i = 1  with lambda_i \in [0,1]

     where x is a vector of the variables in the equation (nve elements) and the 
     sums are for all the corners (nc=2^nve). c_i are the coordinates for each
     corner and e_i is the equation value at this corner.

     Thus we have 2+nve rows in the simplex tableau for each equation.

     With this the simples tableau has the following form (for two equations):

     [         e_1   e_1   e_3  ...  e_4                                 ] = 0
     [-1       c_1_1 c_1_2 c_1_3.... c_1_nc                              ] = 0 
     [   -1    c_2_2 c_2_2 c_2_3.... c_2_nc                              ] = 0
     [      -1 c_3_3 c_3_2 c_3_3.... c_3_nc                              ] = 0
     [         1     1     1         1                                   ] = 1
     [                                      e_1   e_1   e_3  ...  e_4    ] = 0
     [-1                                    c_1_1 c_1_2 c_1_c.... c_1_nc ] = 0
     [   -1                                 c_2_2 c_2_2 c_2_3.... c_2_nc ] = 0
     [       -1                             c_3_3 c_3_2 c_3_3.... c_3_nc ] = 0
     [                                      1     1     1         1      ] = 1
   */

  if (cs->ne==0)
    Error("No equations");
  
  s=&(cs->sp);
  
  /* Maximum number of columns = num vars in the problem + sum of number of lambdas per equation (=number of corners) */
  s->nCols=cs->nv; /* The first columns are the problem variables */
  for(i=0;i<cs->ne;i++)
    s->nCols+=cs->nc[i]; /* Then one block of lambdas for each equation */

  /* Limits for each column. For the initial block of variables changes in each
     use of the simplex. The rest (lambdas) are all [0,1] always 
     We set the limits for the maximum number of columns, although we may use less.
  */
  NEW(s->columnLower,s->nCols,double);
  NEW(s->columnUpper,s->nCols,double);
  NEW(s->obj,s->nCols,double);
  for(i=0;i<s->nCols;i++)
    {
      /* Set the bounds for lambdas (the ones for variables will change in SetSimplex) */
      s->columnLower[i]=0;
      s->columnUpper[i]=1;
      s->obj[i]=0; /* Only the obj for a given variable will change (one at a time) */
    }
 
   /* Maximum number of rows = sum of number of variables in each equation + 2
     the +2 per equation are the constraints for the equation values (e_1...) 
     and for the lambda (1 ....)*/
  s->nRows=0;
  for(i=0;i<cs->ne;i++)
    s->nRows+=(cs->nve[i]+2);

  /* Limits for each column. These are constant: 0 in general but for the \sum lamda equations, which is 1 */
  NEW(s->rowLower,s->nRows,double);
  NEW(s->rowUpper,s->nRows,double);

  /* Compute the maximum number of non-null entries in the matrix */
  nn=0;
  for(i=0;i<cs->ne;i++)
    nn+=cs->nve[i]*(1+cs->nc[i])+2*cs->nc[i];
  
  NEW(s->rowStarts,s->nRows+1,int); // index of columns/elements where a new row starts
  NEW(s->columns,nn,int);           // column for each element
  NEW(s->elements,nn,double);       // non-null elements in the tableau

  /* create the clp model. Will be deleted at the first call to SetSimplex */
  s->lp=Clp_newModel();
}

boolean SetSimplex(double epsilon,TCuikBox *b,TCuikSystem *cs)
{
  TSimplex *s;
  unsigned int i,j,k,k0,l,m,n,v,mask;
  double c,vf,avf,mavf,ctSmall;
  boolean ok;

  ok=TRUE;
  s=&(cs->sp);
  
  /* Delete the current model (initally an empty model) */
  Clp_deleteModel(cs->sp.lp);

  /* Re-build the model */
  s->lp=Clp_newModel();
    
  /* Set the verbosity level*/ 
  Clp_setLogLevel(s->lp,0);

  /* LP: Adjust LP internal parameters */
  Clp_scaling(s->lp,0);

  /* LP: Minimize */
  Clp_setOptimizationDirection(s->lp,1);
  
  /* These parameters depend on epsilon  */
  ctSmall=Clp_primalTolerance(s->lp);
  if (ctSmall>(epsilon*1e-3))
    Clp_setPrimalTolerance(s->lp,epsilon*1e-3);

  ctSmall=Clp_dualTolerance(s->lp);
  if (ctSmall>(epsilon*1e-3))
    Clp_setDualTolerance(s->lp,epsilon*1e-3);

  ctSmall=Clp_getSmallElementValue(s->lp);
  if (ctSmall>epsilon)
    Clp_setSmallElementValue(s->lp,epsilon);
    
  /* LP: Timeout in seconds*/
  Clp_setMaximumSeconds(s->lp,SIMPLEX_TIMEOUT);

  /* Define the new (non-null) elements in the tableau */
  m=cs->nv; /* column index where the block of lambdas for each equation starts */
  
  k=0;               /* index in the column/element arrays (arrays with the non-null elements) */
  l=0;               /* The first row */
  s->rowStarts[l]=k; /* First row starts at 0 */
  
  /* The function equation */

  for(i=0;i<cs->ne;i++)
    {
      k0=k;
      mavf=0.0;
      for(n=0;n<cs->nc[i];n++)
	{
	  vf=cs->f[i][n];
	  avf=fabs(vf);
	  if (avf>mavf) mavf=avf;
	  if (avf>ctSmall)
	    {
	      s->columns[k]=m+n;
	      s->elements[k]=vf;
	      k++;
	    }
	}
      
      if (mavf<ctSmall) // i.e., if the function values are not all 0 !
	{
	  k=k0; // reset the 'k' counter
	}
      else
	{
	  // Scale function values
	  if (mavf<1)
	    {
	      for(n=k0;n<k;n++)
	        s->elements[n]/=mavf;
	    }
	  
	  /* One more row */
	  s->rowLower[l]=0.0; /* f=0 */
	  s->rowUpper[l]=0.0;
	  l++;
	  s->rowStarts[l]=k; /* <- Here is where this row end (next row starts) */
	  	      
	  for(j=0;j<cs->nve[i];j++) /* first block of rows for the equation */
	    {
      	      /* The current variable in the equation (global index) */
	      v=cs->ve[i][j];
	  
	      /* this is the -x_i (coefficient -1)  */
	      s->columns[k]=v;
	      s->elements[k]=-1.0; 
	      k++; /* just added one element */

	      mask=(1<<j);  /* variable mask */
	      for(n=0;n<cs->nc[i];n++)
		{
		  if (n&mask)
		    c=b->u[v]; /* Upper bound of variable v */
		  else
		    c=b->l[v]; /* Lower bound of variable v */

		  if (fabs(c)>ctSmall)
		    {
		      s->columns[k]=m+n;
		      s->elements[k]=c; /* These will be the corners */
		      k++;
		    }
		}
	  
	      /* One more row */
	      s->rowLower[l]=0.0; 
	      s->rowUpper[l]=0.0;
	      l++;
	      s->rowStarts[l]=k; /* <- Here is where this row end (next row starts) */
	    }

	  /* The equations with \sum lamda = 1 */
	  for(n=0;n<cs->nc[i];n++)
	    {
	      s->columns[k]=m+n;
	      s->elements[k]=1.0; /* These will be the function values */
	      k++;
	    }
	  
	  /* The last row of this equation */
	  s->rowLower[l]=1.0;
	  s->rowUpper[l]=1.0;
	  l++;
	  s->rowStarts[l]=k; /* <- Here is where this row end (next row starts) */
      	  
	  m+=cs->nc[i]; /* add nc[i] to the number of rows to skip to find the lambda for the next equation */
	}
    }

  ok=(k>0);
  
  if (ok) // If we added at least one equation to the simplex
    {
      /*Set the number of columns*/
      Clp_resize(s->lp,0,m); /* rows will be added next */
  
      /* Add the new rows (including the bounds) */
      Clp_addRows(s->lp,l,s->rowLower,s->rowUpper,s->rowStarts,s->columns,s->elements);
  
      /* Change the column bounds: only the part of the variables changes */
      memcpy(s->columnLower,b->l,cs->nv*sizeof(double));
      memcpy(s->columnUpper,b->u,cs->nv*sizeof(double));
      Clp_chgColumnLower(s->lp,s->columnLower);
      Clp_chgColumnUpper(s->lp,s->columnUpper);
  
      #if (_DEBUG>3) 
      {
	unsigned int i;
	int j,k;

	fprintf(stderr,"\nSimplex tableau [%u %u]:\n",l,m);
	fprintf(stderr,"Col   :");
	for(i=0;i<m;i++)
	  fprintf(stderr,"%5u ",i);
	fprintf(stderr,"\n");
	fprintf(stderr,"Min   :");
	for(i=0;i<m;i++)
	  fprintf(stderr,"%4.2f ",s->columnLower[i]);
	fprintf(stderr,"\n");
	fprintf(stderr,"Max   :");
	for(i=0;i<m;i++)
	  fprintf(stderr,"%4.2f ",s->columnUpper[i]);
	fprintf(stderr,"\n");
      
	for(i=0;i<l;i++)
	  {
	    fprintf(stderr,"Row %3u: ",i);
	    k=0;
	    for(j=s->rowStarts[i];j<s->rowStarts[i+1];j++)
	      {
		while(k<s->columns[j])
		  {
		    fprintf(stderr,"      ");
		    k++;
		  }
		fprintf(stderr,"%+4.2f ",s->elements[j]);
		k++;
	      }
	    while(k<(signed int)m)
	      {
		fprintf(stderr,"      ");
		k++;
	      }
	    fprintf(stderr," = [%+4.2f,%+4.2f]\n",s->rowLower[i],s->rowUpper[i]);
	  }
      }
      #endif
    }
 
  return(ok);
}

void DeleteSimplex(TCuikSystem *cs)
{
  if (cs->sp.nRows>0)
    {      
      Clp_deleteModel(cs->sp.lp);
      free(cs->sp.rowLower);
      free(cs->sp.rowUpper);
      free(cs->sp.columnLower);
      free(cs->sp.columnUpper);
      free(cs->sp.obj);
      free(cs->sp.rowStarts);
      free(cs->sp.columns);
      free(cs->sp.elements);
      cs->sp.nRows=0;
    }
}

#endif
/************************************************************************/
/************************************************************************/
/************************************************************************/

double EvaluateCSEquation(unsigned int i,double *x,TCuikSystem *cs)
{
  unsigned int start,step,j,k,nc,nve,n,dl;
  double a,*f,*l0,*u0,v;

  f=cs->f[i];
  nc=cs->nc[i];
  memcpy(f,cs->f0[i],sizeof(double)*nc);

  l0=cs->b0->l;
  u0=cs->b0->u;

  nve=cs->nve[i];
  for(j=0;j<nve;j++)
    {
      // Global index of the  j-th variable in the i-th equation
      n=cs->ve[i][j];
      
      // Interpolation factor for 'c'
      a=(x[n]-u0[n])/(u0[n]-l0[n]);
      
      step=(1<<(j+1)); // Distance between the values to compute
      start=step-1;    // First value to compute
      dl=(1<<j);       // Distance to low value
      for(k=start;k<nc;k+=step)
	f[k]+=a*(f[k]-f[k-dl]);
    }
  v=f[nc-1];
  
  return(v);
}

inline void InterpolateFunction(unsigned int nv,
				double l,double u,double newL,double newU,
				unsigned int nc,double *f,TCuikSystem *cs)
{
  register unsigned int k;
  register double a1,b2,d,d2,*fk,*fo;
  register unsigned int o,mask,omask;
  
  /* if the new box 'b' is different from b0 along dimension 'i' */
  d=(u-l);
  a1=(newL-l)/d;
  b2=(newU-u)/d;
  
  mask=(1<<nv);
  omask=(~mask);
  for(k=mask,fk=f+mask;k<nc;k++,fk++)
    {
      if (k&mask) 
	{
	  /* 'k' is an index of 'f' corresponding to an upper limit
	     of variable 'j' (with the other variables being at lower/upper
	     limit).
	     'o' is the index corresponding to an lower limit, but
	     with the same lower/upper selections for the rest of
	     variables */
	  o=((k)&(omask));

	  fo=f+o;
	  
	  /* with the rest of variables constant, the evaluation 
	     is a linear interpolation between the already known values */
	  d2=*fk-*fo;
	  *fo+=a1*d2; /* Update lower */
	  *fk+=b2*d2; /* Update upper */

	  #if (_DEBUG>4)
	    fprintf(stderr,"        f[%u]=%f\n",o,f[o]);
	    fprintf(stderr,"        f[%u]=%f\n",k,f[k]);
	  #endif
	}
    }
}

void InterpolateFunctionFromInitialBox(unsigned int ne,double *l,double *u,double *f,TCuikSystem *cs)
{
  unsigned int j,v,nve,nc;
  double *l0,*u0;

  #if (_DEBUG>3)
    fprintf(stderr,"Interpolation from unit box: eq %u -------------------------\n",ne);
  #endif
  memcpy(f,cs->f0[ne],sizeof(double)*cs->nc[ne]);

  l0=cs->b0->l;
  u0=cs->b0->u;

  nc=cs->nc[ne];
  nve=cs->nve[ne];
  for(j=0;j<nve;j++)
    {
      v=cs->ve[ne][j];

      InterpolateFunction(j,l0[v],u0[v],l[v],u[v],nc,f,cs);

      #if (_DEBUG>3)
      {
	unsigned int k;
	  
	fprintf(stderr,"      v:%u [%f %f]:",j,l[v],u[v]);
	for(k=0;k<cs->nc[ne];k++)
	  fprintf(stderr," %f",f[k]);
	fprintf(stderr,"\n");
      }
      #endif
    }
}

void ComputeTrapezoid(double epsilon,unsigned int nf,double *f,
		      unsigned int v,double *ll,double *lu,double *ul,double *uu,
		      TCuikSystem *cs)
{
  register unsigned int k,mask;
  register double *val;
	    
  /* Compute the bounds */
  mask=(1<<v);
  *ll=f[0];
  *lu=*ll;
  *ul=f[mask];
  *uu=*ul;
  for(k=1,val=f+1;k<nf;k++,val++)
    {
      if (k&mask)
	{
	  /* upper extreme of variable 'j' */
	  if (*val<*ul) *ul=*val;
	  else
	    {
	      if (*val>*uu) *uu=*val;
	    }
	}
      else
	{
	  /* lower extreme of variable 'j' */
	  if (*val<*ll) *ll=*val;
	  else
	    {
	      if (*val>*lu) *lu=*val;
	    }
	}
    }
  
  /* Compensate numerical errors in function interpolation */
  *ll-=epsilon;
  *lu+=epsilon;
  *ul-=epsilon;
  *uu+=epsilon;
}

unsigned int TrapezoidClipping(double epsilon,
			       double l,double u,double *newL,double *newU,
			       double ll,double lu,double ul,double uu)
{
  unsigned int out,nr;
  double d,l1,u1;
  
  if ((ll>lu)||(ul>uu)||((ll>0)&&(ul>0))||((lu<0)&&(uu<0)))
    out=EMPTY; /* empty box */
  else
    {
      d=u-l;
 
      if ((ll<=0)&&(ul>=0)&&((_DEBUG==0)||(fabs(ll-ul)>epsilon)))
	{	
	  u1=l+ll*d/(ll-ul);
	  if (u1<*newU) *newU=u1;
	}
		
      if ((lu>=0)&&(uu<=0)&&((_DEBUG==0)||(fabs(lu-uu)>epsilon)))
	{
	  u1=l+lu*d/(lu-uu);
	  if (u1<*newU) *newU=u1;
	}

      if ((ll>=0)&&(ul<=0)&&((_DEBUG==0)||(fabs(ll-ul)>epsilon)))
	{
	  l1=l+ll*d/(ll-ul);
	  if (l1>*newL) *newL=l1;
	}
	  
		
      if ((lu<=0)&&(uu>=0)&&((_DEBUG==0)||(fabs(lu-uu)>epsilon)))
	{
	  l1=l+lu*d/(lu-uu);
	  if (l1>*newL) *newL=l1;
	}
	  
	
      nr=2; /* assume that both lower and upper limts changed */

      /* if if any of the chages is too small -> revert it */
      if (*newL<l+epsilon)
	{
	  *newL=l;
	  nr--;
	}
		
      if (*newU>u-epsilon)
	{
	  *newU=u;
	  nr--;
	}
      
      /* if at least one limit significantly changed ... */
      switch(nr)
	{
	case 2:
	  out=CONTRACTED;
	  break;
	case 1:
	  out=REDUCED;
	  break;
	case 0:
	  out=NOT_REDUCED;
	  break;
	default:
	  Error("Wrong number of reduced margins in TrapezoidClipping");
	}	  
    }

  return(out);
}

unsigned int CheckSolution(Tparameters *p,TCuikBox *b,TCuikSystem *cs)
{
  double epsilon;
  double ll,lu,ul,uu;
  unsigned int i,j,out;
  boolean mirandaTest;
  
  epsilon=GetParameter(CT_EPSILON,p);

  out=REDUCED;
  mirandaTest=TRUE;

  for(i=0;((i<cs->ne)&&(out!=EMPTY));i++)
    {
      #if (_DEBUG>1)
        fprintf(stderr,"\nChecking equation: %u\n",i);
      #endif
	
      InterpolateFunctionFromInitialBox(i,b->l,b->u,cs->f[i],cs);

      for(j=0;((j<cs->nve[i])&&(out!=EMPTY));j++)
	{ 
	  ComputeTrapezoid(epsilon,cs->nc[i],cs->f[i],j,&ll,&lu,&ul,&uu,cs);
	  
	  if (((lu<0)&&(uu<0))||((ll>0)&&(ul>0)))
	    out=EMPTY;
	  else
	    mirandaTest=((mirandaTest)&&(((lu<0)&&(ul>0))||((ll>0)&&(uu<0))));
	  
	  #if (_DEBUG>1)
	    fprintf(stderr,"  Variable: %u [%u]\n",j,cs->ve[i][j]);
	    fprintf(stderr,"    Trapezoid: [%f %f] [%f %f]\n",ll,lu,ul,uu);
	    if (out==EMPTY)
	      fprintf(stderr,"    Not valid\n");
	  #endif
	}
    }

  if ((out!=EMPTY)&&(mirandaTest))
    out=REDUCED_WITH_SOLUTION;
  
  return(out);
}

unsigned int VerifyBox(Tparameters *p,double *x,TCuikBox *b,TCuikSystem *cs)
{
  unsigned int verify;
  double epsilon;
  unsigned int i,j,v,c;
  double ll,lu,ul,uu;
  boolean mirandaTest;
  
  verify=(unsigned int)GetParameter(CT_VERIFY,p);
  epsilon=GetParameter(CT_EPSILON,p);
  
  switch(verify)
    {
    case VERIFY_MIRANDA:
      mirandaTest=TRUE;

      for(i=0;((i<cs->ne)&&(mirandaTest));i++)
	{
	  /* Evaluate the equation for all corners */
	  InterpolateFunctionFromInitialBox(i,b->l,b->u,cs->f[i],cs);

	  /* For all variables, compute min/max for the lower/upper bounds */
	  for(j=0;((j<cs->nve[i])&&(mirandaTest));j++)
	    {
	      v=cs->ve[i][j];

	      /* Do not consider constant ranges */
	      if ((b->u[v]-b->l[v])>0)
		{   
		  /* Compute the bounds */
		  ComputeTrapezoid(epsilon,cs->nc[i],cs->f[i],j,&ll,&lu,&ul,&uu,cs);
		
		  mirandaTest=((mirandaTest)&&(((lu<0)&&(ul>0))||((ll>0)&&(uu<0))));
		}
	    }
	}
      if (mirandaTest)
	c=REDUCED_WITH_SOLUTION;
      else
	c=NOT_REDUCED;
      break;
      
    case VERIFY_NEWTON:
    case VERIFY_NEWTON_REPLACE:
      /* Newton-based verification */
      CuikBoxCenter(x,b);
  
      if ((Newton(p,b,x,cs)<epsilon)&&(PointInCuikBox(x,b)))
	{
	  c=REDUCED_WITH_SOLUTION;
			  
	  if (verify==VERIFY_NEWTON_REPLACE)
	    {
	      unsigned int i;
				  
	      for(i=0;i<cs->nv;i++)
		{
		  b->l[i]=x[i]-epsilon;
		  b->u[i]=x[i]+epsilon;
		}
	    }
	}
      else
	c=NOT_REDUCED;
      break;
      
    default:
      Error("Unknown verification method");
    }

  return(c);
}

/*
  This is the basic function of the solver. This function includes the
  ShrinkBox procedure described in our papers plus a loop interating
  ShrinkBox as far as the box is significantly reduced (and still not empty).

  Thus, ReduceBox takes an input box 'b' and reduces it as much as possible by removing
  parts that do not include any solution point.
  It returns information about what happened in the box shrinking process:

  - CONTRACTED
  - REDUCED
  - REDUCED_WITH_SOLUTION
  - NOT_REDUCED
  - EMPTY
  .
*/

unsigned int ReduceBox(Tparameters *p,TCuikBox *b,unsigned int *nRed,TCuikSystem *cs)
{
  /*The box can only be reduced if we have constraints*/
  unsigned int i,j,v,out,outV,nc,nnr;
  boolean hasSolution,mirandaTest;
  double rho,sigma,epsilon,v1,v2,s,ll,lu,ul,uu;
    
  rho=GetParameter(CT_RHO,p);
  sigma=GetParameter(CT_SIGMA,p);
  epsilon=GetParameter(CT_EPSILON,p);
      
  out=NOT_REDUCED;
	
  hasSolution=FALSE;

  *nRed=0;
  
  v2=CuikBoxVolume(cs->usedVars,b);
  do {
    (*nRed)++;
    
    #if (_DEBUG>0)
      fprintf(stderr,".");
    #endif
    v1=v2;
    
    #if (_DEBUG>1)
      fprintf(stderr,"-------------------------------------------------------\n");
      fprintf(stderr,"Reducing box: ");
      PrintCuikBox(stderr,b);
    #endif

    nc=0; /* Number of contracted ranges */
    nnr=0;/* Number of non-null ranges */
    for(j=0;j<cs->nv;j++)
      {
	cs->outG[j]=NOT_REDUCED;
	if ((b->u[j]-b->l[j])>0)
	  nnr++;
      }
    
    mirandaTest=TRUE;

    for(i=0;((i<cs->ne)&&(out!=EMPTY));i++)
      {
	#if (_DEBUG>1)
	  fprintf(stderr,"-------------------------------------------------------\n");
	  fprintf(stderr,"  Reducing with equation %u (%u corners)\n",i,cs->nc[i]);
	#endif

	/* Keep the box ranges used to compute the function. */
	memcpy(cs->boxL,b->l,cs->nv*sizeof(double));
	memcpy(cs->boxU,b->u,cs->nv*sizeof(double));
	  
	/* Evaluate the equation for all corners */
	InterpolateFunctionFromInitialBox(i,b->l,b->u,cs->f[i],cs);

	#if (_DEBUG>1)
	{
	  double *fv;
	  fv=cs->f[i];
	    
	  fprintf(stderr,"    f_%u=[",i);
	  for(j=0;j<cs->nc[i];j++)
	    fprintf(stderr," %f",(double)fv[j]);
	  fprintf(stderr,"]\n");
	}
        #endif

	/* For all variables, compute min/max for the lower/upper bounds */
	for(j=0;((j<cs->nve[i])&&(out!=EMPTY));j++)
	  {
	    v=cs->ve[i][j];

	    /* Do not reduce constant ranges */
	    if ((b->u[v]-b->l[v])>0)
	      {
		/* Compute the bounds */
		ComputeTrapezoid(epsilon,cs->nc[i],cs->f[i],j,&ll,&lu,&ul,&uu,cs);
		
		mirandaTest=((mirandaTest)&&(((lu<0)&&(ul>0))||((ll>0)&&(uu<0))));

		#if (_DEBUG>1)
		  fprintf(stderr,"    Reducing range %u [v:%u]\n",j,v);
		  fprintf(stderr,"      Input range: [%f %f]\n",cs->boxL[v],cs->boxU[v]);
		  fprintf(stderr,"      Trapezoid:\n");
		  fprintf(stderr,"         l:[%f %f]\n",ll,lu);
		  fprintf(stderr,"         u:[%f %f]\n",ul,uu);
		#endif
	    
		/* Trapezoid clipping */
		outV=TrapezoidClipping(epsilon,cs->boxL[v],cs->boxU[v],&(b->l[v]),&(b->u[v]),ll,lu,ul,uu);
	    
		switch(outV)
		  {
		  case EMPTY:
		    /* if out==REDUCED_WITH_SOLUTION -> something is wrong */
		    out=EMPTY; /* Stop the whole process */
		    #if (_DEBUG>1)
		      fprintf(stderr,"      Empty\n");
		    #endif
		    break;
		  case NOT_REDUCED: /* Nothing to be done */
		    #if (_DEBUG>1)
		      fprintf(stderr,"      Not reduced\n");
		    #endif
		    break;
		  case CONTRACTED: /* The best possible outcome */
		    if (cs->outG[v]!=CONTRACTED)
		      {
			cs->outG[v]=CONTRACTED;
			nc++;
		      }
		    /* DO NOT BREAK! */
		  case REDUCED:  /* This is great, although not perfect */
		    /* if at any reduction the box is known to have a solution, keep the information */
		    if (out!=REDUCED_WITH_SOLUTION) /*  -> out==REDUCED or out==NOT_REDUCED */
		      out=REDUCED;
		
		    #if (_DEBUG>1)
		      fprintf(stderr,"      Reduced range: [%f %f]\n",b->l[v],b->u[v]);
		    #endif
		    break;
		  default:
		    Error("      Unknow trapezoid clipping outcome\n");
		  }
	      }
	  }
      }
    
    if (out!=EMPTY) /* REDUCED or CONTRACTED or NOT_REDUCED */
      {
	hasSolution=(nc==nnr);
	
	if ((hasSolution)||(mirandaTest))
	  {
	    out=REDUCED_WITH_SOLUTION;
	    #if (_DEBUG>1)
	      fprintf(stderr,"Reduced with solution!!\n");
	    #endif
	  }

	v2=CuikBoxVolume(cs->usedVars,b);
	s=CuikBoxSize(cs->usedVars,b);
	
	#if (_DEBUG>1)
	  fprintf(stderr,"--------------------------------------\n");
	  fprintf(stderr,"New box: v:%g [%g]  s:%g [%g] -> ",v1,v2,s,sigma);
	  PrintCuikBox(stderr,b);
	#endif
      }
  } while((out!=EMPTY)&&(v2/v1<rho)&&(s>sigma));
  
  return(out);
}

#if (SIMPLEX_AVAILABLE)
  unsigned int ReduceBoxSimplex(Tparameters *p,TCuikBox *b,unsigned int *nRed,TCuikSystem *cs)
  {
    unsigned int i,out,nr;
    double rho,sigma,epsilon,v1,v2,s,v;
    TSimplex *sp;
    boolean simplexOK;

    sp=&(cs->sp);
    rho=GetParameter(CT_RHO,p);
    sigma=GetParameter(CT_SIGMA,p);
    epsilon=GetParameter(CT_EPSILON,p);
  
    out=NOT_REDUCED;
    
    *nRed=0;

    v2=CuikBoxVolume(cs->usedVars,b);
    do {
      (*nRed)++;
      
      #if (_DEBUG>0)
        fprintf(stderr,".");
      #endif
      v1=v2;
      
      #if (_DEBUG>1)
        fprintf(stderr,"-------------------------------------------------------\n");
	fprintf(stderr,"Reducing box (simplex): v:%g  s:%g  -> ",v1,CuikBoxSize(cs->usedVars,b));
	PrintCuikBox(stderr,b);
      #endif
      
      /* Evaluate all functions */
      /* As an alternative  we  may evaluate the functions once and update the if any
         bound changes */
	
      for(i=0;i<cs->ne;i++)
	{
	  InterpolateFunctionFromInitialBox(i,b->l,b->u,cs->f[i],cs);
	    
	  #if (_DEBUG>1)
	  {
	    unsigned int j;
	    double *fv;
	    
	    fv=cs->f[i];

	    fprintf(stderr,"    f_%u=[",i);
	    for(j=0;j<cs->nc[i];j++)
	      fprintf(stderr," %f",(double)fv[j]);
	    fprintf(stderr,"]\n");
	  }
          #endif
	}
      
      /* Set the simplex */
      simplexOK=SetSimplex(epsilon,b,cs);
      
      /* Find the new bounds for each variable */
      nr=0;
      for(i=0;((i<cs->nv)&&(out!=EMPTY)&&(simplexOK));i++)
	{
	  /* Minimize for var i */
	  if ((b->u[i]-b->l[i])>0)
	    {
	      #if (_DEBUG>1)
	        fprintf(stderr,"    Reducing range %u\n",i);
	        fprintf(stderr,"      Input range: [%f %f]\n",b->l[i],b->u[i]);
	      #endif
	  
	      sp->obj[i]=1.0;
	      Clp_chgObjCoefficients(sp->lp,sp->obj);
	      Clp_primal(sp->lp,1); /* 1 => something changed in the problem!*/
	      switch (Clp_status(sp->lp))
		{
		case 0: /* optimal */
		  v=Clp_objectiveValue(sp->lp)-epsilon;
		  if (v>b->u[i])
		    {
		      out=EMPTY;
		      #if (_DEBUG>1)
		        fprintf(stderr,"      Emtpy v:%g [%g %g]\n",v,b->l[i],b->u[i]);
		      #endif
		    }
		  else
		    {
		      if (v>b->l[i])
			{
			  if (out!=REDUCED_WITH_SOLUTION)
			    out=REDUCED;
			  b->l[i]=v;
			  nr++;
			  //sp->columnLower[i]=v;
			  //Clp_chgColumnLower(sp->lp,sp->columnLower);
			  #if (_DEBUG>1)
			    fprintf(stderr,"      Improved lower bound: %f\n",b->l[i]);
			  #endif
			}
		    }
		  break;
		case 1: /* primal infeasible*/
		  out=EMPTY;
		  #if (_DEBUG>1)
		    fprintf(stderr,"      Emtpy\n");
		  #endif
		  break;
		#if (_DEBUG>1)
		  /* Other possible outputs are errors in the simplex and, thus
		     we do not use the simplex output (keep bounds) */
		default:
		  fprintf(stderr,"      LP status %u\n",Clp_status(sp->lp));
		#endif
		}

	      if (out!=EMPTY)
		{
		  /* Maximize for var i */
		  sp->obj[i]=-1.0;
		  Clp_chgObjCoefficients(sp->lp,sp->obj);
		  Clp_primal(sp->lp,0); /* 1 => something changed in the problem!*/
		  switch (Clp_status(sp->lp))
		    {
		    case 0: /* optimal */
		      v=-Clp_objectiveValue(sp->lp)+epsilon; /* - since the objective is also negative */
		      if (v<b->l[i])
			{
			  out=EMPTY;
			  #if (_DEBUG>1)
			    fprintf(stderr,"      Emtpy v:%g [%g %g]\n",v,b->l[i],b->u[i]);
			  #endif
			}
		      else
			{
			  if (v<b->u[i])
			    {
			      if (out!=REDUCED_WITH_SOLUTION)
				out=REDUCED;
			      b->u[i]=v;
			      nr++;
			      //sp->columnUpper[i]=v;
			      //Clp_chgColumnUpper(sp->lp,sp->columnUpper);
			      #if (_DEBUG>1)
			        fprintf(stderr,"      Improved upper bound: %f\n",b->u[i]);
			      #endif
			    }
			}
		      break;
		    case 1: /* primal infeasible*/
		      out=EMPTY;
		      #if (_DEBUG>1)
		        fprintf(stderr,"      Emtpy\n");
		      #endif
		      break;
		    #if (_DEBUG>1)
		      /* Other possible outputs are errors in the simplex and, thus
			 we do not use the simplex output (keep bounds) */
		    default:
		      fprintf(stderr,"      LP status %u\n",Clp_status(sp->lp));
		    #endif
		    }
		}
	  
	      /* Reset var objective for i */
	      sp->obj[i]=0.0;
	    }
	}
      
      /* Final consideration  */
      if ((out!=EMPTY)&&(nr>0))
	{ 
	  v2=CuikBoxVolume(cs->usedVars,b);
	  s=CuikBoxSize(cs->usedVars,b);
	  #if (_DEBUG>1)
	    fprintf(stderr,"--------------------------------------\n");
	    fprintf(stderr,"New box: v:%g [%g]  s:%g [%g] -> ",v2,v1,s,sigma);
	    PrintCuikBox(stderr,b);
	  #endif
	}
      
    } while((out!=EMPTY)&&(v2/v1<rho)&&(s>sigma));

    return(out);
  }
#endif


void ComputeJacobian(double *x,double *veq,double *J,TCuikSystem *cs)
{
  unsigned int i,j,k,nv;
  double v;
  
  /*
    Set the Jacobian to 0 (we will only fill the non-null entries)
    For fully dense systems (where all equations involve all variables this
    could be skip
  */
  memset(J,0,sizeof(double)*cs->ne*cs->nv);

  if (veq==NULL)
    {
      veq=cs->veq;
      EvaluateCSEquations(x,veq,cs);
    }
  
  // For all equations 
  for(i=0;i<cs->ne;i++)
    {
      // For all variables in the equation
      for(j=0;j<cs->nve[i];j++)
	{
	  // Global index of the j-th variable in the i-th equation
	  nv=cs->ve[i][j];
	  
	  // Assume that J is stored columnwise (the standard in LAPACK/BLAS)
	  k=nv*cs->ne+i;

	  // Compute the derivative from the global box (control points)
	  v=x[nv];
	  x[nv]+=1;
	  J[k]=EvaluateCSEquation(i,x,cs)-veq[i];
	  x[nv]=v;
	}
    }
}

double Newton(Tparameters *p,TCuikBox *b,double *x,TCuikSystem *cs)
{
  double epsilon,n,nullSingularValue;
  int info,one=1; /* one right hand vector */
  unsigned int it;
  int rank;

  if (cs->nv!=cs->ne)
    { 
      nullSingularValue=1e-10;
    }
  
  epsilon=GetParameter(CT_EPSILON,p);
  
  EvaluateCSEquations(x,cs->veq,cs);
  n=Norm(cs->ne,cs->veq);
  
  it=0;
  info=0;
  //PrintCuikBox(stderr,b);
  //fprintf(stderr,"NE_0: %g [%g]\n",n,epsilon);
  //PrintVector(stderr,"  x0",cs->nv,x);
  while ((info==0)&&(n>=epsilon)&&(it<10))
    {
      ComputeJacobian(x,cs->veq,cs->J,cs);

      //PrintMatrix(stderr,"J",cs->ne,cs->nv,cs->J);
      //PrintVector(stderr,"v",cs->ne,cs->veq);
      
      if (cs->nv==cs->ne)
	{
	  /* Squared system */
          #ifdef _LAPACKE
	    info=(int)LAPACKE_dgesv_work(LAPACK_COL_MAJOR,
					 (lapack_int)cs->ne,one,cs->J,(lapack_int)cs->ne,
					 (lapack_int*)cs->perm,cs->veq,(lapack_int)cs->nv);
          #else
	    dgesv_((int*)&cs->ne,&one,cs->J,(int*)&cs->ne,cs->perm,cs->veq,(int*)&cs->nv,&info);
          #endif
	}
      else
	{
	  /* Non-squared system -> Minimum norm solution */
          #ifdef _LAPACKE
	    info=(int)LAPACKE_dgelss_work(LAPACK_COL_MAJOR,
					  (lapack_int)cs->ne,(lapack_int)cs->nv,one,cs->J,
					  (lapack_int)cs->ne,cs->veq,(lapack_int)cs->maxEV,
					  cs->sing,nullSingularValue,(lapack_int*)&rank,cs->work,cs->lwork);
	  #else
	    dgelss_((int*)&cs->ne,(int*)&cs->nv,&one,cs->J,(int*)&cs->ne,cs->veq,(int*)&cs->maxEV,cs->sing,
		    &nullSingularValue,&rank,cs->work,&cs->lwork,&info);
	  #endif
	}
      
      if (info==0)
	{
	  SubtractVector(cs->nv,x,cs->veq);
	  
	  //PrintVector(stderr,"d",cs->nv,cs->veq);
	  //PrintVector(stderr,"  x",cs->nv,x);
	  
	  EvaluateCSEquations(x,cs->veq,cs);
	  n=Norm(cs->ne,cs->veq);
	  it++;
	  //fprintf(stderr,"NE_%u: %g\n",it,n);
	}
    }

  return(n);
}

/************************************************************************/
/************************************************************************/
/************************************************************************/

/*
  Warms up the cuiksystem structure
*/
void InitCuikSystem(TCuikSystem *cs)
{
  cs->ne=0;
  cs->nv=0;
  cs->nve=NULL;
  cs->ve=NULL;
  cs->nc=NULL;
  cs->b0=NULL;

  cs->f0=NULL;
  
  cs->f=NULL;
    
  cs->outG=NULL;
  cs->boxL=NULL;
  cs->boxU=NULL;

  cs->maxEV=0;
  cs->minEV=0;
  cs->veq=NULL;
  cs->J=NULL;
  cs->perm=NULL;
  cs->sing=NULL;
  cs->lwork=-1;
  cs->work=NULL;
}

void Define6RProblem(Tparameters *p,unsigned int np,unsigned int tp,
		     double *a,double *d,double *alpha,double *R,double *t,
		     TCuikSystem *cs)
{
  unsigned int i,j,mc;
  double q[8],qf[8],qp[8],theta,b,ta,of,a_2,d_2;
  
  cs->nv=6;
  
  NEW(cs->varType,cs->nv,unsigned int);
  for(i=0;i<cs->nv;i++)
    cs->varType[i]=VAR_ROT;
    
  NEW(cs->offset,cs->nv,signed int);
  if (tp==1) /* just a single problem with large ranges -> no offset */
    {
      for(i=0;i<cs->nv;i++)
	cs->offset[i]=0;
    }
  else
    {
      for(i=0;i<cs->nv;i++)
	{
	  if ((np)&(1<<i))
	    cs->offset[i]=+1;
	  else
	    cs->offset[i]=-1;
	}
    }
  
  cs->ne=cs->nv+REDUNDANT_6R;

  NEW(cs->usedVars,cs->nv,boolean);
  for(i=0;i<cs->nv;i++)
    cs->usedVars[i]=TRUE;
  
  NEW(cs->nve,cs->ne,unsigned int);
  NEW(cs->ve,cs->ne,unsigned int*);
  NEW(cs->nc,cs->ne,unsigned int);

  for(i=0;i<cs->ne;i++)
    {
      cs->nve[i]=cs->nv; /* all variables are in all equations */
      
      cs->nc[i]=(1<<cs->nve[i]); // This is 64 for all equations

      // All equations involve all variables
      NEW(cs->ve[i],cs->nve[i],unsigned int);
      for(j=0;j<cs->nve[i];j++)
	cs->ve[i][j]=j;
    }

  NEW(cs->b0,1,TCuikBox);
  UnitCuikBox(cs->nv,cs->b0);

  if (tp==1) /* just a single problem with large ranges -> define the ranges */
    {
      for(i=0;i<cs->nv;i++)
	{
	  cs->b0->l[i]=-LARGE_RANGE_6R;
	  cs->b0->u[i]=+LARGE_RANGE_6R;
	}
    }
  else
    {
      of=GetParameter(CT_OFFSET,p);// size of the assymmetry. Can be random too
      if (of!=0.0)
	{
	  for(i=0;i<cs->nv;i++)
	    {
	      cs->b0->l[i]=A2T(-M_PI_2+of);
	      cs->b0->u[i]=A2T( M_PI_2+of);
	    }
	}
    }
  
  NEW(cs->f0,cs->ne,double*);
  for(i=0;i<cs->ne;i++)
    {
      NEW(cs->f0[i],cs->nc[i],double);
    }

  mc=(1<<cs->nv); /* This must be 64 */
  
  for(j=0;j<mc;j++) // 64=cs->nc[i] for all i's
    {
      /* Evaluation of the equations for corner 'j' and the fixed offsets */
      memset(qf,0,sizeof(double)*8);
      qf[0]=1.0;
      for(i=0;i<cs->nv;i++)
	{
	  ta=tan(alpha[i]/2);

	  // a[i] and d[i] always appear divided by 2
	  a_2=a[i]/2;
	  d_2=d[i]/2;
	  
	  b=(double)(cs->offset[i]);
	  // Tangent of the half-angle at the corner
	  if ((j)&(1<<i))
	    //theta=+1.0;
	    theta=cs->b0->u[i];
	  else
	    //theta=-1.0;
	    theta=cs->b0->l[i];
	  /*
	  fprintf(stderr,"%u ----------------------------------------------------------\n",i);
	  fprintf(stderr,"  a_i=%f;\n",a[i]);
	  fprintf(stderr,"  d_i=%f;\n",d[i]);
	  fprintf(stderr,"  alpha_i=%f;\n",alpha[i]);
	  fprintf(stderr,"  ta=%f;\n",ta);
	  fprintf(stderr,"  b=%f;\n",b);
	  fprintf(stderr,"  theta=%f;\n",theta);
	  */
	  /*
	     Symbolic expressions taken from Matlab:

	     syms d_i epsilon theta b a_i ta  % ta is tan(alpha_i/2)
             epsilon = sym('epsilon');
	     Q = sym(zeros(5,4)); 
	     Q( 1,:) = [1 0 0 d_i/2*epsilon]; 
             Q( 2,:) = [1 0 0 theta]; 
	     Q( 3,:) = [1 0 0 b]; 
	     Q( 4,:) = [1 a_i/2*epsilon 0 0]; 
	     Q( 5,:) = [1 ta 0  0];
	     qt = QuatProdL(Q);

	     [c t]=coeffs(qt(1),epsilon);
	     expand(c(3))  % <- Rotation
	     expand(c(2))  % <- This is the dual part (translation)

	     This results in
	     q0 = 1 - b*theta
	     q1 = ta - b*ta*theta = ta * (1 - b*theta)
	     q2 = b*ta + ta*theta = ta * (q + theta)
	     q3 =  b + theta
	     q4 = (a_i *b*ta*theta)/2 - (a_i *ta)/2 - (d_i *theta)/2 - (b*d_i )/2
	     q5 =  a_i/2 - (a_i *b*theta)/2 - (b*d_i *ta)/2 - (d_i *ta*theta)/2
	     q6 = (a_i *b)/2 + (a_i *theta)/2 + (d_i *ta)/2 - (b*d_i *ta*theta)/2
	     q7 = d_i/2 - (a_i *b*ta)/2 - (b*d_i *theta)/2 - (a_i *ta*theta)/2

	     The equations below are the same but re-using pre-computed expressions.
	   */
	  
	  q[0] = 1 - b*theta; 
	  q[1] = ta*q[0];  
	  q[3] = b + theta;  // Compute q[3] first since it is used in q[2] !!    
	  q[2] = ta*q[3];
  
	  q[4] = -a_2*q[1] - d_2*q[3];
	  q[5] =  a_2*q[0] - d_2*q[2];  
	  q[6] =  a_2*q[3] + d_2*q[1];     
	  q[7] = -a_2*q[2] + d_2*q[0];
	  
	  DualQuaternionProduct(qf,q,qp);
	  memcpy(qf,qp,sizeof(double)*8);
	}
      /* Include the end-effector constraint */
      /* R^-1 * T^-1 */

      /* R^-1 */
      memset(q,0,sizeof(double)*8);
      Rot2Quat(R,q);
      
      DualQuaternionProduct(qf,q,qp);
      
      /* T^-1 */
      memset(q,0,sizeof(double)*8);
      q[0]=1.0;
      q[5]=-t[0]/2;
      q[6]=-t[1]/2;
      q[7]=-t[2]/2;
            
      //PrintDualQuaternion(stderr,"  q",q);
      DualQuaternionProduct(qp,q,qf);
      //PrintDualQuaternion(stderr,"  qf",qf);
      
      cs->f0[0][j]=qf[1];
      cs->f0[1][j]=qf[2];
      cs->f0[2][j]=qf[3];
      cs->f0[3][j]=qf[5];
      cs->f0[4][j]=qf[6];
      cs->f0[5][j]=qf[7];
      #if (REDUNDANT_6R)
        cs->f0[6][j]=qf[4];
      #endif

      //fprintf(stderr,"%u: %f %f %f %f %f %f\n",j,cs->f0[0][j],cs->f0[1][j],cs->f0[2][j],cs->f0[3][j],cs->f0[4][j],cs->f0[5][j]);
    }

  cs->nSimp=0; //No simplification !
  
  /*
  for(i=0;i<6;i++)
    fprintf(stderr,"%d ",cs->offset[i]);
  fprintf(stderr,"-----------------------------------------------------------------------\n");
  for(i=0;i<6;i++)
    {
      fprintf(stderr,"f%u_%u=[",np,i);
      for(j=0;j<cs->nc[i];j++)
	fprintf(stderr," %.12f",cs->f0[i][j]);
      fprintf(stderr,"];\n");
    }
  */

  /* We do not initialize auxiliary space (cs->f, cs->boxL, etc) since if necessary it is
     initialized in ReadCuikSystemFromFile */
} 

unsigned int ReadProblemFromFile(Tparameters *p,char *filename,TCuikSystem **csa)
{
  Tfilename fn,fmaf,fcuik,floops;
  unsigned int i,s,ns;
  TCuikSystem *cs;
  char *ext;

  ns=0;
  
  CreateFileName(NULL,filename,NULL,NULL,&fn);
  ext=GetFileExtension(&fn);
  
  if (ext==NULL)
    {
      /* 1) we try to read a 6R file (Manocha syntax) */
      ns=Read6RFile(p,filename,csa);
    }
  if (ns==0)
    {
      /* 2) if not, see if we can read a loops file */
      if ((ext!=NULL)&&(strcmp(ext,LOOPS_EXT)==0))
	{
	  CreateFileName(NULL,filename,NULL,LOOPS_EXT,&floops);
	  ns=ReadLoopsFile(p,GetFileFullName(&floops),csa);
	  DeleteFileName(&floops);
	}

      if (ns==0)
	{
	  if ((ext!=NULL)&&(strcmp(ext,CUIK_EXT)==0))
	    {
	      /* 3) if not, try to read an equation file */
	      CreateFileName(NULL,filename,NULL,CUIK_EXT,&fcuik);
	      ns=ReadCuikFile(p,GetFileFullName(&fcuik),csa);
	      DeleteFileName(&fcuik);
	    }
	  
	  if (ns==0)
	    {
	      /* 4) As a last option we try to read a MAF file */
	      CreateFileName(NULL,filename,NULL,MAF_EXT,&fmaf);
	      ns=ReadMAFFile(p,GetFileFullName(&fmaf),csa);
	      DeleteFileName(&fmaf);
	    }
	}
    }
  
  DeleteFileName(&fn);
  
  if (ns==0)
    Error("Can not read the problem file");
	      
  /* Allocate some auxiliary space necessary independently of
     how the system is generated */
  if (ns!=0)
    {
      #pragma omp parallel for private(s,i,cs) if (_DEBUG)
      for(s=0;s<ns;s++)
	{ 
	  cs=&((*csa)[s]);
	  
	  /* Space for temporary function evaluation */
	  NEW(cs->f,cs->ne,double*); // Space to interpolate for a sub-box
	  for(i=0;i<cs->ne;i++)
	    {
	      NEW(cs->f[i],cs->nc[i],double);
	    }

	  NEW(cs->outG,cs->nv,boolean); // Reduction on each dimension
	  NEW(cs->boxL,cs->nv,double);  // Lower limit used to the interpolation
	  NEW(cs->boxU,cs->nv,double);  // Upper limit used in the interpolation

	  // For the Newton process
	  cs->minEV=(cs->ne<cs->nv?cs->ne:cs->nv);  
	  cs->maxEV=(cs->ne>cs->nv?cs->ne:cs->nv);
	  NEW(cs->veq,cs->maxEV,double);   // values of the equations
	  NEW(cs->J,cs->ne*cs->nv,double); // Jacobian
	  NEW(cs->perm,cs->nv,int);        // Permutation of variables
	  
	  if (cs->nv!=cs->ne)
	    {
	      // Set up the Newton for overconstrained systems
	      int rank,one=1,info;
	      double w,e=1e-6;
  
	      NEW(cs->sing,cs->minEV,double);
	      cs->lwork=-1;
	      #ifdef _LAPACKE
	        info=(int)LAPACKE_dgelss_work(LAPACK_COL_MAJOR,
					      (lapack_int)cs->ne,(lapack_int)cs->nv,one,cs->J,
					      (lapack_int)cs->ne,cs->veq,(lapack_int)cs->maxEV,
					      cs->sing,e,(lapack_int*)&rank,&w,cs->lwork);
	      #else
		dgelss_((int*)&cs->ne,(int*)&cs->nv,&one,cs->J,(int*)&cs->ne,cs->veq,(int*)&cs->maxEV,cs->sing,&e,&rank,&w,&cs->lwork,&info);
	      #endif
	      if (info!=0)
		Error("Clapack error in init dgelss");
	      
	      cs->lwork=(int)w;	
	      NEW(cs->work,cs->lwork,double);
	    }
	  else
	    {
	      cs->minEV=0;
	      cs->maxEV=0;
	      cs->sing=NULL;
	      cs->lwork=-1;
	      cs->work=NULL;
	    }
	}
    }
  
  return(ns);
}

unsigned int Read6RFile(Tparameters *p,char *filename,TCuikSystem **csa)
{
  FILE *f_in;
  unsigned int i,j,ns;
  double *a;
  double *d;
  double *alpha;
  double *R;
  double *t;
  
  /* Read the 6R data */
 
  f_in=fopen(filename,"r");
  if (!f_in)
    ns=0;
  else
    { 
      fprintf(stderr,"Reading 6R file: %s\n",filename);
      NEW(a,6,double);
      for(i=0;i<6;i++)
	{
	  if (fscanf(f_in,"%lf",&(a[i]))!=1)
	    Error("Missing data in the 6R file");
	}

      NEW(d,6,double);
      for(i=0;i<6;i++)
	{
	  if (fscanf(f_in,"%lf",&(d[i]))!=1)
	    Error("Missing data in the 6R file");
	}
      
      NEW(alpha,6,double);
      for(i=0;i<6;i++)
	{
	  if (fscanf(f_in,"%lf",&(alpha[i]))!=1)
	    Error("Missing data in the 6R file");
	  alpha[i]*=DEG2RAD;
	}

      /* Observe that we transpose the rotation while reading it!  */
      NEWZ(R,16,double);
      for(i=0;i<3;i++)
	{
	  for(j=0;j<3;j++)
	    {
	      if (fscanf(f_in,"%lf",&(R[i*4+j]))!=1)
		Error("Missing data in the 6R file");
	    }
	}
      R[15]=1.0;
      
      NEWZ(t,3,double);
      for(i=0;i<3;i++)
	{
	  if (fscanf(f_in,"%lf",&(t[i]))!=1)
	    Error("Missing data in the 6R file");
	}
      
      fclose(f_in);

      #if (SINGLE_6R)
        ns=1;
      #else
	ns=64;
      #endif
	
      NEW(*csa,ns,TCuikSystem);
      
      #pragma omp parallel for private(i) if (_DEBUG)
      for(i=0;i<ns;i++)
	Define6RProblem(p,i,ns,a,d,alpha,R,t,&((*csa)[i]));
      
      free(a);
      free(d);
      free(alpha);
      free(R);
      free(t);
    }
    
  return(ns);
}

unsigned int ReadMAFFile(Tparameters *p,char *filename,TCuikSystem **csa)
{
  unsigned int i,j,s,ns;
  double *l,*u;
  TCuikSystem *cs;
  FILE *f;

  ns=0;
  
  fprintf(stderr,"Reading maf file: %s\n",filename);
  
  f=fopen(filename,"r");
  if (f)
    {
      fscanf(f,"%u",&ns); // Number of sub-problems
      NEW(*csa,ns,TCuikSystem);
	      
      for(s=0;s<ns;s++)
	{
	  cs=&((*csa)[s]);
      
	  fscanf(f,"%u",&(cs->nv)); // Number of variables
      
	  NEW(l,cs->nv,double);
	  NEW(u,cs->nv,double);
  
	  NEW(cs->varType,cs->nv,unsigned int);
	  for(i=0;i<cs->nv;i++)
	    fscanf(f,"%u",&(cs->varType[i])); // Type of each variable
	  
	  NEW(cs->offset,cs->nv,signed int);
	  for(i=0;i<cs->nv;i++)
	    fscanf(f,"%d",&(cs->offset[i])); // Offset for each varible
	  
	  for(i=0;i<cs->nv;i++)
	    {
	      fscanf(f,"%lf",&(l[i])); // Lower limit for each variable
	      fscanf(f,"%lf",&(u[i])); // Upper limit for each variable
	    }
  
	  fscanf(f,"%u",&(cs->ne)); // Number of equations
  
	  NEW(cs->nve,cs->ne,unsigned int); 
	  NEW(cs->ve,cs->ne,unsigned int*);
	  NEW(cs->nc,cs->ne,unsigned int);
	  NEW(cs->usedVars,cs->nv,boolean);
	  for(i=0;i<cs->nv;i++)
	    cs->usedVars[i]=FALSE;
	      
	  for(i=0;i<cs->ne;i++) // for all equations
	    {
	      fscanf(f,"%u",&(cs->nve[i])); // Number of variables in the equation

	      cs->nc[i]=(1<<cs->nve[i]); // Number of combinations of variables
      
	      NEW(cs->ve[i],cs->nve[i],unsigned int);
	      for(j=0;j<cs->nve[i];j++)
		{
		  fscanf(f,"%u",&(cs->ve[i][j])); // ID of the variable in the equation
		  cs->usedVars[cs->ve[i][j]]=TRUE;
		}
	    }
	  
	  NEW(cs->f0,cs->ne,double*); // Space to evaluate the equations
	  for(i=0;i<cs->ne;i++)
	    {
	      NEW(cs->f0[i],cs->nc[i],double); // Space for the evaluation at each corner
	      for(j=0;j<cs->nc[i];j++)
		fscanf(f,"%lf",&(cs->f0[i][j]));
	    }

	  NEW(cs->b0,1,TCuikBox);
	  InitCuikBox(l,u,cs->nv,cs->b0); // The box where the problem is defined
	  
	  free(l);
	  free(u);
	}

      // At the end of the file we add simplification information, in some cases only :)
      if (fscanf(f,"%u",&(cs->nSimp))!=1)
	{
	  unsigned int m,n;
	      
	  NEW(cs->nIn,cs->nSimp,unsigned int);
	  NEW(cs->nOut,cs->nSimp,unsigned int);
	  NEW(cs->vIn,cs->nSimp,unsigned int*);
	  NEW(cs->vOut,cs->nSimp,unsigned int*);
	  NEW(cs->S,cs->nSimp,double*);
	  
	  for(i=0;i<cs->nSimp;i++)
	    {
	      fscanf(f,"%u",&(cs->nIn[i]));
	      if (cs->nIn[i]>0)
		{
		  for(j=0;j<cs->nIn[i];j++)
		    fscanf(f,"%u",&(cs->vIn[i][j]));
		      
		  fscanf(f,"%u",&(cs->nOut[i]));
		  for(j=0;j<cs->nOut[i];j++)
		    fscanf(f,"%u",&(cs->vOut[i][j]));

		  m=(1<<cs->nIn[i]);
		  n=(1<<cs->nOut[i]);

		  NEW(cs->S[i],n*m,double);
		  for(j=0;j<n*m;j++)
		    fscanf(f,"%lf",&(cs->S[i][j]));
		}
	      else
		{
		  cs->nOut[i]=0;
		  cs->vIn[i]=NULL;
		  cs->vOut[i]=NULL;
		  cs->S[i]=NULL;
		}
	    }
	}
      else
	cs->nSimp=0; // No simplification
      
      fclose(f);
    }
  return(ns);
}

void SaveCuikSystemToMAF(char *filename,unsigned int ns,TCuikSystem *csa)
{
  Tfilename fout;
  FILE *f;
  unsigned int i,j,s;
  double *l,*u;
  TCuikSystem *cs;
  
  /* Now generate the file encoding the problem */
  
  CreateFileName(NULL,filename,NULL,MAF_EXT,&fout);
  
  fprintf(stderr,"Creating maf file: %s\n",GetFileFullName(&fout));
  
  f=fopen(GetFileFullName(&fout),"w");
  if (!f)
    {
      char ErrorText[500];

      sprintf(ErrorText,"File %.450s can not be created",GetFileFullName(&fout));
      Error(ErrorText);
    }

  // Number of subproblems
  fprintf(f,"%u\n",ns);
  for(s=0;s<ns;s++)
    {
      cs=&(csa[s]);
      
      fprintf(f,"%u\n",cs->nv); // Number of variables
      
      for(i=0;i<cs->nv;i++)
	fprintf(f,"%u ",cs->varType[i]); // Type of each variable
      fprintf(f,"\n");
	  
      for(i=0;i<cs->nv;i++)
	fprintf(f,"%d ",cs->offset[i]); // Offset for each varible
      fprintf(f,"\n");

      l=cs->b0->l;
      u=cs->b0->u;

      for(i=0;i<cs->nv;i++)
	fprintf(f,"%.16f %.16f ",l[i],u[i]); // Lower/Upper limit for each variable
      fprintf(f,"\n");

      fprintf(f,"%u\n",cs->ne); // Number of equations
  
      for(i=0;i<cs->ne;i++) // for all equations
	{
	  fprintf(f,"%u ",cs->nve[i]); // Number of variables in the equation

	  for(j=0;j<cs->nve[i];j++)
	    fprintf(f,"%u ",cs->ve[i][j]); // ID of the variable in the equation
	  fprintf(f,"\n");
	}
  
      for(i=0;i<cs->ne;i++)
	{
	  for(j=0;j<cs->nc[i];j++)
	    fprintf(f,"%.16f ",cs->f0[i][j]);
	  fprintf(f,"\n");
	}
    }

  if (cs->nSimp>0)
    {
      unsigned int n,m;
      
      fprintf(f,"%d\n",cs->nSimp);

       for(i=0;i<cs->nSimp;i++)
	 {
	   fprintf(f,"%u\n",cs->nIn[i]);
	   if (cs->nIn[i]>0)
	     {
	       for(j=0;j<cs->nIn[i];j++)
		 fprintf(f,"%u ",cs->vIn[i][j]);
	       fprintf(f,"\n");
	   
	       fprintf(f,"%u\n",cs->nOut[i]);
	       for(j=0;j<cs->nOut[i];j++)
		 fprintf(f,"%u ",cs->vOut[i][j]);
	       fprintf(f,"\n");
	   
	       m=(1<<cs->nIn[i]);
	       n=(1<<cs->nOut[i]);
	   
	       for(j=0;j<n*m;j++)
		 fprintf(f,"%.16f ",cs->S[i][j]);
	       fprintf(f,"\n");
	     }
	 }
    }

  fclose(f);
  DeleteFileName(&fout);  
}


void CopyCuikSystem(TCuikSystem *cs_dst,TCuikSystem *cs_src)
{
  unsigned int j,k;
  
  cs_dst->ne=cs_src->ne;
  cs_dst->nv=cs_src->nv;
  if (cs_dst->ne==0)
    {
      cs_dst->nc=NULL;
      cs_dst->nve=NULL;
      cs_dst->ve=NULL;
      cs_dst->f0=NULL;
      cs_dst->f=NULL;

      cs_dst->outG=NULL;
      cs_dst->boxL=NULL;
      cs_dst->boxU=NULL;
    }
  else
    {
      NEW(cs_dst->nve,cs_dst->ne,unsigned int);
      NEW(cs_dst->ve,cs_dst->ne,unsigned int*);
      NEW(cs_dst->nc,cs_dst->ne,unsigned int);
      NEW(cs_dst->f0,cs_dst->ne,double*);

      for(j=0;j<cs_dst->ne;j++)
	{
	  cs_dst->nve[j]=cs_src->nve[j];
	  
	  cs_dst->nc[j]=cs_src->nc[j];
	  
	  NEW(cs_dst->ve[j],cs_dst->nve[j],unsigned int);
	  for(k=0;k<cs_dst->nve[j];k++)
	    cs_dst->ve[j][k]=cs_src->ve[j][k];
	  
	  NEW(cs_dst->f0[j],cs_dst->nc[j],double);
	  memcpy(cs_dst->f0[j],cs_src->f0[j],sizeof(double)*cs_dst->nc[j]);
	}
      NEW(cs_dst->f,cs_dst->ne,double*);
      for(j=0;j<cs_dst->ne;j++)
	{
	  NEW(cs_dst->f[j],cs_dst->nc[j],double);
	}
      
      NEW(cs_dst->outG,cs_dst->nv,boolean);
      NEW(cs_dst->boxL,cs_dst->nv,double);
      NEW(cs_dst->boxU,cs_dst->nv,double);
    }

  if (cs_src->nv>0)
    { 
      NEW(cs_src->b0,1,TCuikBox);
      CopyCuikBox(cs_dst->b0,cs_src->b0);
    }
  else
    cs_dst->b0=NULL;

  cs_dst->minEV=cs_src->minEV;
  cs_dst->maxEV=cs_src->maxEV;
  NEW(cs_dst->veq,cs_dst->maxEV,double);
  NEW(cs_dst->J,cs_dst->ne*cs_dst->nv,double);
  NEW(cs_dst->perm,cs_dst->nv,int);

  if (cs_dst->nv!=cs_dst->ne)
    {
      NEW(cs_dst->sing,cs_dst->minEV,double);
      cs_dst->lwork=cs_src->lwork;
      NEW(cs_dst->work,cs_dst->lwork,double);
    }
  else
    {
      cs_dst->minEV=0;
      cs_dst->maxEV=0;
      cs_dst->sing=NULL;
      cs_dst->lwork=-1;
      cs_dst->work=NULL;
    }
}

/*
  Returns the number of variables in the CuikSystem
*/
unsigned int GetCSNumVariables(TCuikSystem *cs)
{
  return(cs->nv);
}

/*
  Returns the number of equations in the CuikSytem
*/
unsigned int GetCSNumEquations(TCuikSystem *cs)
{
  return(cs->ne);
}

/*
  Takes a cuik systema and returns all solutions. 
  Solutions are stored in file 'f_out' (if defined) and in list 'sol'
  (if defined), or both.

  This procedure is used for single-cpu versions of Cuik.
 */
void SolveCuikSystem(unsigned int s,Tparameters *p,Tlist *searchSpace,
		     FILE *f_out,Tlist *solutions,Tlist *verifiedS,
		     TStatistics *st,TCuikSystem *cs)
{
  unsigned int rm,nr,simplify;
  Tlist boxes;
  TCuikBox *b,*b1;
  unsigned int c,cf; /* output of ReduceBox */
  double sigma,epsilon;
  boolean small;
  unsigned int nBoxes,nSols,vSols,nEmpty,nBisected,nReductions,maxLevel;
  boolean saveAngles;
  unsigned int clean,verify;
  double *x;
  
  /************************************************************************************/
  /************************************************************************************/
  /************************************************************************************/

  clean=(unsigned int)GetParameter(CT_CLEAN,p);
  verify=(unsigned int)GetParameter(CT_VERIFY,p);
  saveAngles=(GetParameter(CT_SAVE_ANGLES,p)>0);
  rm=(unsigned int)GetParameter(CT_REDUCTION,p);
  simplify=(unsigned int)GetParameter(CT_SIMPLIFY,p);
  
  sigma=GetParameter(CT_SIGMA,p); 
  epsilon=GetParameter(CT_EPSILON,p);
  
  #if (SIMPLEX_AVAILABLE)
    if ((rm==RED_SIMPLEX)||(clean==CLEAN_SIMPLEX))
      InitSimplex(epsilon,cs);
  #else
    if (rm==RED_SIMPLEX)
      Error("Simplex not available. Change the reduction method in the parameter file.");
    if (clean==CLEAN_SIMPLEX)
      Error("Simplex not available. Change the clean method in the parameter file.");
  #endif
    
  if ((verify==VERIFY_NEWTON)||(verify==VERIFY_NEWTON_REPLACE))
    {
      NEW(x,cs->nv,double);
    }
    
  #if (_DEBUG>0)    
    if ((f_out==NULL)&&(solutions==NULL))
      Error("We need a valid output file or a list to store the solutions.");
  #endif

  if (searchSpace==NULL)
    {
      InitList(sizeof(TCuikBox *),&boxes);
	
      NEW(b,1,TCuikBox);
      CopyCuikBox(b,cs->b0);
      
      AddLastElement(&b,&boxes);
    }
  else
    CopyCuikBoxList(&boxes,searchSpace);

  
  if (solutions!=NULL)
      InitList(sizeof(TCuikBox *),solutions);
  if (verifiedS!=NULL)
      InitList(sizeof(TCuikBox *),verifiedS);
  
  /* Space for the clipping operations */
  nBoxes=0;
  nEmpty=0;
  nSols=0;
  vSols=0;
  nBisected=0;
  nReductions=0;
  maxLevel=0;
	    
  /*the cuik solve process itself*/
  while ((!ListEmpty(&boxes))&&(nBoxes<10000000)) /* Until everything is completed */
    {
 
      /*Get a new box*/
      nBoxes++;
            
      /*The box is extracted to avoid another process 
	to get the same box*/
      ExtractFirst(&b,&boxes); 
	      
      if (b->level>maxLevel)
	maxLevel=b->level;
      
      #if (_DEBUG>1)
        fprintf(stderr,"--------------------------------------------------------\n");
        fprintf(stderr,"--------------------------------------------------------\n");
        fprintf(stderr,"Processing box -----------------------------------------\n");
        PrintCuikBox(stderr,b);
      #endif
      #if (_DEBUG>0)
        fprintf(stderr,"<%g %g>[%u]",CuikBoxVolume(cs->usedVars,b),CuikBoxSize(cs->usedVars,b),b->level);
	#if (_DEBUG>1)
  	  fprintf(stderr,"\n");
	#endif
      #endif

     /* Reduce the box by all vars */
     if ((SIMPLEX_AVAILABLE)&&(rm==RED_SIMPLEX))
       {
	 /* Simplex-based reduction */
	 c=ReduceBoxSimplex(p,b,&nr,cs);
       }
     else
       {
	 /* Trapezoid-based reduction */
	 c=ReduceBox(p,b,&nr,cs);
       }
     
      nReductions+=nr;
      
      #if (_DEBUG>0)
        fprintf(stderr,"  -> ");
      #endif  

      small=(CuikBoxSize(cs->usedVars,b)<=sigma);

      /* clean the potential solution (only for trapezoid clipping) */
      if (small)
	{ 
	  /* CLEAN the potential solution (this can declare the box empty) */
	  #if (_DEBUG)
	    fprintf(stderr,"-> Check");
	  #endif
	    
	  cf=CheckSolution(p,b,cs); /* A minimum (fast) sanity check even if clean is off */

	  /* cf is EMPTY, REDUCED or REDUCED_WITH_SOLUTION */
	  c=(cf==EMPTY?EMPTY:(c==REDUCED_WITH_SOLUTION?c:cf));
	     
	  /* Clean the solution */
	  if ((c!=EMPTY)&&(SIMPLEX_AVAILABLE)&&(clean==CLEAN_SIMPLEX))
	    {
	      #if (_DEBUG)
	        fprintf(stderr,"-> Clean");
	      #endif
		   
	      cf=ReduceBoxSimplex(p,b,&nr,cs);
		     
	      /* cf is EMPTY, NOT_REDUCED, REDUCED or REDUCED_WITH_SOLUTION */
	      c=(cf==EMPTY?EMPTY:(c==REDUCED_WITH_SOLUTION?c:cf));
	    }
	}
	
      if (c==EMPTY)
	{
	  nEmpty++;
	  DeleteCuikBox(b);
	  free(b);
	  #if (_DEBUG>0)
	    fprintf(stderr,"E\n");
          #endif
	}
      else
	{
	  #if (_DEBUG>0)
	    fprintf(stderr,"<%g %g>",CuikBoxVolume(cs->usedVars,b),CuikBoxSize(cs->usedVars,b));
	  #endif

	  if (small)
	    {
	      /* VERIFY the potential solution (No need to verify if already verified) */
	      if ((rm==RED_TRAPEZOID)&&(c!=REDUCED_WITH_SOLUTION)&&((verify!=VERIFY_MIRANDA)||(clean==CLEAN_SIMPLEX))) /* here c is NOT_REDUCED, REDUCED, REDUCED_WITH_SOLUTION */
		{
		  /* if rm==RED_TRAPEZOID -> verification is only implemented for the trapezoid method.
		     if c!=REDUCED_WITH_SOLUTION -> theres is no need to verify if the box is already known to include a solution
		     if verify!=VERIFY_MIRANDA || clean==NO_CLEAN VerifyBox -> if the verification method is MIRANDA we are done (unless the last step was a clean-based simplex) */
		  #if (_DEBUG)
		    fprintf(stderr,"-> Verify ");
		  #endif
		    
		  c=VerifyBox(p,x,b,cs); /* c is NOT_REDUCED or REDUCED_WITH_SOLUTION */ 
		}
	      
	      nSols++;
	      if (c==REDUCED_WITH_SOLUTION)
		vSols++;
 
	      #if (_DEBUG>0)
	        if (c==REDUCED_WITH_SOLUTION)
		  fprintf(stderr,"S\n");
		else
		  fprintf(stderr,"s\n");
	      #endif  
		    
	      if (f_out!=NULL)
		{
		  double e;

		  // Error is computed before modifying the box (offset, angles)
		  e=ErrorInSolution(b,cs);
		      
		  /* We print the solutions in tangent space but applying
		     the offset, if any. In this way, the management (plot,
		     transformation, etc) of solution  files is easier. */

		  if (saveAngles)
		    CuikBox2AnglesWithOffset(cs->usedVars,cs->varType,cs->offset,b);
		  else
		    CuikBoxOffset(cs->usedVars,cs->varType,cs->offset,b);

		  // Can not write in parallel to the same file
		  #pragma omp critical
		  {
		    fprintf(f_out," %3u err:%g ",nSols,e);
		    if (c==REDUCED_WITH_SOLUTION)
		      fprintf(f_out,"<S> ");
		    PrintCuikBox(f_out,b);
		    fflush(f_out);
		  }
		}
		  
	      if (solutions!=NULL)
		{
		  if ((c==REDUCED_WITH_SOLUTION)&&(verifiedS!=NULL))
		    AddLastElement(&b,verifiedS);
		  else
		    AddLastElement(&b,solutions);
		}
	      else
		{
		  DeleteCuikBox(b);
		  free(b);
		}
	    } 
	  else
	    {
	      /* Non empty large box */
	      unsigned int d;

	      nBisected++;
	      
	      d=ComputeSplitDim(p,b,cs);

	      NEW(b1,1,TCuikBox);
	      SplitCuikBox(d,CUT_POINT,b1,b);

	      #if (SEARCH_MODE==DEPTH_FIRST_SEARCH)
	        AddFirstElement(&b,&boxes);
	        AddFirstElement(&b1,&boxes);
	      #else
	        AddLastElement(&b,&boxes);
	        AddLastElement(&b1,&boxes);
	      #endif
	      
	      #if (_DEBUG>0)
	        fprintf(stderr,"b[%u]\n",d);
	      #endif
	    }
	} 
    }
  
  if ((simplify)&&(solutions!=NULL))
    PropagateSolutions(simplify,epsilon,solutions,cs);
  
  if ((simplify)&&(verifiedS!=NULL))
    PropagateSolutions(simplify,epsilon,verifiedS,cs);
  
  InitStatistics(nBoxes,nReductions,nBisected,
		 nEmpty,nSols,vSols,maxLevel,st);

  /* the list might not be empty if we reached the maximum
     number of boxes to process */
  DeleteCuikBoxList(&boxes);
  
  if ((verify==VERIFY_NEWTON)||(verify==VERIFY_NEWTON_REPLACE))
    free(x);
  
  #if (SIMPLEX_AVAILABLE)
    if ((rm==RED_SIMPLEX)||(clean==CLEAN_SIMPLEX))
      DeleteSimplex(cs);
  #endif
}

void PropagateSolutions(unsigned int simplify,double epsilon,Tlist *solutions,TCuikSystem *cs)
{
  if (simplify>0)
    {
      unsigned int i,nu;
      boolean empty;
      
      if ((simplify!=2)&&(simplify!=3))
	Error("Undefined simplication level");

      // If the siplification failed of if we use redundacy, all
      // variables might be used.
      nu=0;
      for(i=0;i<cs->nv;i++)
	{
	  if (!cs->usedVars[i]) nu++;
	}
      if (nu>0)
	{
	  Titerator it;
	  TCuikBox *b;
    
	  InitIterator(&it,solutions);
	  First(&it);
	  while(!EndOfList(&it))
	    {
	      b=*(TCuikBox**)GetCurrent(&it);

	      empty=FALSE;
	      if ((simplify==2)&&(cs->nSimp>0))
		{
		  for(i=0;i<cs->nv;i++)
		    {
		      if ((!cs->usedVars[i])&&
			  (cs->varType[i]==VAR_ROT))
			{
			  if (cs->offset[i]==0)
			    {
			      // non-used (simplified) rotational variables with offset
			      // Just keep the ranges given by the user but in tangent space
			      b->l[i]=A2T(cs->b0->l[i]);
			      b->u[i]=A2T(cs->b0->u[i]);
			    }
			  else
			    {
			      // non-used (simplified) rotational variables with offset
			      // have ranges in [-pi,pi] -> infinity in tangent
			      // space
			      b->l[i]=-INF;
			      b->u[i]=+INF;
			    }
			}
		      // From now, we will have values for the eliminated variables
		      // Do not considere them as "eliminated" any more
		      // Since the propagation of ranges is the last step in the solvin
		      // process, this only affects the output of the solution boxes
		      cs->usedVars[i]=TRUE;
		    }
		  // When removing two variables, we can do better
		  empty=(!PropagateRanges(epsilon,b,cs));
		}

	      if (empty)
		{
		  DeleteCuikBox(b);
		  DeleteCurrent(&it);
		}
	      else
		Advance(&it);
	    }
	}
    }
}

boolean PropagateRanges(double epsilon,TCuikBox *b,TCuikSystem *cs)
{
  unsigned int i,j,k,t,m,n,nv;
  Tinterval *x;
  Tinterval *y;
  Tinterval r,eps;
  boolean empty;

  /* 
     We have to compute y=S*x
     With x the monomials of the know variables
     and y the ones the variables to recover.
     Right now x involves 4 variables and x 2.
  */ 
  #if (_DEBUG>1)
    fprintf(stderr,"Offsets: [");
    for(i=0;i<cs->nv;i++)
      fprintf(stderr," %d ",cs->offset[i]);
    fprintf(stderr,"]\n");
  #endif
  
  empty=FALSE;

  NewInterval(-epsilon,epsilon,&eps);
  
  // For each simplification rule
  for(i=0;((!empty)&&(i<cs->nSimp));i++)
    {
      #if (_DEBUG>1)
        fprintf(stderr,"Simplification rule: %u\n",i);
	fprintf(stderr,"In: [ ");
	for(j=0;j<cs->nIn[i];j++)
	  fprintf(stderr,"%u ",cs->vIn[i][j]);
	fprintf(stderr,"]\n");
	fprintf(stderr,"Out: [ ");
	for(j=0;j<cs->nOut[i];j++)
	  fprintf(stderr,"%u ",cs->vOut[i][j]);
	fprintf(stderr,"]\n");
      #endif
	
      if ((cs->nIn[i]==4)&&(cs->nOut[i]==2))
	{
	  m=(1<<cs->nIn[i]);
	  n=(1<<cs->nOut[i]);
	  
	  NEW(x,m,Tinterval);
	  NEW(y,n,Tinterval);
  
	  // Define x (ranges)
	  for(j=0;j<m;j++)
	    {
	      
	      NewInterval(1,1,&(x[j]));
	      for(k=0;k<cs->nIn[i];k++)
		{
		  if (j & (1<<k))
		    {
		      nv=cs->vIn[i][k];
		      NewInterval(b->l[nv],b->u[nv],&r);
		      IntervalProduct(&r,&(x[j]),&(x[j]));
		    }
		}
	    }
          #if (_DEBUG>1)
	    PrintCuikBox(stderr,b);
	    fprintf(stderr,"x=[ ");
	    for(j=0;j<m;j++)
	      {
	 	PrintInterval(stderr,&(x[j]));
		fprintf(stderr," ");
	      }
	    fprintf(stderr,"]\n");
	    fprintf(stderr,"xc=[");
	      for(j=0;j<m;j++)
		fprintf(stderr," %f ",IntervalCenter(&(x[j])));
	    fprintf(stderr,"]\n");
	    PrintMatrix(stderr,"S",n,m,cs->S[i]);
	  #endif
	  
	  // y = S*x
	  for(j=0;j<n;j++)
	    {
	      NewInterval(0,0,&(y[j]));
	      for(k=0;k<m;k++)
		{
		  IntervalScale(&(x[k]),cs->S[i][RC2INDEX(j,k,n,m)],&r);
		  IntervalAdd(&r,&(y[j]),&(y[j]));
		}
	    }
	  #if (_DEBUG>1)
	    fprintf(stderr,"y=[ ");
	    for(j=0;j<n;j++)
	      {
		PrintInterval(stderr,&(y[j]));
		fprintf(stderr," ");
	      }
	    fprintf(stderr," ]\n");
	  #endif
	    
	  // Intersect y with the ranges we have so far
	  for(k=0;((!empty)&&(k<cs->nOut[i]));k++)
	    {
	      t=(1<<k);
	      nv=cs->vOut[i][k];
	      NewInterval(b->l[nv],b->u[nv],&r);
	      IntervalDivision(&(y[t]),&(y[0]),&(y[t]));
	      IntervalAdd(&(y[t]),&eps,&(y[t])); // To compensate minor numerical errors in S!
	      #if (_DEBUG>1)
	        fprintf(stderr,"Intersection: ");
		PrintInterval(stderr,&r);
		fprintf(stderr," y[%u]=",t);
		PrintInterval(stderr,&(y[t]));
		fprintf(stderr," -> ");
	      #endif
	      empty=(!Intersection(&(y[t]),&r,&r));
	      if (!empty)
		{
		  b->l[nv]=LowerLimit(&r);
		  b->u[nv]=UpperLimit(&r);
		  #if (_DEBUG>1)
		    PrintInterval(stderr,&r);
		  #endif
		}
	      #if (_DEBUG>1)
	      else
		fprintf(stderr,"empty");
	      fprintf(stderr,"\n");
	      #endif
	    }
	  
	  free(x);
	  free(y);
	}
    }
  return(!empty);
}

void IntersectWithCuikSystem(Tparameters *p,Tlist *listIn,Tlist *listOut,TCuikSystem *cs)
{
  Titerator it;
  TCuikBox *b,*bOut;
  Tinterval *bRef;
  unsigned int i;
  boolean empty;
  double of;
  Tinterval r;
  boolean saveAngles;

  /* if saveAngles assume linstIn is also in angles */
  saveAngles=(GetParameter(CT_SAVE_ANGLES,p)>0);
      
  InitList(sizeof(TCuikBox *),listOut);

  /* Ranges covered by this CuikSystem (depending on the offsets) */
  of=GetParameter(CT_OFFSET,p);
  NEW(bRef,cs->nv,Tinterval);
  for(i=0;i<cs->nv;i++)
    {
      switch(cs->offset[i])
	{
	case -1:
	  NewInterval(-M_PI+of,of,&(bRef[i]));
	  break;
	case 0:
	  NewInterval(-INF,INF,&(bRef[i]));
	  break;
	case 1:
	  NewInterval(of,M_PI+of,&(bRef[i]));
	  break;
	default:
	  Error("Undefined offset in IntersectWithCuikSystem");
	}
    }
  
  InitIterator(&it,listIn);
  First(&it);
  empty=FALSE;
  while(!EndOfList(&it))
    {
      b=*(TCuikBox**)GetCurrent(&it);

      if (!empty)
	{
	  NEW(bOut,1,TCuikBox);
	}
      CopyCuikBox(bOut,b);
            
      empty=FALSE;
      for(i=0;((!empty)&&(i<cs->nv));i++)
	{
	  if ((!saveAngles)&&(cs->varType[i]==VAR_ROT))
	    NewInterval(T2A(b->l[i]),T2A(b->u[i]),&r);
	  else
	    NewInterval(b->l[i],b->u[i],&r);
	  
	  empty=!(Intersection(&(bRef[i]),&r,&r));
	  if (!empty)
	    {
	      if (cs->varType[i]==VAR_ROT)
		{
		  bOut->l[i]=A2T(OA(LowerLimit(&r),-cs->offset[i]));
		  bOut->u[i]=A2T(OA(UpperLimit(&r),-cs->offset[i]));
		}
	      else
		{
		  bOut->l[i]=LowerLimit(&r);
		  bOut->u[i]=UpperLimit(&r);
		}
	    }
	}

      if (!empty)
	AddLastElement(&bOut,listOut);
      
      Advance(&it);
    }
  free(bRef);
}

/*
  Selects one dimension along which to split box 'b'. The dimension can
  be selected according to the size or according to the error that each
  variable induce in each equation.
  The output is a index in the simplified system
 */
unsigned int ComputeSplitDim(Tparameters *p,TCuikBox *b,TCuikSystem *cs)
{
  unsigned int i;
  unsigned int n;
  double m,s;
  
  if (cs->nv==0)
    {
      Error("Splitting an empty cuiksystem");
      return(NO_UINT);
    }
  else
    {      
      m=-1.0;
      n=0;
      for(i=0;i<cs->nv;i++)
	{
	  if (cs->usedVars[i])
	    {
	      s=b->u[i]-b->l[i];
	      if (s>m)
		{
		  n=i;
		  m=s;
		}
	    }
	}
 
      if (m<0) 
	{
	  Warning("There is no way where to bisect the box");
	  n=randomMax(cs->nv-1);
	}

      return(n);
    }
}

void EvaluateCSEquations(double *x,double *r,TCuikSystem *cs)
{
  unsigned int i;

  for(i=0;i<cs->ne;i++)
    r[i]=EvaluateCSEquation(i,x,cs);
}

double ErrorInSolution(TCuikBox *b,TCuikSystem *cs)
{
  double n;
    
  n=0; /* norm of the error */
  if ((cs->nv>0)&&(cs->ne>0))
    {
      double *x;

      NEW(x,cs->nv,double);
      CuikBoxCenter(x,b);

      EvaluateCSEquations(x,cs->veq,cs);
      n=Norm(cs->ne,cs->veq);
      
      free(x);
    }
  #if (_DEBUG>1)
    fprintf(stderr,"Error in solution: %g\n",n);
  #endif
  return(n);
}

/*
  Destructor of the cuiksystem structure.
*/
void DeleteCuikSystem(TCuikSystem *cs)
{
  if (cs->ne>0)
    {
      unsigned int i;
      
      DeleteCuikBox(cs->b0);
      free(cs->b0);
      
      free(cs->usedVars);
      
      for(i=0;i<cs->ne;i++)
	{
	  free(cs->ve[i]);
	  free(cs->f0[i]);
	}
      
      free(cs->f0);
      free(cs->ve);

      for(i=0;i<cs->ne;i++)
	free(cs->f[i]);
      free(cs->f);
      
      free(cs->varType);
      free(cs->offset);
	
      free(cs->nc);
      free(cs->nve);
      cs->nv=0;
      cs->ne=0;
      
      free(cs->outG);
      free(cs->boxL);
      free(cs->boxU);

      if (cs->nSimp>0)
	{
	  for(i=0;i<cs->nSimp;i++)
	    {
	      if (cs->nIn[i]>0)
		{
		  free(cs->vIn[i]);
		  free(cs->vOut[i]);
		  free(cs->S[i]);
		}
	    }
	  free(cs->nIn);
	  free(cs->vIn);
	  free(cs->nOut);
	  free(cs->vOut);
	  free(cs->S);
	}
      
      free(cs->veq);
      free(cs->J);
      free(cs->perm);
      if (cs->sing!=NULL)
	free(cs->sing);
      if (cs->work!=NULL)
	free(cs->work);
    }
}


