
#include "cuiksystem.h"
#include "parameters.h"

#include "defines.h"
#include "error.h"
#include "filename.h"

#include <string.h>
#include <stdlib.h>
#include <time.h>

#ifdef _OPENMP
  #include <omp.h>
#endif


/** \file cuikverify.c
    \brief Main body of the \ref cuikverify.c "cuikcluter" application.
 
    <center>
    \dot
    digraph applications {
      size="7,15"
      node [shape=record, style=filled, fontname=Helvetica, fontsize=12];

      verify    [label="_verify.sol"    color="#99ff99" shape=ellipse ];
      
      kin     [label="cuik"   color="#ff9999"];
      sol     [label="sol"     color="#ff9999"];

      sol -> cuik
      kin -> verify
    }
    \enddot
    </center>

    This application verifys solution boxes. 
    <br>

    \sa cuiksystem.h cuiksystem.c
*/

/** 
   \brief Main body of the \b cuikverify application.

   Main body of the \b cuikverify application.

   Use:
      - \b cuikverify \e problem_name.sol \e solution_file.sol
      .
   where
      - \e problem_name contains the solutions to cluster
      - \e solution_file [optional] the file with the solutions to refine.
      .

   The \e .sol extension is optional.
   
   \param argc Number of elements in the command line that starts the application (i.e., the
               \b cuik command itself and its arguments).
   \param arg Strings with the arguments.
   
   \return EXIT_SUCCESS (0) if the execution worked out fine and EXIT_FAILURE if not.
*/
int main(int argc, char **arg)
{
  FILE *f_out;             /* File where the solutions are printed as they 
			      are found */
 
  FILE *f_in;            /* Used to read-and-print the equation input 
			    file in the output file */

  Tparameters parameters; 
  unsigned int ns,i;
  TCuikSystem *cuiksystem;
  
  Tlist solutions;

  Tfilename fparam;
  Tfilename fsols;
  Tfilename fverify;

  boolean angles;

  struct timespec t0,t1;
  
  if (argc>1)
    {
      /*Init parameters*/
      CreateFileName(NULL,arg[1],NULL,PARAM_EXT,&fparam);
      InitParametersFromFile(GetFileFullName(&fparam),&parameters);
      
      angles=(GetParameter(CT_SAVE_ANGLES,&parameters)>0);
      
      ns=ReadProblemFromFile(&parameters,arg[1],&cuiksystem);
      
      /* Read the list of boxes */

      if (argc>2)
	CreateFileName(NULL,arg[2],NULL,SOL_EXT,&fsols);
      else
	CreateFileName(NULL,arg[1],NULL,SOL_EXT,&fsols);
      f_in=fopen(GetFileFullName(&fsols),"r");
      if (!f_in)
	Error("Can not open the file with the solutions");

      ReadCuikBoxList(f_in,&solutions);
      
      /* Save the list of boxes */
      if (argc>2)
	CreateFileName(NULL,arg[2],"_verify",SOL_EXT,&fverify);
      else
	CreateFileName(NULL,arg[1],"_verify",SOL_EXT,&fverify);
      f_out=fopen(GetFileFullName(&fverify),"w");
      if (!f_out)
	Error("Can not open the file for the verifyed solutions");
      
      clock_gettime(CLOCK_REALTIME,&t0);

      VerifyCuikBoxes(&parameters,angles,f_out,ns,cuiksystem,&solutions);
      
      clock_gettime(CLOCK_REALTIME,&t1);
      fprintf(stderr,"Execution time: %f seconds",(t1.tv_sec+t1.tv_nsec*1e-9)-(t0.tv_sec+t0.tv_nsec*1e-9));
      fprintf(stderr,"\n");

      fclose(f_out);
      
      /* Release memory */
      DeleteCuikBoxList(&solutions);
      for(i=0;i<ns;i++)
	DeleteCuikSystem(&(cuiksystem[i]));
      free(cuiksystem);

      DeleteParameters(&parameters);
      
      DeleteFileName(&fsols);
      DeleteFileName(&fverify);
      DeleteFileName(&fparam);
    }
  else
    {
      fprintf(stderr,"  Wrong number of parameters.\n");
      fprintf(stderr,"  Use:\n");   
      fprintf(stderr,"      cuikverify <problem file> <solution file>\n");
      fprintf(stderr,"  where <problem file> contains the solutions\n");
      fprintf(stderr,"        <solution file> [optional] the solutions to refine\n");
    }

  return(EXIT_SUCCESS);
}

