#include "equation.h"

#include "defines.h"

#include <string.h>
#include <math.h>

/** \file equation.c
    \brief Implementation of the functions operating on Tequation.

    Implementation of the functions operating on Tequation.
    
    \sa Tequation, equation.h,
*/

/** 
   \brief Constructor.

   Constructor.

   \param t The term to initialize.

*/ 
void InitTerm(Tterm *t);

/** 
   \brief Copy constructor for terms.

   Copy constructor for terms.

   \param t_dst The term to create.
   \param t_src The term to copy.
*/
void CopyTerm(Tterm *t_dst,Tterm *t_src);

/** 
   \brief Adds a variable to a term.

   Adds a variable to a term.

   \param id Identifier of the variable to add.
   \param t The term to modify.
*/
void AddVariable2Term(unsigned int id,Tterm *t);

/** 
   \brief Adds a sub-equaton to a term.

   Adds a sub-equaton to a term.

   \param eq The sub-equation.
   \param t The term to modify.
*/
void AddEquation2Term(Tequation *eq,Tterm *t);

/** 
   \brief Evalates a term.

   Evalates a term.

   \param varValues Values for the variables.
   \param t The term to evaluate.

   \return The evaluation.
*/
double EvaluateTerm(double *varValues,Tterm *t);

/** 
   \brief Prints a term to a file.

   Prints a term to a file.

   \param f The file.
   \param varNames The names of the variables.
   \param t The term.
*/
void PrintTerm(FILE *f,char **varNames,Tterm *t);

/** 
   \brief Destructor.

   Destructor.

   \param t The term to delete.
*/
void DeleteTerm(Tterm *t);

/*******************************************************************/
/*******************************************************************/
/*******************************************************************/

void InitTerm(Tterm *t)
{
  t->maxItems=INIT_NUM_ITEMS;
  t->nItems=0;
  NEW(t->id,t->maxItems,unsigned int);
  NEW(t->eq,t->maxItems,void*);
}

void CopyTerm(Tterm *t_dst,Tterm *t_src)
{
  unsigned int i;
  Tequation *eq;
  
  t_dst->maxItems=t_src->maxItems;
  t_dst->nItems=t_src->nItems;
  NEW(t_dst->id,t_dst->maxItems,unsigned int);
  NEW(t_dst->eq,t_dst->maxItems,void *);
  for(i=0;i<t_dst->nItems;i++)
    {
      if (t_src->id[i]!=NO_UINT)
	{
	  t_dst->id[i]=t_src->id[i];
	  t_dst->eq[i]=NULL;
	}
      else
	{
	  t_dst->id[i]=NO_UINT;
	  NEW(eq,1,Tequation);
	  t_dst->eq[i]=(void*)eq;
	  CopyEquation(eq,(Tequation*)t_src->eq[i]);
	}
    }
}

void AddVariable2Term(unsigned int id,Tterm *t)
{  
  if (t->nItems==t->maxItems)
    {
      MEM_DUP(t->id,t->maxItems,unsigned int);
      MEM_EXPAND(t->eq,t->maxItems,void*);
    }

  t->id[t->nItems]=id;
  
  t->eq[t->nItems]=NULL;
  
  t->nItems++;
}

void AddEquation2Term(Tequation *eq,Tterm *t)
{
  Tequation *eqAux;
  
  if (t->nItems==t->maxItems)
    {
      MEM_DUP(t->id,t->maxItems,unsigned int);
      MEM_EXPAND(t->eq,t->maxItems,void*);
    }
  
  t->id[t->nItems]=NO_UINT;
  
  NEW(eqAux,1,Tequation);
  t->eq[t->nItems]=(void*)eqAux;
  CopyEquation(eqAux,eq);

  t->nItems++;
}

double EvaluateTerm(double *varValues,Tterm *t)
{
  unsigned int i;
  double v;

  v=1.0;
  for(i=0;i<t->nItems;i++)
    {
      if (t->eq[i]==NULL)
	v*=varValues[t->id[i]];
      else
	v*=EvaluateEquation(varValues,(Tequation*)t->eq[i]);
    }
  return(v);
}

void PrintTerm(FILE *f,char **varNames,Tterm *t)
{
  unsigned int i;

  for(i=0;i<t->nItems;i++)
    { 
      if (t->eq[i]==NULL)
	{
	  if (varNames!=NULL)
	    fprintf(f,"%s",varNames[t->id[i]]);
	  else
	    fprintf(f,"v_%u",t->id[i]);
	}
      else
	{
	  Tequation *eq;

	  eq=(Tequation *)t->eq[i];

	  if ((eq->nTerms>1)||(fabs(eq->value)>ZERO))
	    {
	      fprintf(f,"(");
	      PrintEquation(f,varNames,eq);
	      fprintf(f,")");
	    }
	  else
	    PrintEquation(f,varNames,eq);
	}
      
      if (i<t->nItems-1)
	fprintf(f,"*");
    }
} 

void DeleteTerm(Tterm *t)
{
  unsigned int i;

  for(i=0;i<t->nItems;i++)
    {
      if (t->eq[i]!=NULL)
	{
	  DeleteEquation((Tequation*)t->eq[i]);
	  free(t->eq[i]);
	}
    }
  free(t->id);
  free(t->eq);
}

/*******************************************************************/
/*******************************************************************/
/*******************************************************************/

void InitEquation(Tequation *eq)
{
  eq->cmp=NOCMP;
  eq->value=0.0;

  eq->maxTerms=INIT_NUM_TERMS;
  eq->nTerms=0;
  NEW(eq->scale,eq->maxTerms,double);
  NEW(eq->terms,eq->maxTerms,Tterm *);

  InitVarSet(&(eq->vars));
}

void CopyEquation(Tequation *eq_dst,Tequation *eq_orig)
{
  unsigned int i;

  eq_dst->cmp=eq_orig->cmp;
  eq_dst->value=eq_orig->value;
  
  eq_dst->maxTerms=eq_orig->maxTerms;
  eq_dst->nTerms=eq_orig->nTerms;
  
  NEW(eq_dst->scale,eq_dst->maxTerms,double);
  NEW(eq_dst->terms,eq_dst->maxTerms,Tterm*);
  
  for(i=0;i<eq_orig->nTerms;i++)
    {
      eq_dst->scale[i]=eq_orig->scale[i];
      
      NEW(eq_dst->terms[i],1,Tterm);
      CopyTerm(eq_dst->terms[i],eq_orig->terms[i]);
    }

  CopyVarSet(&(eq_dst->vars),&(eq_orig->vars));
}

void AccumulateEquations(Tequation *eqn,double ct,Tequation *eq)
{
  if (fabs(ct)>ZERO)
    {
      unsigned int i;

      if (eq->maxTerms<eq->nTerms+eqn->nTerms)
	{
	  unsigned int m;
      
	  m=2*(eq->nTerms+eqn->nTerms);
	  MEM_EXPAND(eq->scale,m,double);
	  MEM_EXPAND(eq->terms,m,Tterm*);
	  eq->maxTerms=m;
	}

      for(i=0;i<eqn->nTerms;i++)
	{
	  eq->scale[eq->nTerms+i]=ct*eqn->scale[i];
	  NEW(eq->terms[eq->nTerms+i],1,Tterm);
	  CopyTerm(eq->terms[eq->nTerms+i],eqn->terms[i]);
	}
      eq->nTerms+=eqn->nTerms;

      eq->value+=(ct*eqn->value);

      UnionVarSet(&(eqn->vars),&(eq->vars));
    }
}

void ProductEquations(Tequation *eq1,Tequation *eq2,Tequation *eqOut)
{
  unsigned int i;
  Tequation *eq;

  if (VarSetIntersect(&(eq1->vars),&(eq2->vars)))
    Error("Non-multilinear equation");
  
  if (eq1->cmp!=eq2->cmp)
    Error("Product of equations of different type in ProductEquations");

  /* In many cases eq1 is just a constant of a single variable -> perform a simple product*/

  if ((eqOut==eq1)||(eqOut==eq2))
    {
      NEW(eq,1,Tequation);
    }
  else
    eq=eqOut;
  
  if (eq1->nTerms==0)
    {
      /* alpha * eq2 */
      if (fabs(eq1->value)>ZERO)
	{
	  CopyEquation(eq,eq2);
	  for(i=0;i<eq->nTerms;i++)
	    eq->scale[i]*=eq1->value;
	  eq->value*=eq1->value;
	}
      else
	{
	  /* 0 * eq2 */
	  InitEquation(eq);
	  eq->cmp=eq2->cmp;
	}
    }
  else
    {
      if ((eq1->nTerms==1)&&(eq1->terms[0]->nItems==1)&&(eq1->terms[0]->id[0]!=NO_UINT)&&(fabs(eq1->value)<ZERO))
	{
	  /* x * eq2 */
	  unsigned int nv;

	  nv=eq1->terms[0]->id[0]; /* index of the variable in the simple equation (eq1) */
	  
	  CopyEquation(eq,eq2);
	  
	  for(i=0;i<eq->nTerms;i++)
	    AddVariable2Term(nv,eq->terms[i]);
	  
	  if (fabs(eq->value)>ZERO)
	    {
	      if (eq->nTerms==eq->maxTerms)
		{
		  MEM_DUP(eq->terms,eq->maxTerms,Tterm*);
		}
	      
	      NEW(eq->terms[eq->nTerms],1,Tterm);
	      InitTerm(eq->terms[eq->nTerms]);
	      
	      eq->scale[eq->nTerms]=eq->value;
	      AddVariable2Term(nv,eq->terms[eq->nTerms]);
	      
	      eq->nTerms++;
	    }

	  if ((eq->nTerms>0)||(fabs(eq->value)>ZERO))
	    AddVariable2Set(nv,&(eq->vars));
	  
	  eq->value=0;
	}
      else
	{
	  /* product of two general equations */
	  InitEquation(eq);
	  eq->cmp=eq->cmp;
  
	  NEW(eq->terms[0],1,Tterm);
	  InitTerm(eq->terms[eq->nTerms]);
  
	  eq->scale[0]=1.0;
	  AddEquation2Term(eq2,eq->terms[0]);
	  AddEquation2Term(eq1,eq->terms[0]);
	  
	  eq->nTerms=1;
	  
	  UnionVarSet(&(eq1->vars),&(eq->vars));
	  UnionVarSet(&(eq2->vars),&(eq->vars));
	}
    }
      
  if ((eqOut==eq1)||(eqOut==eq2))
    {
      DeleteEquation(eqOut);
      CopyEquation(eqOut,eq);
      DeleteEquation(eq);
      free(eq);
    }
}

void ResetEquation(Tequation *eq)
{
  unsigned int i;
  
  eq->cmp=NOCMP;
  eq->value=0.0;

  for(i=0;i<eq->nTerms;i++)
    {
      DeleteTerm(eq->terms[i]);
      free(eq->terms[i]);
    }

  eq->nTerms=0;

  ResetVarSet(&(eq->vars));
}

void SetEquationCmp(unsigned int cmp,Tequation *eq)
{
  if ((cmp!=EQU)&&(cmp!=LEQ)&&(cmp!=GEQ))
    Error("Unknow equation cmp");

  eq->cmp=cmp;
}

void SetEquationValue(double v,Tequation *eq)
{
  eq->value=v;
}

unsigned int GetEquationCmp(Tequation *eq)
{
  return(eq->cmp);
}

double GetEquationValue(Tequation *eq)
{
  return(eq->value);
}

Tvariable_set *GetEquationVariables(Tequation *eq)
{
  return(&(eq->vars));
}

unsigned int GetEquationNumVariables(Tequation *eq)
{
  return(VariableSetSize(GetEquationVariables(eq)));
}

void EquationFromVariable(unsigned int i,Tequation *eq)
{
  InitEquation(eq);
  eq->nTerms=1;
  eq->scale[0]=1.0;
  NEW(eq->terms[0],1,Tterm);
  InitTerm(eq->terms[0]);
  AddVariable2Term(i,eq->terms[0]);
  AddVariable2Set(i,&(eq->vars));
}

void EquationFromSubExpression(Tequation *seq,Tequation *eq)
{
  InitEquation(eq);
  eq->nTerms=1;
  eq->scale[0]=1.0;
  NEW(eq->terms[0],1,Tterm);
  InitTerm(eq->terms[0]);
  AddEquation2Term(seq,eq->terms[0]);
  UnionVarSet(&(seq->vars),&(eq->vars));
}

double EvaluateEquation(double *varValues,Tequation *eq)
{
  double v;
  unsigned int i;

  #if (_DEBUG>2)
    if (eq->cmp==NOCMP)
      Error("Evaluation of an undefined equation");
  #endif

  v=eq->value;
  for(i=0;i<eq->nTerms;i++)
    {
      if (eq->terms[i]->nItems==0)
	v+=eq->scale[i];
      else
	v+=(eq->scale[i]*EvaluateTerm(varValues,eq->terms[i]));
    }
    
  return(v);
}

void PrintEquation(FILE *f,char **varNames,Tequation *eq)
{
  signed int i;

  /* We print in reverse order because we accumulate in reverse order too */
  for(i=eq->nTerms-1;i>=0;i--)
    {
      if (eq->terms[i]->nItems==0)
	{
	  fprintf(f,"%+.16g",eq->scale[i]);
	}
      else
	{
	  if (fabs(eq->scale[i]-1.0)>ZERO)
	    {
	      /* We have a scale different from -1*/
	      if (fabs(eq->scale[i]+1.0)>ZERO)
		{
		  /* We have a scale different form -1 / +1 */
		  fprintf(f,"%+.16g*",eq->scale[i]);
		}
	      else
		{
		  if (i<(signed int)eq->nTerms-1)
		    fprintf(f,"-");
		}
	    }
	  else
	    {
	      if (i<(signed int)eq->nTerms-1)
		fprintf(f,"+");
	    }
      
	  PrintTerm(f,varNames,eq->terms[i]); 
	}	
    }
  if (fabs(eq->value)>ZERO)
    fprintf(f,"%+.16g",eq->value);
  
  switch(eq->cmp)
    {
    case EQU:
      fprintf(f," = 0;\n");
      break;
    case GEQ:
      fprintf(f," >= 0;\n");
      break;
    case LEQ:
      fprintf(f," <= 0;\n");
      break;
    }
}

void DeleteEquation(Tequation *eq)
{  
  ResetEquation(eq);
  free(eq->scale);
  free(eq->terms);
  
  DeleteVarSet(&(eq->vars));
}
