#include "loops.h"

#include "defines.h"


/** \file loops.c
    \brief Implementation of the Tloops functions.

    Implementation of the Tloops functions.

    \sa Tloops, loops.h.
*/

void InitLoops(Tloops *lps)
{
  lps->n=0;
  lps->m=INIT_NUM_LOOPS;
  NEW(lps->ts,lps->m,Ttransforms*);
  
  lps->neq=0;
  InitVarSet(&(lps->vars));
}


void CopyLoops(Tloops *lps_dst,Tloops *lps_src)
{
  unsigned int i;
  
  lps_dst->m=lps_src->m;
  NEW(lps_dst->ts,lps_dst->m,Ttransforms*);
  lps_dst->n=lps_src->n;
  for(i=0;i<lps_src->n;i++)
    {
      NEW(lps_dst->ts[i],1,Ttransforms);
      CopyTransforms(lps_dst->ts[i],lps_src->ts[i]);
    }

  CopyVarSet(&(lps_dst->vars),&(lps_src->vars));
  lps_dst->neq=lps_src->neq;
}

void AddLoop(Ttransforms *ts,Tloops *lps)
{
  if (lps->n==lps->m)
    {
      MEM_DUP(lps->ts,lps->m,Ttransforms*);
    }

  NEW(lps->ts[lps->n],1,Ttransforms);
  CopyTransforms(lps->ts[lps->n],ts);

  UnionVarSet(GetTransformsVariables(ts),&(lps->vars));
  lps->neq+=NTransformsEquations(ts);
  
  lps->n++;
}

inline unsigned int NLoops(Tloops *lps)
{
  return(lps->n);
}

inline Tvariable_set *GetVariablesInLoops(Tloops *lps)
{
  return(&(lps->vars));
}

inline unsigned int NEquationsInLoops(Tloops *lps)
{ 
  return(lps->neq);
}

inline unsigned int NVariablesInLoops(Tloops *lps)
{ 
  return(VariableSetSize(&(lps->vars)));
}

inline Ttransforms *GetLoop(unsigned int nl,Tloops *lps)
{
  if (nl<lps->n)
    return(lps->ts[nl]);
  else
    return(NULL);
}

void EvaluateLoops(double *val,signed int *offs,double *l,Tloops *lps)
{
  unsigned int i,k;

  k=0;
  for(i=0;i<lps->n;i++)
    k+=EvaluateTransforms(val,offs,&(l[k]),lps->ts[i]);
}

void SimplifyLoops(unsigned int simplify,
		   signed int *offs,Tloops *lps_simp,Tloops *lps)
{

  CopyLoops(lps_simp,lps);

  if (simplify>0)
    {
      unsigned int i;
	  
      lps_simp->neq=0;
      ResetVarSet(&(lps_simp->vars));
      for(i=0;i<lps_simp->n;i++)
	{	  
	  if (!SimplifyTransforms(simplify,offs,lps_simp->ts[i]))
	    {
	      if (simplify==3)
		SimplifyTransforms(simplify-1,offs,lps_simp->ts[i]);
	    }
	    
	  UnionVarSet(GetTransformsVariables(lps_simp->ts[i]),&(lps_simp->vars));
	  lps_simp->neq+=NTransformsEquations(lps_simp->ts[i]);
	  
	
	  #if (_DEBUG>1)
	    if (offs==NULL)
	      {
		fprintf(stderr,"Loops %u -> ",i);
		if (SimplifiedTransforms(lps_simp->ts[i]))
		  {
		    fprintf(stderr,"Removed vars: ");
		    PrintVarSet(stderr,NULL,GetTransformsRemovedVariables(lps_simp->ts[i]));
		    fprintf(stderr,"\n");
		  }
		else
		  fprintf(stderr,"Non simplified\n");
	      }
	  #endif
	}
    }
}

void ReSimplifyLoops(signed int *offs,Tloops *lps_simp,Tloops *lps)
{
  unsigned int i;
  
  CopyLoops(lps_simp,lps);
  
  for(i=0;i<lps_simp->n;i++)
    {
      #if (_DEBUG>1)
        fprintf(stderr,"Resimplification of loop %u - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - \n",i);
      #endif
      ReSimplifyTransforms(offs,lps_simp->ts[i]);
    }
}

void PrintLoops(FILE *f,char **varnames,Tloops *lps)
{
  unsigned int i;

  for(i=0;i<lps->n;i++)
    {
      fprintf(f,"%% Loop equation: %u\n",i);
      PrintTransforms(f,varnames,lps->ts[i]);
      fprintf(f,"\n");
    }
}

void PrintLoopsEquations(FILE *f,char **varnames,Tloops *lps)
{
  unsigned int i;
  
  for(i=0;i<lps->n;i++)
    {
      fprintf(f,"%% Loop equation: %u\n",i);
      fprintf(f,"%% ");
      PrintTransforms(f,varnames,lps->ts[i]);
      fprintf(f,"\n");
      PrintTransformsEquations(f,varnames,lps->ts[i]);
      fprintf(f,"\n\n");
      
    }
}

void PrintLoopsMatrixVector(FILE *f,char **varnames,Tloops *lps)
{
  unsigned int i;
  
  for(i=0;i<lps->n;i++)
    PrintTransformsMatrixVector(f,varnames,lps->ts[i]);
}

void DeleteLoops(Tloops *lps)
{
  unsigned int i;

  for(i=0;i<lps->n;i++)
    {
      DeleteTransforms(lps->ts[i]);
      free(lps->ts[i]);
    }
  DeleteVarSet(&(lps->vars));
  free(lps->ts);
}
