#include "parameters.h"

#include "error.h"
#include "filename.h"
#include "defines.h"

#include <stdlib.h>
#include <strings.h>
#include <math.h>

/** 
   \file parameters.c

   \brief Implementation of the functions operating on Tparameters.

   Implementation of functions to deal with Tparameters.

   \sa Tparameters, parameters.h.
*/

/** 
   \brief Reads a parameter set from a file.

   Reads a parameter set from a file.

   \param file The name of the file.
   \param p The parameter set to update.

   \return TRUE if the the file with the parameter exists.
*/

boolean ReadParameters(char *file,Tparameters *p);

/*
 * Inits a parameter structure and defines all parameters equal to 0
 */
void InitParameters(Tparameters *p)
{
  unsigned int i;

  for(i=0;i<NPARAMETERS;i++)
    {
      (*p)[i].name=NULL;
      (*p)[i].value=0.0;
      (*p)[i].timesSet=0;
    }
}

/*
 * Inits a parameter structure from the given file
 */
void InitParametersFromFile(char *file,Tparameters *p)
{
  Tfilename dp;
  unsigned int i;
  boolean ok;

  InitParameters(p);

  
  CreateFileName(_CUIK_SUITE_MAIN_DIR,DEFAULT_PARAMS,NULL,PARAM_EXT,&dp);
  ok=ReadParameters(GetFileFullName(&dp),p);
  DeleteFileName(&dp);
  if (!ok)
    {
      /* try to read from a system directory (at a user given folder) */
      CreateFileName(_PARAM_PREFIX,DEFAULT_PARAMS,NULL,PARAM_EXT,&dp);
      ok=ReadParameters(GetFileFullName(&dp),p);
      DeleteFileName(&dp);
      if (!ok)
	{
	  /* try to read from a system directory (at a user general folder) */
	  CreateFileName("/usr",DEFAULT_PARAMS,NULL,PARAM_EXT,&dp);
	  if (!ReadParameters(GetFileFullName(&dp),p))
	    Error("Can not find the default parameter file");
	  DeleteFileName(&dp);
	}
    }

  /* The default initialization does not count. Reset the 'timesSet' */
  for(i=0;i<NPARAMETERS;i++)
    (*p)[i].timesSet=0;

  /* The existence of the parameter file associated with the
     problem is compulsatory */
  if (!ReadParameters(file,p))
    {
      char ErrorText[500];

      sprintf(ErrorText,"File %s does not exists. Using the default parameters",file);
      Warning(ErrorText);

      /* fix the 'timeSet' */
      for(i=0;i<NPARAMETERS;i++)
	(*p)[i].timesSet=1;
    }
}

/*
 * Returns the value of parameter number 'n'
 */
double GetParameter(unsigned int n,Tparameters *p)
{
  #if (_DEBUG>0)
    if (n<NPARAMETERS)
      {
	if ((*p)[n].name==NULL)
	  Error("Undefined parameter");
	else
	  return((*p)[n].value);
      }
    else
      Error("Number of parameter out of range");
    return(0.0);
  #else
    return((*p)[n].value);
  #endif 
}

/*
 * Returns the parameter identifier given the parameter name.
 */
unsigned int GetParameterID(char *name,Tparameters *p)
{
  boolean found;
  unsigned int i;

  found=FALSE;
  i=0;
  while((!found)&&(i<NPARAMETERS))
    {
      found=(((*p)[i].name!=NULL)&&(strcasecmp((*p)[i].name,name)==0));
      if (!found) i++;
    }
  if (found)
    return(i);
  else
    return(NO_UINT);
}

/*
 * Check is a parameter is already set.
 */
boolean ParameterSet(unsigned int n,Tparameters *p)
{
  return(((*p)[n].name!=NULL)&&((*p)[n].timesSet>0));
}

/*
 * Sets a new parameter with identifier 'n', with name 'name', and with value 'v'
 */
void SetParameter(unsigned int n,char *name,double v,Tparameters *p)
{
  if (n<NPARAMETERS)
    {
      if ((*p)[n].name!=NULL)
	{
	  if ((*p)[n].timesSet>0)
	    Warning("Redefined parameter");
	  ChangeParameter(n,v,p);
	  (*p)[n].timesSet++;
	}
      else
	{
	  (*p)[n].name=name;
	  (*p)[n].value=v;
	  (*p)[n].timesSet=1;
	}
    }
  else
    Error("Number of parameter out of range");
}

/*
 * Changes the value of parameter 'n'
 */
void ChangeParameter(unsigned int n,double v,Tparameters *p)
{
  if (n<NPARAMETERS)
    {
      if ((*p)[n].name==NULL)
	Error("Changing a non-defined parameter");
      
      (*p)[n].value=v;
    }
  else
    Error("Number of parameter out of range");
}

/*
 * Prints a set of parameters.
 * Only those explicitly used are shown.
 */
void PrintParameters(FILE *f,Tparameters *p)
{
  unsigned int i,v;
  
  fprintf(f,"%% **************************************************\n");
  fprintf(f,"%% COMPILATION FLAGS:\n");
  fprintf(f,"%%   DEBUG: %u\n",_DEBUG);
  fprintf(f,"%%   SIMPLEX AVAILABLE: %u\n",SIMPLEX_AVAILABLE);
  #ifdef _CBLAS
    fprintf(f,"%%   CBLAS: 1\n");
  #else
    fprintf(f,"%%   CBLAS: 0\n");
  #endif
  #ifdef _LAPACK
    fprintf(f,"%%   LAPACK: 1\n");
  #else
     #ifdef _LAPACKE
      fprintf(f,"%%   LAPACKE: 1\n");
    #else
      fprintf(f,"%%  ERROR: LAPACK missing\n");
    #endif
  #endif
  fprintf(f,"%% PARAMETERS:\n");
  for(i=0;i<NPARAMETERS;i++)
    {
      if ((*p)[i].name!=NULL)
	{
	  switch(i)
	    {
	    case CT_REDUCTION:
	      if ((*p)[i].value==RED_SIMPLEX)
		fprintf(f,"   %s = SIMPLEX\n",(*p)[i].name);
	      else
		fprintf(f,"   %s = TRAPEZOID\n",(*p)[i].name);
	      break;
	    case CT_CLEAN:
	      v=(unsigned int)round((*p)[i].value);
	      switch (v)
		{
		case NO_CLEAN:
		  fprintf(f,"   %s = OFF\n",(*p)[i].name);
		  break;
		case CLEAN_SIMPLEX:
		  fprintf(f,"   %s = SIMPLEX\n",(*p)[i].name);
		  break;
		default:
		  Error("Unknow value for parameter CLEAN");
		  break;
		}
	      break;
	    case CT_VERIFY:
	      v=(unsigned int)round((*p)[i].value);
	      switch (v)
		{
		case VERIFY_MIRANDA:
		  fprintf(f,"   %s = MIRANDA\n",(*p)[i].name);
		  break;
		case VERIFY_NEWTON:
		  fprintf(f,"   %s = MIRANDA_NEWTON\n",(*p)[i].name);
		  break;
		case VERIFY_NEWTON_REPLACE:
		  fprintf(f,"   %s = MIRANDA_NEWTON_REPLACE\n",(*p)[i].name);
		  break;
		default:
		  Error("Unknow value for parameter VERIFY");
		  break;
		}
	      break;
	    case CT_SAVE_ANGLES:
	      if ((*p)[i].value>0.5)
		fprintf(f,"   %s = ON\n",(*p)[i].name);
	      else
		fprintf(f,"   %s = OFF\n",(*p)[i].name);
	      break;
	    case CT_SIMPLIFY:
	      {
		unsigned int s=round((*p)[i].value);
		
		fprintf(f,"   %s = %u\n",(*p)[i].name,s);
	      }
	      break;
	    default:
	      fprintf(f,"   %s = %g\n",(*p)[i].name,(*p)[i].value);
	    }
	}
    }
}

void DeleteParameters(Tparameters *p)
{
}
 
