%{ 
/** \cond */
#include "readcuiksystemtypes.h"
#include "cuiksystem.h"

#include "boolean.h"
#include "error.h"
#include "interval.h"
#include "constants.h"
#include "equations.h"
#include "equation.h"
#include "variable.h"
#include "variable_set.h"
#include "defines.h"
#include "parameters.h"
#include "filename.h"

#include <stdlib.h> 
#include <stdio.h>
#include <string.h>
#include <math.h>

  /*Lex and Yacc variables*/
  extern FILE *Readcuiksystemin; 

  /*Lex and Yacc functions*/
  int Readcuiksystemlex(void); 

  /*Our own variables*/
  extern unsigned int RCSline; /*line number currently processed (incremented by the LEX processor)*/
  
  Tparameters *rcs_parameters;

  /*Global pointer to allow the different parts of the parser to acces the equations being initialized*/
  Tconstants rcs_constants;
  Tequations rcs_equations;
  Tvariables rcs_variables;
  
%}

%union 
{
  char *id; 
  int int_number;
  double real_number;
  Texpr expr;
  Tequation equation;
} 
 
%start problem

%token _CONSTANTS _VARS _OFFSETS _EQS _EQU _LEQ _GEQ _ASSIGN _INF _PI _SIN _COS _TAN _COT _ACOS _ASIN _ATAN _ATAN2 _EXP _PWP _SQRT _ABS _DIFF _PARAMETER _PRINT _EQUAL _LESS _GREATER _NOT_EQUAL

%token <id> _IDENTIFIER
%token <int_number> _INTEGER
%token <real_number> _REAL

%type <expr> expr
%type <int_number> cmp 
%type <int_number> var_type 
%type <int_number> power
%type <real_number> sign
%type <real_number> opt_sign
%type <equation> equation
%type <equation> monomials
%type <equation> more_monomials
%type <equation> monomial
%type <equation> item


%left _MIN_PRECEDENCE
%left _EQUAL _NOT_EQUAL _LEQ _GEQ _LESS _GREATER
%left '+' '-' 
%left '*' '/'
%left '^'
%left '\''
%right _MAX_PRECEDENCE 
 
%% 

problem: ct_defs var_defs offsets_defs system_eqs 
       ;

ct_defs : _CONSTANTS ct_list  
        |
        ;
         
ct_list : ct_definition ct_list
        |
        ;

ct_definition: _IDENTIFIER _ASSIGN expr
               {
		 if (GetConstantWithName($1,&rcs_constants)!=NO_UINT)
		   {
		     char s[250];
		     
		     sprintf(s,"Duplicated constant %.150s",$1);
		     ReadcuiksystemSemanticError(s);
		   }

		 if ($3.id!=NULL)
		   ReadcuiksystemSemanticError("Constant assigned to a variable");

	         AddConstant($1,$3.value,&rcs_constants);
		 free($1);
               }
             | _PRINT _IDENTIFIER
	       {
		 unsigned int nc;
		 
		 nc=GetConstantWithName($2,&rcs_constants);
		 if (nc==NO_UINT)
		   {
		     char ms[250];
		     
		     sprintf(ms,"Undefined constant: %.150s",$2);
		     ReadcuiksystemSemanticError(ms);
		   }
		 fprintf(stderr,"  %s = %.12f\n",$2,GetConstantValue(nc,&rcs_constants));
		 free($2);
	       }
             ;


expr : '+' expr    %prec _MAX_PRECEDENCE
         {
	   if ($2.id!=NULL)
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$2.value;
         }
        | '-' expr    %prec _MAX_PRECEDENCE
         {
	   if ($2.id!=NULL)
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=-$2.value;
         }
        | expr '+' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$1.value+$3.value;
         }
        | expr '-' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$1.value-$3.value;
         }
        | expr '*' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$1.value*$3.value;
         }
        | expr '^' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=pow($1.value,$3.value);
         }
        | expr '/' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   if ($3.value==0.0)
	     ReadcuiksystemSemanticError("Division by zero");
	   
	   $$.id=NULL;
	   $$.value=$1.value/$3.value;
         }
        | '(' expr ')'
         {
	   if ($2.id!=NULL)
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$2.value;
         }
        | '(' expr '?' expr ':' expr ')'
          {
	    if (($2.id!=NULL)||($4.id!=NULL)||($6.id!=NULL))
	     ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    if ($2.value)
	      $$.value=$4.value;
	    else
	      $$.value=$6.value;
	  }
        | expr _EQUAL expr 
          {
	    if (($1.id!=NULL)||($3.id!=NULL))
	     ReadcuiksystemSemanticError("No operation is allowed with variables");

	    $$.id=NULL;
	    $$.value=($1.value==$3.value);
	  }
        | expr _NOT_EQUAL expr 
          {
	    if (($1.id!=NULL)||($3.id!=NULL))
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=($1.value!=$3.value);
	  }
        | expr _LEQ expr  
	  {
	    if (($1.id!=NULL)||($3.id!=NULL))
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=($1.value<=$3.value);
	  }
        | expr _GEQ expr     
          {
	    if (($1.id!=NULL)||($3.id!=NULL))
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=($1.value>=$3.value);
	  }
        | expr _LESS expr        
	  {
	    if (($1.id!=NULL)||($3.id!=NULL))
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=($1.value<$3.value);
	  }
        | expr _GREATER expr   
	  {
	    if (($1.id!=NULL)||($3.id!=NULL))
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=($1.value>$3.value);
	  }
        | _PI 
          {
	    $$.id=NULL;
	    $$.value=M_PI;
	  }
        | _INF
          {
	    $$.id=NULL;
	    $$.value=INF;
	  }
        | _SIN '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=sin($3.value);
	  }
        | _COS '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=cos($3.value);
	  }
        | _TAN '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=tan($3.value);
	  }
        | _COT '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=tan(M_PI_2-$3.value);
	  }
        | _ASIN '(' expr ')'
	  {
	    if ($3.id!=NULL)
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=asin($3.value);
	  }
        | _ACOS '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=acos($3.value);
	  }
        | _ATAN '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=atan($3.value);
	  }
        | _ATAN2 '(' expr ',' expr ')'
          {
	    if (($3.id!=NULL)||($5.id!=NULL))
	      ReadcuiksystemSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=atan2($3.value,$5.value);
	  }
        | _EXP '(' expr ')'
         {
	   if ($3.id!=NULL)
	     ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	   $$.id=NULL;
	   $$.value=exp($3.value);
	 }
        | _PWP '(' expr ')'
         {
	   if ($3.id!=NULL)
	     ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	   $$.id=NULL;
	   if ($3.value<0.0)
	     $$.value=$3.value*$3.value;
	   else
	     $$.value=0.0;
	 }
        | _SQRT '(' expr ')'
         {
	   if ($3.id!=NULL)
	     ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	   $$.id=NULL;
	   $$.value=sqrt($3.value);
	 }
        | _ABS '(' expr ')'
         {
	   if ($3.id!=NULL)
	     ReadcuiksystemSemanticError("No operation is allowed with variables");
	   
	   $$.id=NULL;
	   $$.value=fabs($3.value);
	 }
        | _PARAMETER '(' _IDENTIFIER ')'
          {
	    unsigned int n;
	    
	    n=GetParameterID($3,rcs_parameters);
	    if (n==NO_UINT)
	      {
		char ms[250];
		
		sprintf(ms,"Undefined parameter: %.150s",$3);
		ReadcuiksystemSemanticError(ms);
	      }
	    
	    $$.id=NULL;
	    $$.value=GetParameter(n,rcs_parameters);
	  }
        | _IDENTIFIER
         {
	   unsigned int nc;

	   nc=GetConstantWithName($1,&rcs_constants);

	   if (nc!=NO_UINT)
	     {
	       $$.id=NULL;
	       $$.value=GetConstantValue(nc,&rcs_constants);
	     }
	   else
	     {
	       if (GetVariableID($1,&rcs_variables)==NO_UINT) 
		 {
		   char s[250];
		   
		   sprintf(s,"Undefined variable %.150s",$1);
		   ReadcuiksystemSemanticError(s);
		 }
	       else
		 $$.id=$1;
	     }
	 }
        | _INTEGER
         {
	   $$.id=NULL;
	   $$.value=(double)$1;
         } 
        | _REAL
         {
	   $$.id=NULL;
	   $$.value=$1;
         }
        ; 

var_defs : _VARS
           range_list
         ;

range_list: range_definition range_list
          |
          ;

range_definition : _IDENTIFIER var_type '[' expr ',' expr ']'
                  {
		    Tinterval i_user; 
		    Tvariable v;
		    
		    if (($4.id!=NULL)||($6.id!=NULL))
		      ReadcuiksystemSemanticError("Can not define a range with variables");

		    if (GetConstantWithName($1,&rcs_constants)!=NO_UINT)  
		      {
			char s[250];
			
			sprintf(s,"Constant %.150s redefined as a variable",$1);
			ReadcuiksystemSemanticError(s);
		      }

		    if (GetVariableID($1,&rcs_variables)!=NO_UINT) 
		      {
			char s[250];
			
			sprintf(s,"Repeated range declaration for variable %s",$1);
			ReadcuiksystemSemanticError(s);
		      }
		    
		    NewInterval($4.value,$6.value,&i_user);
		    
		    if (EmptyInterval(&i_user))
		      ReadcuiksystemSemanticError("Empty Interval");


		    NewVariable($1,$2,&v);
		    
		    SetVariableInterval(&i_user,&v);
		    
		    AddVariable(&v,&rcs_variables);

		    DeleteVariable(&v);
		    free($1);
		  }
                 ;

var_type : ':'
           {
	     $$=VAR_TRANS;
	   }
         |
           '~'
           {
	     $$=VAR_ROT;
	   }
         ;

offsets_defs : _OFFSETS
               offset_list
             |
             ;

offset_list : offset_definition offset_list
            |
            ;

offset_definition: _IDENTIFIER ':' _IDENTIFIER
		  {
		    Tvariable v;
		    unsigned int nv;
		    
		    if (GetConstantWithName($1,&rcs_constants)!=NO_UINT)  
		      {
			char s[250];
			
			sprintf(s,"Constant %.150s redefined as a variable",$1);
			ReadcuiksystemSemanticError(s);
		      }

		    if (GetVariableID($1,&rcs_variables)!=NO_UINT) 
		      {
			char s[250];
			
			sprintf(s,"Repeated declaration of variable %s",$1);
			ReadcuiksystemSemanticError(s);
		      }

		    nv=GetVariableID($3,&rcs_variables);
		    if (nv==NO_UINT) 
		      {
			char s[250];
			
			sprintf(s,"Unknow variable in offset %s",$3);
			ReadcuiksystemSemanticError(s);
		      }
		    else
		      {
			Tvariable *sv;
			
			sv=GetVariable(nv,&rcs_variables);
			if (IsOffsetVariable(sv))
			  {
			    char s[250];
			
			    sprintf(s,"Can not shift an offset variable (%s)",$3);
			    ReadcuiksystemSemanticError(s);
			  }
		      }

		    NewOffsetVariable($1,nv,&v);
		    		    
		    AddVariable(&v,&rcs_variables);

		    DeleteVariable(&v);
		    free($1);
		  }
                 ;

system_eqs : _EQS
             eq_list
           |
           ;

eq_list: equation 
         {
	   AddEquation(&($1),&rcs_equations);
	   DeleteEquation(&($1));
	 }
         eq_list
         |
         ;

equation: monomials cmp monomials ';'
          {
	    CopyEquation(&($$),&($1));
	    AccumulateEquations(&($3),-1.0,&($$));
	    SetEquationCmp($2,&($$));

	    DeleteEquation(&($1));
	    DeleteEquation(&($3));
	  }
          ;

monomials : opt_sign monomial more_monomials
          {
	    CopyEquation(&($$),&($3));
	    
	    AccumulateEquations(&($2),$1,&($$));
	    
	    DeleteEquation(&($2));
	    DeleteEquation(&($3));
	  }
        ;

more_monomials: sign monomial more_monomials  
              {
		CopyEquation(&($$),&($3));

		AccumulateEquations(&($2),$1,&($$));
		
	        DeleteEquation(&($2));
		DeleteEquation(&($3));
	      }
            |
	      {
		InitEquation(&($$));
	      }
            ;

opt_sign : sign
           {
	     $$=$1;
	   }
         |
	   {
	     $$=+1.0;
	   }
         ;

sign : '+'
       {
	 $$=+1.0;
       }
     | '-'
       {
	 $$=-1.0;
       }
     ;

monomial : monomial '*' item
           {
	     ProductEquations(&($3),&($1),&($$));
	     
	     DeleteEquation(&($1));
	     DeleteEquation(&($3));
	   }
         | item
	   {
	     CopyEquation(&($$),&($1));
	     DeleteEquation(&($1));
	   }
         ;

item : _REAL power
       {
	 InitEquation(&($$));
	 
	 if ($2==1)
	   SetEquationValue($1,&($$));
	 else
	   SetEquationValue(pow($1,(double)$2),&($$));
       } 
     | _INTEGER power
       {
	 InitEquation(&($$));
	 
	 if ($2==1)
	   SetEquationValue((double)$1,&($$));
	 else
	   SetEquationValue(pow((double)$1,(double)$2),&($$));
       } 
     | _SIN '(' expr ')' power
       {
	 InitEquation(&($$));
	 
	 if ($3.id!=NULL)
	   {
	     unsigned int id;
	     
	     id=GetConstantWithName($3.id,&rcs_constants);
	     if (id!=NO_UINT) /*a known constant*/
	       SetEquationValue(sin(GetConstantValue(id,&rcs_constants)),&($$));
	     else
	       {
		 char s[250];
		 
		 sprintf(s,"Non-linear function: sin(%.150s)",$3.id);
		 ReadcuiksystemSemanticError(s);
	       }
	     free($3.id);
	   }
	 else
	   {
	     if ($5==1)
	       SetEquationValue(sin($3.value),&($$)); 
	     else
	       SetEquationValue(pow(sin($3.value),(double)$5),&($$));  
	   }
       }
     | _COS '(' expr ')' power
       {
	 InitEquation(&($$));
	 
	 if ($3.id!=NULL)
	   {
	     unsigned int id;
	     
	     id=GetConstantWithName($3.id,&rcs_constants);
	     if (id!=NO_UINT) /*a known constant*/
	       SetEquationValue(cos(GetConstantValue(id,&rcs_constants)),&($$));
	     else
	       {
		 char s[250];
		 
		 sprintf(s,"Non-linear function: cos(%.150s)",$3.id);
		 ReadcuiksystemSemanticError(s);
	       }
	     free($3.id);
	   }
	 else
	   {
	     if ($5==1)
	       SetEquationValue(cos($3.value),&($$)); 
	     else
	       SetEquationValue(pow(cos($3.value),(double)$5),&($$));  
	   }
       }
     | _TAN '(' expr ')' power
       {
	 InitEquation(&($$));
	 
	 if ($3.id!=NULL)
	   {
	     unsigned int id;
	     
	     id=GetConstantWithName($3.id,&rcs_constants);
	     if (id!=NO_UINT) /*a known constant*/
	       SetEquationValue(tan(GetConstantValue(id,&rcs_constants)),&($$));
	     else
	       {
		 char s[250];
		 
		 sprintf(s,"Non-linear function: tan(%.150s)",$3.id);
		 ReadcuiksystemSemanticError(s);
	       }
	     free($3.id);
	   }
	 else
	   {
	     if ($5==1)
	       SetEquationValue(tan($3.value),&($$)); 
	     else
	       SetEquationValue(pow(tan($3.value),(double)$5),&($$));  
	   }
       }
     | _EXP '(' expr ')' power
       {
	 InitEquation(&($$));
	 
	 if ($3.id!=NULL)
	   {
	     unsigned int id;
	     
	     id=GetConstantWithName($3.id,&rcs_constants);
	     if (id!=NO_UINT) /*a known constant*/
	       SetEquationValue(pow(GetConstantValue(id,&rcs_constants),(double)($5)),&($$));
	     else
	       {
		 char s[250];
		 
		 sprintf(s,"Non-linear function: exp(%.150s)",$3.id);
		 ReadcuiksystemSemanticError(s);
	       }
	     free($3.id);
	   }
	 else
	   SetEquationValue(pow(exp($3.value),(double)$5),&($$));
       }
     | _PI power
       {
	 InitEquation(&($$));
	 
	 SetEquationValue(pow(M_PI,(double)($2)),&($$));
       }
     | '(' monomials ')'
       {
	 EquationFromSubExpression(&($2),&($$));
	 DeleteEquation(&($2));
       }
     | _IDENTIFIER 
       {
	 unsigned int id;
	 unsigned int nc;

	 nc=GetConstantWithName($1,&rcs_constants);
	 if (nc!=NO_UINT)
	   {
	     InitEquation(&($$));
	     SetEquationValue(GetConstantValue(nc,&rcs_constants),&($$));
	   }
	 else
	   {
	     id=GetVariableID($1,&rcs_variables);
	     if (id!=NO_UINT) /*a known variable*/
	       EquationFromVariable(id,&($$));
	     else
	       {
		 char s[250];
		 
		 sprintf(s,"Unknown variable or constant %.150s",$1);
		 ReadcuiksystemSemanticError(s);
	       }
	   }
	 free($1);
       }
     ;

power : '^' _INTEGER
        {
	  $$=$2;
	}
      |
        {
	  $$=1;
	}
      ;

cmp : _EQU
      {
        $$=EQU;
      }
    | _LEQ
      {
        $$=LEQ;
      }
    | _GEQ
      {
        $$=GEQ;
      }
    ;

%%

#include "readcuiksystemtypes.h"

/** \endcond */

/*
 * Reads a file containing the constants definitions, kinematic equations to be solved, 
 * and the range constrains for the variables.
 */
unsigned int ReadCuikFile(Tparameters *p,char *filename,TCuikSystem **csa)
{
  unsigned int s,ns;

  ns=0;

  fprintf(stderr,"Reading cuik file: %s\n",filename);
  
  Readcuiksystemin=fopen(filename,"r");
  if (Readcuiksystemin) 
    {
      unsigned int no,nav;
      double of;
      
      /*Start the data structures*/
      InitConstants(&rcs_constants);
      InitVariables(&rcs_variables);
      InitEquations(&rcs_equations);
  
      rcs_parameters=p;
      /*Reset the lines numbering*/
      RCSline=1;

      /*and process the file*/
      Readcuiksystemparse();

      fclose(Readcuiksystemin);

      /* Define the cuiksystem structure from the parsed information */
      nav=NVariables(&rcs_variables); // All variables
      no=NOffsetVariables(&rcs_variables);
      ns=(1<<no);

      of=GetParameter(CT_OFFSET,p);// size of the assymmetry. Can be random too
	  
      NEW(*csa,ns,TCuikSystem);
      
      #pragma omp parallel for private(s) if (!_DEBUG)
      for(s=0;s<ns;s++)
	{
	  TCuikSystem *cs;
	  Tequation *eq;
	  Tvariable *var;
	  Tvariable_set *vars;
	  Tinterval *r;
	  unsigned int i,j,nv,k,v;
	  double *c,*low,*up;
	  
	  cs=&((*csa)[s]);

	  cs->ne=NEquations(&rcs_equations);
	  cs->nv=nav-no; // The variable set includes offset variables
	  
	  NEW(cs->varType,cs->nv,unsigned int);
	  for(i=0;i<cs->nv;i++)
	    {      
	      var=GetVariable(i,&rcs_variables);
	      cs->varType[i]=GetVariableType(var);
	      if (cs->varType[i]==VAR_OFFSET)
		Error("Offset variables must be at the end");
	    }
	  
	  // Space for the corners. Used to evaluate the equations (including offsets!!)
	  NEWZ(c,nav,double);

	  // Offsets
	  NEWZ(cs->offset,cs->nv,signed int); /* space for the offsets (one for variable). Default = No offset */
	  for(i=cs->nv,k=0;i<nav;i++,k++)
	    {
	      var=GetVariable(i,&rcs_variables);

	      if (s & (1<<k))
		{
		  cs->offset[ShiftedVariable(var)]=+1;
		  c[i]=+1.0; // Fix the offset variables for the evaluation
		}
	      else
		{
		  cs->offset[ShiftedVariable(var)]=-1;   
		  c[i]=-1.0; // Fix the offset variables for the evaluation
		}
	    }
	  
	  NEW(cs->nve,cs->ne,unsigned int);
	  NEW(cs->ve,cs->ne,unsigned int*);
	  NEW(cs->nc,cs->ne,unsigned int);
	  NEWZ(cs->usedVars,cs->nv,boolean);
	  for(i=0;i<cs->nv;i++)
	    cs->usedVars[i]=FALSE;
	  for(i=0;i<cs->ne;i++)
	    {
	      eq=GetEquation(i,&rcs_equations);
	      vars=GetEquationVariables(eq); 
	      k=VariableSetSize(vars);
	      cs->nve[i]=0;
	      NEW(cs->ve[i],k,unsigned int); // we allocate more space than needed but...
	      for(j=0;j<k;j++)
		{
		  nv=GetVariableN(j,vars);
		  var=GetVariable(nv,&rcs_variables);
		  if (GetVariableType(var)!=VAR_OFFSET)
		    {
		      if (nv>=cs->nv)
			Error("Offset variables are not at the end?");
		      // A translation/rotation variable
		      cs->ve[i][cs->nve[i]]=nv;
		      cs->nve[i]++;
		      cs->usedVars[nv]=TRUE;
		    }
		}
	      cs->nc[i]=(1<<cs->nve[i]);
	    }

	  NEW(low,cs->nv,double);
	  NEW(up,cs->nv,double);
	  
	  for(i=0;i<cs->nv;i++)
	    {
	      var=GetVariable(i,&rcs_variables);
	      r=GetVariableInterval(var);
      
	      if ((no>0)&&(of!=0.0)&&(cs->varType[i]==VAR_ROT))
		{
		  low[i]=A2T(-M_PI_2+of);
		  up[i] =A2T( M_PI_2+of);
		}
	      else
		{
		  low[i]=LowerLimit(r);
		  up[i]=UpperLimit(r);
		}
	    }
	  
	  NEW(cs->b0,1,TCuikBox);
	  InitCuikBox(low,up,cs->nv,cs->b0);

	  NEW(cs->f0,cs->ne,double*);
	  for(i=0;i<cs->ne;i++)
	    {
	      eq=GetEquation(i,&rcs_equations);

	      NEW(cs->f0[i],cs->nc[i],double);
	      for(j=0;j<cs->nc[i];j++)   // For all corners
		{
		  /* Set the corner values for the non-offset variables */
		  for(k=0;k<cs->nve[i];k++) 
		    {
		      v=cs->ve[i][k];
		      if (j & (1<<k)) // First variable is in the lowest bit (right hand one)
			c[v]=up[v];
		      else
			c[v]=low[v];
		    }
		  cs->f0[i][j]=EvaluateEquation(c,eq);
		}
	    }

	  cs->nSimp=0; // No simplification!
	  
	  free(low);
	  free(up);
	  free(c);
	}
      
      DeleteVariables(&rcs_variables);
      DeleteEquations(&rcs_equations);
      DeleteConstants(&rcs_constants);
    }

  return(ns);
} 

