#include "transform.h"

#include "basic_algebra.h"
#include "error.h"

#include <math.h>

/** \file transform.c
    \brief Implementation of the Ttransform functions.

    Implementation of the Ttransform functions.

    \sa Ttransform, transform.h.
*/

void NewIDTransform(Ttransform *tr)
{
  memset(tr->q,0,8*sizeof(double));

  tr->q[0]=1;
  tr->type=TR_CT;
  tr->var=NO_UINT;
  tr->inv=FALSE;
}

void CopyTransform(Ttransform *tr_dst,Ttransform *tr_src)
{
  memcpy(tr_dst->q,tr_src->q,8*sizeof(double));
  tr_dst->type=tr_src->type;
  tr_dst->var=tr_src->var;
  tr_dst->inv=tr_src->inv;
}

inline boolean VariableTransform(Ttransform *tr)
{
  return(tr->type!=TR_CT);
}

inline boolean CtTransform(Ttransform *tr)
{
  return(tr->type==TR_CT);
}

boolean IsIdentity(Ttransform *tr)
{
  return((tr->type==TR_CT)&&(Norm(7,&(tr->q[1]))<1e-8));
}

inline unsigned int GetTransformVariable(Ttransform *tr)
{
  return(tr->var);
}

void InvertTransform(Ttransform *tr)
{
  /* Constant transforms are directly inverted.
     Variable transforms are "implicitly" inverted. */
  if (tr->type==TR_CT)
    {
      /* A general transform has been define as
	   q = qt*qr

         with qt = [1 0 0 0 b] = 1+E*b
	 and  qr = [a 0 0 0 0] = a

	 The product results in

	    (1+E*b)*a = a+E*b*a
	   
	 The inverse is defined as

	    q^-1=qr^-1*qt^-1

	 To define the inverse we have first to recover a and b from q
	 and, then, define qr^-1 ant qt^-1
	 a is directly the non-dual part of q (a^-1 this dual part inverted)
	 b is b*a*a^-1  where b*a is the dual part of q
       */

      double qri[8],qti[8]; // qr^-1 and qt^-1
      double ba[4];
      
      /* qr^-1 . The first four elements are a^-1*/
      memset(qri,0,8*sizeof(double)); // dual(qri)=0
      qri[0]= tr->q[0];  //real(qri) = real(q)^-1
      qri[1]=-tr->q[1];
      qri[2]=-tr->q[2];
      qri[3]=-tr->q[3];
      
      /* b*a  -> dual part of q */
      ba[0]= tr->q[4];
      ba[1]= tr->q[5];
      ba[2]= tr->q[6];
      ba[3]= tr->q[7];

      /* qti = 1-E*b*a*a^-1 */
      memset(qti,0,8*sizeof(double)); // real(qti) = [1 0 0 0]
      qti[0]=1.0; 
      QuaternionProduct(ba,qri,&(qti[4])); // dual(qti) = b*a*a^-1
      qti[4]=-qti[4];
      qti[5]=-qti[5];
      qti[6]=-qti[6];
      qti[7]=-qti[7];
      
      DualQuaternionProduct(qri,qti,tr->q);

      // Constant transforms are never implicitly inverted
      tr->inv=FALSE;
    }
  else
    tr->inv=!(tr->inv);
}

void NewCtTransform(unsigned int tt, double value,boolean inv,Ttransform *tr)
{  
  memset(tr->q,0,8*sizeof(double));

  tr->q[0]=1;
  switch(tt)
    {
    case TR_RX:
      if (fabs(fabs(value)-M_PI)<1e-3)
	Warning("Angle close to PI in RX transform");
      tr->q[1]=tan((inv?-1:+1)*value/2.0);
      break;
    case TR_RY:
      if (fabs(fabs(value)-M_PI)<1e-3)
	Warning("Angle close to PI in RX transform");
      tr->q[2]=tan((inv?-1:+1)*value/2.0);
      break;
    case TR_RZ:
      if (fabs(fabs(value)-M_PI)<1e-3)
	Warning("Angle close to PI in RX transform");
      tr->q[3]=tan((inv?-1:+1)*value/2.0);
      break;
    case TR_TX:
      tr->q[5]=(inv?-1:+1)*value/2.0;
      break;
    case TR_TY:
      tr->q[6]=(inv?-1:+1)*value/2.0;
      break;
    case TR_TZ:
      tr->q[7]=(inv?-1:+1)*value/2.0;
      break;
    default:
      Error("Wrong transform type in NewCtTransform");
    }
  
  tr->type=TR_CT;
  tr->var=NO_UINT;
  tr->inv=FALSE; // already inverted above if necessary
}

void NewVarTransform(unsigned int tt,unsigned int id,boolean inv,Ttransform *tr)
{
  if ((tt!=TR_RX)&&(tt!=TR_RY)&&(tt!=TR_RZ)&&
      (tt!=TR_TX)&&(tt!=TR_TY)&&(tt!=TR_TZ))
    Error("Wrong transform type in NewVarTransform");
  
  tr->type=tt;
  tr->var=id;
  tr->inv=inv;
}

void NewDHRTransform(double d,unsigned int theta,double a,double alpha,boolean inv,Ttransform *tr)
{

  // We use the space for the quaternion to store intermediat values. The actual quaternion will
  // be computed on-the-fly
  tr->q[0]=d;               // d
  tr->q[1]=a;               // a
  tr->q[2]=tan(alpha/2);    // tan(alpha/2)

  tr->type=TR_DHR;
  tr->var=theta;
  tr->inv=inv;

}

void NewTransformFromMatrix(double m11,double m12,double m13,double m14,
			    double m21,double m22,double m23,double m24,
			    double m31,double m32,double m33,double m34,boolean inv,Ttransform *tr)
{
  double s;
  double R[16];
  double qr[8];
  double qt[8];

  memset(qr,0,8*sizeof(double));
  memset(qt,0,8*sizeof(double));

  s=(inv?-1.0:+1.0);
  
  // translation (possibly inverted)
  // T or T^-1
  qt[0]=1.0;
  qt[5]=s*m14/2;
  qt[6]=s*m24/2;
  qt[7]=s*m34/2;

  // Rotation matrix
  R[0]=m11; R[4]=m12; R[ 8]=m13; R[12]=0;
  R[1]=m21; R[5]=m22; R[ 9]=m23; R[13]=0;
  R[2]=m31; R[6]=m32; R[10]=m33; R[14]=0;
  R[3]=0;   R[7]=0;   R[11]=0;   R[15]=1;
    
  Rot2Quat(R,qr); /* R */
  qr[1]*=s; /* R^-1 */
  qr[2]*=s; 
  qr[3]*=s;

  if (inv)
    /* R^-1 * T^-1 */
    DualQuaternionProduct(qr,qt,tr->q);
  else
    /* T * R */
    DualQuaternionProduct(qt,qr,tr->q);
  
  tr->type=TR_CT;
  tr->var=NO_UINT;
  tr->inv=FALSE;
}

void NewTransformFromDualQuat(double q1,double q2,double q3,double q4,
			      double q5,double q6,double q7,double q8,boolean inv,Ttransform *tr)
{  
  tr->q[0]=q1;
  tr->q[1]=q2;
  tr->q[2]=q3;
  tr->q[3]=q4;
  
  tr->q[4]=q5;
  tr->q[5]=q6;
  tr->q[6]=q7;
  tr->q[7]=q8;
  
  tr->type=TR_CT;
  tr->var=NO_UINT;
  tr->inv=inv;

  if (inv)
    InvertTransform(tr);  
}

void Transform2DualQuad(double *val,signed int *offs,double *q,Ttransform *tr)
{
  double b; //offset. Only affects rotational variables
  double s;

  s=(tr->inv?-1.0:1.0);
  
  switch(tr->type)
    {
    case TR_CT:
      memcpy(q,tr->q,8*sizeof(double));
      if (tr->inv)
	Error("Ct transforms should not be implicitly inverted!");
      break;
    case TR_RX:
      memset(q,0,8*sizeof(double));
      if (offs==NULL)
	b=0;
      else
	b=(double)offs[tr->var];
      q[0]=1.0-b*val[tr->var];
      q[1]=s*(b+val[tr->var]); // assuming that vars are already in tan(angle/2) 
      break;
    case TR_RY:
      memset(q,0,8*sizeof(double));
      if (offs==NULL)
	b=0;
      else
	b=(double)offs[tr->var];
      q[0]=1.0-b*val[tr->var];
      q[2]=s*(b+val[tr->var]); // assuming that vars are already in tan(angle/2) 
      break;
    case TR_RZ:
      memset(q,0,8*sizeof(double));
      if (offs==NULL)
	b=0;
      else
	b=(double)offs[tr->var];
      q[0]=1.0-b*val[tr->var];
      q[3]=s*(b+val[tr->var]); // assuming that vars are already in tan(angle/2) 
      break;
    case TR_TX:
      memset(q,0,8*sizeof(double));
      q[0]=1.0;
      q[5]=s*val[tr->var]/2;
      break;
    case TR_TY:
      memset(q,0,8*sizeof(double));
      q[0]=1.0;
      q[6]=s*val[tr->var]/2;
      break;
    case TR_TZ:
      memset(q,0,8*sizeof(double));
      q[0]=1.0;
      q[7]=s*val[tr->var]/2; 
      break;
    case TR_DHR:
      {
	double t,a,d,ta;

	/*
	  Symbolic expressions taken from Matlab:

	      syms d epsilon t b a ta  % ta is tan(alpha/2)
	      epsilon = sym('epsilon');
	      Q = sym(zeros(5,4)); 
	      
	      s=1;
	      Q( 1,:) = [1 0 0 s*d/2*epsilon]; % Tz
	      Q( 2,:) = [1 0 0 s*t];           % Rz
	      Q( 3,:) = [1 0 0 s*b];           % offset
	      Q( 4,:) = [1 s*a/2*epsilon 0 0]; % Tx
	      Q( 5,:) = [1 s*ta 0  0];         % Rx
	      qt = QuatProdL(Q(1:5,:));

	  For the inverse, we have to evaluate

	      s=-1;
	      Q( 1,:) = [1 s*ta 0  0];         % Rx^-1
	      Q( 2,:) = [1 s*a/2*epsilon 0 0]; % Tx^-1
	      Q( 3,:) = [1 0 0 s*b];           % offset^-1
	      Q( 4,:) = [1 0 0 s*t];           % Rz^-1
              Q( 5,:) = [1 0 0 s*d/2*epsilon]; % Tz^-1
	      qt = QuatProdL(Q(1:5,:));

          Generate the equations as

              Eq=cell(1,8);
	      for i=1:4
	         [C, ~] = coeffs(qt(i), epsilon);
	         Eq{i  } = C(length(C));
	         Eq{4+i} = C(length(C)-1);
	      end

	  The expressions are:    

	      q0 = 1 - b*t;
	      q1 = s*ta*q0;
	      q3 = s*(b + t);    
	      q2 = ta*q3; 
	
	      q4 = (-s*a*q1 - s*d*q3)/2;
	      q5 = ( s*a*q0 -   d*q2)/2;
	      q6 = (   a*q3 +   d*q1)/2;
	      q7 = (  -a*q2 + s*d*q0)/2;

	  Test the expressions as: 

	     simplify(Eq{1}-q0)
	     simplify(Eq{2}-q1)
	     simplify(Eq{3}-q2)
	     simplify(Eq{4}-q3)
	     simplify(Eq{5}-q4)
	     simplify(Eq{6}-q5)
	     simplify(Eq{7}-q6)
	     simplify(Eq{8}-q7)
	*/

	// Recover parameters
	if (offs==NULL)
	  b=0;
	else
	  b=(double)offs[tr->var]; // offset
	// a/d always appear divided by two. Do it once.
	d=tr->q[0]/2.0;  // d/2 
	a=tr->q[1]/2.0;  // a/2
	
	ta=tr->q[2];     // tan(alpha/2)
	t=val[tr->var];  // rotation
	
	q[0] = 1 - b*t;
	q[1] = s*ta*q[0];
	q[3] = s*(b + t);    // used in q[2]!
	q[2] = ta*q[3]; 
	
	q[4] = -s*a*q[1] - s*d*q[3];  // Not /2 since a,d are already /2
	q[5] =  s*a*q[0] -   d*q[2];
	q[6] =    a*q[3] +   d*q[1];
	q[7] =   -a*q[2] + s*d*q[0];
      }
      break;
    default:
      Error("Unknown transform type in Transform2DualQuad");
    }
}

void PrintTransform(FILE *f,char **varname,Ttransform *tr)
{
  switch(tr->type)
    {
    case TR_RX:
      fprintf(f,"Rx");
      break;
    case TR_RY:
      fprintf(f,"Ry");
      break;
    case TR_RZ:
      fprintf(f,"Rz");
      break;
    case TR_TX:
      fprintf(f,"Tx");
      break;
    case TR_TY:
      fprintf(f,"Ty");
      break;
    case TR_TZ:
      fprintf(f,"Tz");
      break;
    case TR_CT:
      {
	unsigned int i;

	if ((tr->q[2]==0)&&(tr->q[3]==0)&&
	    (tr->q[5]==0)&&(tr->q[6]==0)&&(tr->q[7]==0))
	  fprintf(f,"Rx(%f)",2.0*atan(tr->q[1]));
	else
	  {
	    if ((tr->q[1]==0)&&(tr->q[3]==0)&&
		(tr->q[5]==0)&&(tr->q[6]==0)&&(tr->q[7]==0))
	      fprintf(f,"Ry(%f)",2.0*atan(tr->q[2]));
	    else
	      {
		if ((tr->q[1]==0)&&(tr->q[2]==0)&&
		    (tr->q[5]==0)&&(tr->q[6]==0)&&(tr->q[7]==0))
		  fprintf(f,"Rz(%f)",2.0*atan(tr->q[3]));
		else
		  {
		    if ((tr->q[1]==0)&&(tr->q[2]==0)&&(tr->q[3]==0)&&
			(tr->q[6]==0)&&(tr->q[7]==0))
		      fprintf(f,"Tx(%f)",2.0*tr->q[5]);
		    else
		      {
			if ((tr->q[1]==0)&&(tr->q[2]==0)&&(tr->q[3]==0)&&
			    (tr->q[5]==0)&&(tr->q[7]==0))
			  fprintf(f,"Ty(%f)",2.0*tr->q[6]);
			else
			  {
			    if ((tr->q[1]==0)&&(tr->q[2]==0)&&(tr->q[3]==0)&&
				(tr->q[5]==0)&&(tr->q[6]==0))
			      fprintf(f,"Tz(%f)",2.0*tr->q[7]);
			    else
			      {
				// General transform
				fprintf(f,"TRQ(%f",tr->q[0]);
				for(i=1;i<8;i++)
				  fprintf(f,",%f",tr->q[i]);
				fprintf(f,")");
			      }
			  }
		      }
		  }
	      }
	  }	
      }
      break;
    case TR_DHR:
      // d
      fprintf(f,"DHR(%f,",tr->q[0]); 

      // theta
      if (varname!=NULL)
	fprintf(f,"%s,",varname[tr->var]);
      else
	fprintf(f,"t_%u,",tr->var);
      
      // a, alpha
      fprintf(f,"%f,%f)",tr->q[1],atan(2*tr->q[2])); 
      break;
    }
  if ((tr->type!=TR_CT)&&(tr->type!=TR_DHR))
    {
      if (varname!=NULL)
	fprintf(f,"(%s)",varname[tr->var]);
      else
	fprintf(f,"(var_%u)",tr->var);
    }
  if (tr->inv)
    fprintf(f,"^-1");
}

void DeleteTransform(Ttransform *tr)
{}

