#include "transforms.h"

#include "error.h"

#include <stdlib.h>
#include <math.h>

/** \file transforms.c
    \brief Implementation of the Ttransforms functions.

    Implementation of the Ttransforms functions.

    \sa Ttransforms, transforms.h.
*/

/** 
   \brief Auxiliary structure.

    Auxiliary structure to re-order the vector of monomials
*/
typedef struct {
  unsigned int ndx;    /**< Index of the monomial. */
  unsigned int nbits;  /**< Number of active bits in the monomial (number of variables in the monomial). */
} Tmono;

/** 
   \brief Compare monomials

   Used to re-order the monimials

   \param p1 Pointer to the fist monomial (Tmono structure)
   \param p2 Pointer to the second monomial (Tmono structure)
*/
int cmpMono(const void *p1,const void *p2);
  
int cmpMono(const void *p1,const void *p2)
{
  Tmono *m1,*m2;
  int r;

  m1=(Tmono*)p1;
  m2=(Tmono*)p2;

  if (m1->nbits<m2->nbits)
    r=1;
  else
    {
      if (m1->nbits==m2->nbits)
	{
	  unsigned int a,b,m,i;
	  
	  r=0;
	  a=m1->ndx;
	  b=m2->ndx;
	  i=0;
	  while ((i<16) && ((a&1)==(b&1)))
	    {
	      a=(a>>1);
	      b=(b>>1);
	      i++;
	    }
	  if (i<16)
	    {
	      if ((a&1)>(b&1))
		r=1;
	      else
		r=-1;
	    }
	}
      else
	r=-1;
    }
  return(r);
}

void InitTransforms(Ttransforms *ts)
{
  ts->m=INIT_NUM_TRANSFORMS;
  ts->n=0;

  NEW(ts->tr,ts->m,Ttransform*);

  InitVarSet(&(ts->vars));

  #if (REDUNDANT_EQ_IN_LOOP)
    // By default we have 7 equations
    ts->neq=7;

    // Three rotations (from the real part of the dual quaternion)
    ts->ndx[0]=1;
    ts->ndx[1]=2;
    ts->ndx[2]=3;
    
    // four translations (from the dual part of the dual quaternion)
    ts->ndx[3]=4;
    ts->ndx[4]=5;
    ts->ndx[5]=6;
    ts->ndx[6]=7;
  #else
    // By default we have 6 equations
    ts->neq=6;

    // Three rotations (from the real part of the dual quaternion)
    ts->ndx[0]=1;
    ts->ndx[1]=2;
    ts->ndx[2]=3;
    
    // Three translations (from the dual part of the dual quaternion)
    ts->ndx[3]=5;
    ts->ndx[4]=6;
    ts->ndx[5]=7;
  #endif
  
  ts->simp=0;
  /* rvars will be initialized when simplifying */
  ts->nr=0;
  ts->nc=0;
  ts->M=NULL;
  ts->x=NULL;
  ts->S=NULL;
  ts->ts1=NULL;
}

void CopyTransforms(Ttransforms *ts_dst,Ttransforms *ts_src)
{
  unsigned int i;

  ts_dst->m=ts_src->m;
  ts_dst->n=ts_src->n;
  
  NEW(ts_dst->tr,ts_dst->m,Ttransform*);
  
  for(i=0;i<ts_dst->n;i++)
    {
      NEW(ts_dst->tr[i],1,Ttransform);
      CopyTransform(ts_dst->tr[i],ts_src->tr[i]);
    }
  
  CopyVarSet(&(ts_dst->vars),&(ts_src->vars));
  
  ts_dst->neq=ts_src->neq;
  memcpy(ts_dst->ndx,ts_src->ndx,ts_src->neq*sizeof(double));

  ts_dst->simp=ts_src->simp;
      
  if (ts_src->simp>0)
    {
      unsigned int sz;
      Ttransforms *ts1;
      
      CopyVarSet(&(ts_dst->rvars),&(ts_src->rvars));
      
      ts_dst->nr=ts_src->nr;
      ts_dst->nc=ts_src->nc;

      sz=ts_dst->nr*ts_dst->nc;
      
      NEW(ts_dst->M,sz,double);
      memcpy(ts_dst->M,ts_src->M,sz*sizeof(double));
      
      NEW(ts_dst->x,ts_dst->nc,double);
      memcpy(ts_dst->x,ts_src->x,ts_dst->nc*sizeof(double));

      if (ts_src->S!=NULL)
	{
	  NEW(ts_dst->S,sz,double);
	  memcpy(ts_dst->S,ts_src->S,sz*sizeof(double));
	}
      else
	ts_dst->S=NULL;

      CopyQRInfo(&(ts_dst->qr),&(ts_src->qr));
      
      NEW(ts1,1,Ttransforms);
      ts_dst->ts1=(void*)ts1;
      CopyTransforms(ts1,(Ttransforms*)(ts_src->ts1));
    }
  else
    {      
      ts_dst->nr=0;
      ts_dst->nc=0;
      ts_dst->M=NULL;
      ts_dst->x=NULL;
      ts_dst->S=NULL;
      ts_dst->ts1=NULL;
    }
}

inline Tvariable_set *GetTransformsVariables(Ttransforms *ts)
{
  /* For simplified sequences, this is the simplified set! */
  return(&(ts->vars));
}

inline Tvariable_set *GetTransformsRemovedVariables(Ttransforms *ts)
{
  if (ts->simp>0)
    return(&(ts->rvars));
  else
    return(NULL);
}

inline boolean SimplifiedTransforms(Ttransforms *ts)
{
  return(ts->simp>0);
}

inline unsigned int NTransformsEquations(Ttransforms *ts)
{
  /* For simplified sequences, this is the simplified set! */
  return(ts->neq);
}

inline unsigned int NTransforms(Ttransforms *ts)
{
  return(ts->n);
}

inline unsigned int NTransformsVariables(Ttransforms *ts)
{
  /* For simplified sequences, this is the simplified set! */
  return(VariableSetSize(&(ts->vars)));
}

void RotateTransforms(unsigned int nv,boolean var,Ttransforms *ts)
{
  unsigned int i,k,tv;
  boolean found;

  if (ts->simp>0)
    Error("Can not rotate simplified transform sequences");

  if (ts->n==0)
    Error("Rotating an empty transform sequence");
  
  if (var)
    {
      tv=VariableSetSize(&(ts->vars));
      if (nv>=tv)
	Error("Wrong rotation in RotateTransforms (I)");
  
      i=0; // Current transform
      k=0; // How many variables we have seen so far 
      found=FALSE;
      while((!found)&&(i<ts->n))
	{
	  if (VariableTransform(ts->tr[i]))
	    {
	      found=(k==nv);
	      k++;  // One variable more
	    }
	  if (!found)
	    i++;  // One transform more
	}
    }
  else
    {
      if (nv>=ts->n-1)
	Error("Wrong rotation in RotateTransforms (II)");
      
      i=nv;
      found=TRUE;
    }

  if (found)
    {
      unsigned int j;
      Ttransform** sq;
       
      // 'i' points to the position of sought after variable
      NEW(sq,i+1,Ttransform*);
       
      for(k=0;k<=i;k++)
	sq[k]=ts->tr[k];
       
      for(k=i+1,j=0;k<ts->n;k++,j++)
	ts->tr[j]=ts->tr[k];
       
      for(k=0;k<=i;k++,j++)
	ts->tr[j]=sq[k];

      free(sq);
    }
  else
    Error("Wrong rotation in RotateTransforms (III)");
}

unsigned int CutTransforms(unsigned int nv,Ttransforms *ts1,Ttransforms *ts2,Ttransforms *ts)
{
  unsigned int cp,k,tv;
  boolean found;

  if (ts->simp>0)
    Error("Can not cut simplified transform sequences");
  
  tv=VariableSetSize(&(ts->vars));
  if ((nv==0)||(nv>=tv))
    Error("Wrong variable in CutTransforms");

   cp=0; // cut point
   k=0;  // Number of variables found so far
   found=FALSE;
   while((!found)&&(cp<ts->n))
     {
       if (VariableTransform(ts->tr[cp]))
	 {
	   k++;
	   found=(k>nv);
	 }
       if (!found)
	 cp++;
     }

   if (found)
     {
       Ttransform *t;
       unsigned int i,n; 
       
       ts1->n=0;
       ts2->n=0;
       
       ts1->m=ts->m;
       ts2->m=ts->m;
       
       NEW(ts1->tr,ts->m,Ttransform*);
       NEW(ts2->tr,ts->m,Ttransform*);

       InitVarSet(&(ts1->vars));
       InitVarSet(&(ts2->vars));

       // From 0 to k are kept in the LHS
       for(i=0;i<cp;i++)
	 {
	   t=ts->tr[i];
	   
	   NEW(ts1->tr[i],1,Ttransform);
	   CopyTransform(ts1->tr[i],t);
	   if (VariableTransform(t))
	     AddVariable2Set(GetTransformVariable(t),&(ts1->vars));
	   ts1->n++;
	 }

       // From k+1 to n-1 are moved to the RHS 
       for(i=ts->n-1,n=0;i>=cp;i--,n++)
	 {	   
	   t=ts->tr[i];
	   
	   NEW(ts2->tr[n],1,Ttransform);
	   CopyTransform(ts2->tr[n],t);
	   InvertTransform(ts2->tr[n]);
	   if (VariableTransform(t))
	     AddVariable2Set(GetTransformVariable(t),&(ts2->vars));
	   ts2->n++;
	 }
       
       ts1->neq = ts2->neq = ts->neq;
       memcpy(ts1->ndx,ts->ndx,ts->neq*sizeof(unsigned int));
       memcpy(ts2->ndx,ts->ndx,ts->neq*sizeof(unsigned int));
       
       ts1->simp= ts2->simp = 0;
       ts1->nr  = ts2->nr   = 0;
       ts1->nc  = ts2->nc   = 0;
       ts1->M   = ts2->M    = NULL;
       ts1->x   = ts2->x    = NULL;
       ts1->S   = ts2->S    = NULL;
       ts1->ts1 = ts2->ts1  = NULL;
     }
   else
     Error("Wrong rotation in CutTransforms");

   if (found)
     return(cp+1);
   else
     return(NO_UINT);
}

void PreMultiplyTransforms(Ttransform *tr,Ttransforms *ts1,Ttransforms *ts)
{
  unsigned int i;
  
  /* ts = tr * ts1  */

  if (ts1->simp>0)
    Error("Can not use PreMultiplyTransforms on simplified transform sequences");
  
  if (IsIdentity(tr))
    CopyTransforms(ts,ts1);
  else
    {
      // Directly operate ct transforms at the beginning of the sequence
      if ((COMPACT_TRANSFORMS)&&(ts1->n>0)&&(CtTransform(tr))&&(CtTransform(ts1->tr[0])))
	{
	  double q1[8],q2[8],qp[8];
	  
	  CopyTransforms(ts,ts1);

	  // Ct transforms do not need variables nor offsets
	  Transform2DualQuad(NULL,NULL,q1,tr); 
	  Transform2DualQuad(NULL,NULL,q2,ts1->tr[0]);      
	  DualQuaternionProduct(q1,q2,qp);
	  DeleteTransform(ts->tr[0]);
	  NewTransformFromDualQuat(qp[0],qp[1],qp[2],qp[3],
				   qp[4],qp[5],qp[6],qp[7],FALSE,ts->tr[0]);

	  // We can remove the leading transform
	  if (IsIdentity(ts->tr[0]))
	    RemoveFirstTransform(ts);
	}
      else
	{
	  ts->m=ts1->m+1;
	  ts->n=ts1->n+1;

	  NEW(ts->tr,ts->m,Ttransform*);
	  for(i=0;i<ts1->n;i++)
	    {
	      NEW(ts->tr[i+1],1,Ttransform);
	      CopyTransform(ts->tr[i+1],ts1->tr[i]);
	    }
	  NEW(ts->tr[0],1,Ttransform);
	  CopyTransform(ts->tr[0],tr);

	  CopyVarSet(&(ts->vars),&(ts1->vars));
	  if (VariableTransform(tr))
	    AddVariable2Set(GetTransformVariable(tr),&(ts->vars));

          #if (REDUNDANT_EQ_IN_LOOP)
  	    ts->neq = 7;

	    ts->ndx[0] = 1;
	    ts->ndx[1] = 2;
	    ts->ndx[2] = 3;
	    ts->ndx[3] = 4;
	    ts->ndx[4] = 5;
	    ts->ndx[5] = 6;
	    ts->ndx[6] = 7;
	  #else
	    ts->neq = 6;
	    
	    ts->ndx[0] = 1;
	    ts->ndx[1] = 2;
	    ts->ndx[2] = 3;
	    ts->ndx[3] = 5;
	    ts->ndx[4] = 6;
	    ts->ndx[5] = 7;
	  #endif
	  ts->simp=0;
	  ts->nr=0;
	  ts->nc=0;
	  ts->M=NULL;
	  ts->x=NULL;
	  ts->S=NULL;
	  ts->ts1=NULL;
	}
    }
}

void RemoveFirstTransform(Ttransforms *ts)
{
  if (ts->n>0)
    {
      unsigned int i;
  
      DeleteTransform(ts->tr[0]);
      free(ts->tr[0]);
      for(i=1;i<ts->n;i++)
	ts->tr[i-1]=ts->tr[i];
      ts->n--;
    }
}

void PostMultiplyTransforms(Ttransforms *ts1,boolean inv,boolean compact,Ttransforms *ts)
{
  if (ts1->n>0)
    {
      /* ts = ts * ts1^inv */
      unsigned int i;
  
      if (ts->simp>0)
	Error("Can not use PostMultiplyTransforms on simplified transform sequences");
  
      if (ts->n+ts1->n>=ts->m)
	{
	  ts->m=2*(ts->n+ts1->n);
	  MEM_EXPAND(ts->tr,ts->m,Ttransform*);
	}

      // Add the variables in 'ts1' to 'ts'
      for(i=0;i<ts1->n;i++)
	{
	  if (VariableTransform(ts1->tr[i]))
	    AddVariable2Set(GetTransformVariable(ts1->tr[i]),&(ts->vars));
	}
  
      if (inv)
	{      
	  for(i=0;i<ts1->n;i++)
	    {
	      NEW(ts->tr[ts->n+i],1,Ttransform);
	      CopyTransform(ts->tr[ts->n+i],ts1->tr[ts1->n-1-i]);
	      InvertTransform(ts->tr[ts->n+i]);
	    }
	}
      else
	{
	  for(i=0;i<ts1->n;i++)
	    {
	      NEW(ts->tr[ts->n+i],1,Ttransform);
	      CopyTransform(ts->tr[ts->n+i],ts1->tr[i]);
	    }
	}
  
      if ((COMPACT_TRANSFORMS)&&
	  (CtTransform(ts->tr[ts->n-1]))&&
	  (CtTransform(ts->tr[ts->n])))
	{
	  // We can compact the constant transforms in the concatenation point
	  double q1[8],q2[8],qp[8];
	  unsigned int n;
      	   
	  Transform2DualQuad(NULL,NULL,q1,ts->tr[ts->n-1]);
	  Transform2DualQuad(NULL,NULL,q2,ts->tr[ts->n]);
	  DualQuaternionProduct(q1,q2,qp);
	  DeleteTransform(ts->tr[ts->n-1]);
	  NewTransformFromDualQuat(qp[0],qp[1],qp[2],qp[3],
				   qp[4],qp[5],qp[6],qp[7],FALSE,ts->tr[ts->n-1]);
	  DeleteTransform(ts->tr[ts->n]);
	  free(ts->tr[ts->n]);

	  n=ts->n+1;
	  ts->n+=ts1->n;
	  for(i=n;i<ts->n;i++)
	    ts->tr[i-1]=ts->tr[i];
	  ts->n--;
      	}
      else
	ts->n+=ts1->n;
    }
  
  // Check wheter we can campact first and last constant transforms
  if ((compact)&&
      (CtTransform(ts->tr[0]))&&
      (CtTransform(ts->tr[ts->n-1])))
    {
      double q1[8],q2[8],qp[8];
	  
      Transform2DualQuad(NULL,NULL,q1,ts->tr[ts->n-1]);
      Transform2DualQuad(NULL,NULL,q2,ts->tr[0]);
      DualQuaternionProduct(q1,q2,qp);
      DeleteTransform(ts->tr[0]);
      NewTransformFromDualQuat(qp[0],qp[1],qp[2],qp[3],
			       qp[4],qp[5],qp[6],qp[7],FALSE,ts->tr[0]);
      DeleteTransform(ts->tr[ts->n-1]);
      free(ts->tr[ts->n-1]);
      ts->n--;
    }
}

boolean SimplifyTransforms(unsigned int simplify,signed int *offs,Ttransforms *ts)
{
  unsigned int nv;
  
  nv=VariableSetSize(&(ts->vars));
  
  if (nv>2)
    {
      unsigned int i,dof;
      Ttransforms *ts1,ts2;
      unsigned int nv1,nv2;  // Number of variables 
      unsigned int nr1,nr2,nc1,nc2; // Number of rows and columns
      double *M1,*M2;
      signed int info;

      // Degrees of freedom of the loop.
      // The loop in the simplified system keed the same balance
      // between equations and vars
      dof=nv-6; // Each loop constraint 6dof
      
      // The simplification procedure uses 8 equations!
      ts->neq=8;
      for(i=0;i<ts->neq;i++)
	ts->ndx[i]=i;
      
      nv2=simplify;  // Variables move to the right-hand side (the ones to remove)
      nv1=nv-nv2;    // Variables to keep in the left-hand side (the ones to keep)

      #if (_DEBUG>0)
	fprintf(stderr,"------------------------------------------------------------------------------------\n");
	fprintf(stderr,"------------------------------------------------------------------------------------\n");
	fprintf(stderr,"Simplify: %u\n",simplify);
	fprintf(stderr,"nv: %u\n",nv);
	fprintf(stderr,"dof: %u\n",dof);
        PrintTransforms(stderr,NULL,ts);
	fprintf(stderr,"=1\n\n");
      #endif
	
      // Split the transforms in two block (left- and right-hand sides)
      NEW(ts1,1,Ttransforms);
      CutTransforms(nv1,ts1,&ts2,ts);

      #if (_DEBUG>0)
        PrintTransforms(stderr,NULL,ts1);
	fprintf(stderr," = ");
	PrintTransforms(stderr,NULL,&ts2);
	fprintf(stderr,"\n\n");
      #endif
	
      // Rows in M1/M2
      nr1=NTransformsEquations(ts1);  // This is 8!
      nr2=NTransformsEquations(&ts2); // This is 8 also

      // Columns in A/B
      nc1=(1<<nv1);
      nc2=(1<<nv2); // this is 4

      #if (_DEBUG>0)
        fprintf(stderr,"nv1=%u nr1=%u nc1=%u\n",nv1,nr1,nc1);
	fprintf(stderr,"nv1=%u nr2=%u nc2=%u\n",nv2,nr2,nc2);
      #endif
	
      // Coefficient matrices for each sub-sequence
      CoefficientMatrix(offs,&M1,FALSE,ts1);
      #if (_DEBUG>0)
        PrintMatrix(stderr,"M1",nr1,nc1,M1);
      #endif

      // For the RHS we do not take into account offsets
      CoefficientMatrix(NULL,&M2,FALSE,&ts2);
      #if (_DEBUG>0)
        PrintMatrix(stderr,"M2",nr2,nc2,M2);
      #endif
	
      ts->M=ts->S=NULL;

      // expected size of M
      #if (REDUNDANT_SIMP)
        ts->nr=4;
      #else
        ts->nr=nv1-dof; // We can change this if we want to use redundant equations
      #endif
      ts->nc=nc1;
	
      info=InitQR(nr2,nc2,&(ts->qr)); // We will decompose M2
      if (!info)
	{
	  info=QR(1e-8,M2,&(ts->qr));
	   
	  if (!info)
	    {
	      NEW(ts->M,ts->nr*ts->nc,double);
	      
	      if (simplify==2)
		{
		  NEW(ts->S,4*nc1,double);
		  info=ApplyQR(nc1,M1,ts->nr,ts->M,4,ts->S,&(ts->qr));
		}
	      else
		info=ApplyQR(nc1,M1,ts->nr,ts->M,0, NULL,&(ts->qr));
	      #if (_DEBUG>0)
	        if (info)
		  fprintf(stderr,"Error applying QR decomposition [%d]\n",info);
	      #endif
	    }
	  #if (_DEBUG>0)
	  else
	    fprintf(stderr,"Error initializing QR decomposition [%d]\n",info);
	  #endif
	}
      
      if (!info)
	{
	  ts->simp=simplify; /* 2 or 3*/

	  ts->ts1=(void*)ts1;
	      
	  #if (_DEBUG>0)
	    PrintMatrix(stderr,"M",ts->nr,ts->nc,ts->M);
	    if (simplify==2)
	      PrintMatrix(stderr,"S",4,ts->nc,ts->S);
	  #endif
	    
	  NEW(ts->x,ts->nc,double);
	  
	  ts->neq=ts->nr; // As many equations as rows in M
	  for(i=0;i<ts->neq;i++)
	    ts->ndx[i]=i;

	  // Removed variables. To use when solving
	  CopyVarSet(&(ts->rvars),&(ts2.vars));

	  #if (_DEBUG>0)
	    fprintf(stderr,"Removed vars: ");
	    PrintVarSet(stderr,NULL,&(ts->rvars));
	    fprintf(stderr,"\n");
	  #endif
	    
	  // From now one, the transform sequence is simplified and only involves
	  // a reduced set of variables!!
	  DeleteVarSet(&(ts->vars));
	  CopyVarSet(&(ts->vars),&(ts1->vars));
	      
	  #if (_DEBUG>0)
	    fprintf(stderr,"Kept vars: ");
	    PrintVarSet(stderr,NULL,&(ts->vars));
	    fprintf(stderr,"\n");
	  #endif
	}
      else
	{
          #if (_DEBUG>0)
	    fprintf(stderr,"Simplification failed\n");
	  #endif

	  // Reset any simplification-related info
	  ts->simp=0;
	  ts->nr=0;
	  ts->nc=0;
	  
	  if (ts->M!=NULL)
	    free(ts->M);
	  ts->M=NULL;
	  
	  if (ts->S!=NULL)
	    free(ts->S);
	  ts->S=NULL;
	  
	  DeleteTransforms(ts1);
	  free(ts1);
	  ts->ts1=NULL;

	  DeleteQR(&(ts->qr));
	  #if (REDUNDANT_EQ_IN_LOOP)
	    ts->neq=7;
	    ts->ndx[0]=1; 
	    ts->ndx[1]=2; 
	    ts->ndx[2]=3;  
	    ts->ndx[3]=4; 
	    ts->ndx[4]=5; 
	    ts->ndx[5]=6; 
	    ts->ndx[6]=7;
	  #else
	    ts->neq=6;
	    ts->ndx[0]=1; 
	    ts->ndx[1]=2; 
	    ts->ndx[2]=3; 
	    ts->ndx[3]=5; 
	    ts->ndx[4]=6; 
	    ts->ndx[5]=7;
	  #endif
	}
      
      free(M1);
      free(M2);
      DeleteTransforms(&ts2);
    }
  
  return(ts->simp>0);
}

void ReSimplifyTransforms(signed int *offs,Ttransforms *ts)
{
  if (ts->simp>0)
    {
      int info;
      unsigned int nv1,nc1;
      Ttransforms *ts1;
      double *M1;

      // The LHS of the simplification
      ts1=(Ttransforms *)ts->ts1;

      nv1=NTransformsVariables(ts1);
      nc1=(1<<nv1);
           
      // Coefficient matrices for each sub-sequence
      CoefficientMatrix(offs,&M1,FALSE,ts1);

      if (ts->simp==2)
	info=ApplyQR(nc1,M1,ts->nr,ts->M,4,ts->S,&(ts->qr));
      else
	info=ApplyQR(nc1,M1,ts->nr,ts->M,0, NULL,&(ts->qr)); /* assuming ts->simp=3 */

      if (info)
	Error("Error in re-simplification");
      
      free(M1);
    }
}

inline void GetTransformSimplification(double **S,Ttransforms *ts)
{
  if (ts->simp>0)
    {
      NEW(*S,ts->nr*ts->nc,double);
      memcpy(*S,ts->S,ts->nr*ts->nc*sizeof(double));
    }
  else
    *S=NULL;
}

unsigned int EvaluateTransforms(double *val,signed int *offs,double *q,Ttransforms *ts)
{
  if (ts->simp>0)
    {
      // Evaluate from matrices (offsets already considered when defining M)
      EvaluateTransformsFromCoefficients(val,ts->M,q,ts);
    }
  else
    {
      double qf[8],q1[8],qp[8];
      unsigned int i;

      // Identity dual quaternion
      memset(qf,0,8*sizeof(double));
      qf[0]=1;

      for(i=0;i<ts->n;i++)
	{
	  Transform2DualQuad(val,offs,q1,ts->tr[i]);
	  DualQuaternionProduct(qf,q1,qp);
	  memcpy(qf,qp,8*sizeof(double));
	}

      for(i=0;i<ts->neq;i++)
	q[i]=qf[ts->ndx[i]];
    }
  
  return(ts->neq);
}

unsigned int EvaluateTransformsFromCoefficients(double *val,double *m,double *q,Ttransforms *ts)
{
  unsigned int i,j,n,nc,v;
  double *xx,p;

  n=VariableSetSize(&(ts->vars)); 
  nc=(1<<n);
  if (ts->simp>0)
    xx=ts->x;
  else
    {
      NEW(xx,nc,double); 
    }

  
  for(i=0;i<nc;i++)
    {
      p=1.0;
      for(j=0;j<n;j++)
	{
	  if (i & (1<<j))
	    {
	      v=GetVariableN(j,&(ts->vars));
	      p*=val[v];
	    }
	}
      xx[i]=p;
    }
  
  MatrixVectorProduct(ts->neq,nc,m,xx,q);
  
  if (ts->simp==0)
    free(xx);
 
  return(ts->neq);
}

void CoefficientMatrix(signed int *offs,double **c,boolean normalize,Ttransforms *ts)
{
  double *val;
  unsigned int nv,tv,m,i,j,s,nb,nb1;
  double q[8],*p,*d;

  if (ts->simp>0)
    {
      NEW(*c,ts->nr*ts->nc,double);
      memcpy(*c,ts->M,ts->nr*ts->nc);
    }
  else
    {
      nv=MaxVarInSet(&(ts->vars))+1; // if largest var is nv, we can have vars from 0 to nv -> nv+1
      NEWZ(val,nv,double);

      tv=VariableSetSize(&(ts->vars)); //number of vars in the transform sequence
      m=(1<<tv); // Size of the vector of monomials
      NEW(d,ts->neq*m,double); //neq rows x m columns
      for(i=0;i<m;i++)
	{
	  for(j=0;j<tv;j++)
	    val[GetVariableN(j,&(ts->vars))]=(i&(1<<j)?1:0);
      
	  EvaluateTransforms(val,offs,q,ts);
	  
	  SetColumn(q,i,ts->neq,m,d);
	}
      
      // What we actually get is not the matrix of coefficients but a linear
      // combination. We can find the matrix of coefficients by multiplying
      // 'm' with the appropiate matrix
      // c=d*p
      NEWZ(p,m*m,double);
      for(i=0;i<m;i++)
	{
	  nb=pop(i);
     
	  p[RC2INDEX(0,i,m,m)]=(nb%2 ? -1.0 : 1.0);
      
	  for(j=1;j<i;j++)
	    {
	      s=j&i;
	      if (s==j)
		{
		  nb1=pop(s);
		  if (nb1>0)
		    p[RC2INDEX(j,i,m,m)]=((nb-nb1)%2 ? -1.0 : 1.0);
		}
	    }
	  p[RC2INDEX(i,i,m,m)]=1.0;
	}
  
      //c=d*p
      NEW(*c,ts->neq*m,double);
  
      MatrixMatrixProduct(ts->neq,m,d,m,p,*c);

      // Optionally, we can normalize the equations
      if (normalize)
	{
	  unsigned int lc;
      
	  for(i=0;i<ts->neq;i++)
	    {
	      lc=AbsMaxRowElement(i,ts->neq,m,*c);
	      ScaleRow(i,1.0/(*c)[RC2INDEX(i,lc,ts->neq,m)],ts->neq,m,*c);
	    }
	}
      
      free(d);
      free(p);
      free(val);
    }
}


void PrintTransforms(FILE *f,char **varname,Ttransforms *ts)
{
  if (ts->n>0)
    {
      unsigned int i;

      if (ts->simp>0)
	fprintf(f,"%% Non-simplified sequende\n");
      PrintTransform(f,varname,ts->tr[0]);
      for(i=1;i<ts->n;i++)
	{
	  fprintf(f,"*");
	  PrintTransform(f,varname,ts->tr[i]);
	}
    }
}

void PrintTransformsEquations(FILE *f,char **varname,Ttransforms *ts)
{
  if (ts->n>0)
    {
      double *c,v;
      unsigned int i,j,k,tv,nv,m;

      if (ts->simp>0)
	{
	  fprintf(f,"%% Simplified equations\n");
	  c=ts->M;
	}
      else
	CoefficientMatrix(NULL,&c,TRUE,ts);
      
      tv=VariableSetSize(&(ts->vars)); //number of vars in the (simplified) transform sequence
      m=(1<<tv); // Size of the vector of monomials

      // The equations defined by the sequence
      for(i=0;i<ts->neq;i++)
	{
	  for(j=0;j<m;j++)
	    {
	      v=c[RC2INDEX(i,j,ts->neq,m)];
	      if (fabs(v)>1e-6)
		{
		  fprintf(f,"%+.8f",v);
		  for(k=0;k<tv;k++)
		    {
		      if (j & (1<<k))
			{
			  nv=GetVariableN(k,&(ts->vars));
			  if (varname==NULL)
			    fprintf(f,"*t_%u",nv);
			  else
			    fprintf(f,"*%s",varname[nv]);  
			}
		    }
		}
	    }
	  fprintf(f," = 0;\n");
	}

      if (ts->simp==0)
	free(c);
    }
}

void PrintTransformsMatrixVector(FILE *f,char **varname,Ttransforms *ts)
{
   if (ts->n>0)
    {
      double *c;
      unsigned int j,k,tv,nv,m,v;
      boolean first;
      Tmono *mono;

      if (ts->simp>0)
	{
	  fprintf(f,"%% Simplified equations\n");
	  c=ts->M;
	}
      else
	CoefficientMatrix(NULL,&c,TRUE,ts);

      tv=VariableSetSize(&(ts->vars)); //number of vars in the (simplified) transform sequence
      m=(1<<tv); // Size of the vector of monomials
      
      // Re-order in (anti) lexicografical order
      NEW(mono,m,Tmono);
      for(j=0;j<m;j++)
	{
	  mono[j].ndx=j;        //Momonimal index
	  mono[j].nbits=pop(j); //Number of variables in the monomial
	}
      // Sort the monomials to get the most complex ones first
      qsort(mono,m,sizeof(Tmono),cmpMono);
      
      fprintf(stderr,"[");
      for(j=0;j<ts->neq;j++)
	{
	  for(k=0;k<m;k++)
	    {
	      v=mono[k].ndx; // Get the column to print
	      fprintf(stderr,"%.16f ",c[RC2INDEX(j,v,ts->neq,m)]);
	    }
	  if (j<ts->neq-1) fprintf(f,"\n");
	  else fprintf(f,"]\n");
	}
      fprintf(stderr,"*\n[");
      for(j=0;j<m;j++)
	{
	  v=mono[j].ndx;
	  first=TRUE;
	  for(k=0;k<tv;k++)
	    {
	      if (v & (1<<k))
		{
		  if (first)
		    first=FALSE;
		  else
		    fprintf(f,"*");
		  nv=GetVariableN(k,&(ts->vars));
		  if (varname==NULL)
		    fprintf(f,"t_%u",nv);
		  else
		    fprintf(f,"%s",varname[nv]);
		}
	    }
	  if (v==0) fprintf(f,"1");
	  if (j<m-1) fprintf(f,";\n");
	}
      fprintf(f,"];\n");

      free(mono);
      if (ts->simp==0)
	free(c);
    }
}

void DeleteTransforms(Ttransforms *ts)
{
  unsigned int i;
  
  for(i=0;i<ts->n;i++)
    {
      DeleteTransform(ts->tr[i]);
      free(ts->tr[i]);
    }
  free(ts->tr);
  DeleteVarSet(&(ts->vars));
  
  if (ts->simp>0)
    {
      DeleteVarSet(&(ts->rvars));
      free(ts->M);
      if (ts->S!=NULL)
	free(ts->S);
      free(ts->x);
      
      DeleteQR(&(ts->qr));
      DeleteTransforms((Ttransforms *)ts->ts1);
      free(ts->ts1);
    }
}

