#include "variable_set.h"

#include "defines.h"

#include <string.h>
#include <math.h>


/** 
   \file variable_set.c

   \brief Implementation of the functions operating on box Tvariable_set.
 
   Implementation of the functions operating on box Tvariable_set.

   \sa Tvariable_set, variable_set.h.
*/

/*private function of adding a var to vs*/

/** 
  \brief Adds an element to a variable set.

  Adds a variable index with a given power to a variable set. 
  Private function used by different public functions.

  \param varid The index of the variable to be added to the set.
  \param vs The set of variables to update.
*/
void AddVariableInt(unsigned int varid,Tvariable_set *vs);

void AddVariableInt(unsigned int varid,Tvariable_set *vs)
{  
  signed int j;
  unsigned int s;

  if (vs->nvars==vs->maxvars)
    {
      MEM_DUP(vs->var_id,vs->maxvars,unsigned int);
    }
  
  vs->var_id[vs->nvars]=varid;
  vs->nvars++;

  /* Keep variables in order from lower to upper ID. */
  j=vs->nvars-1;
  while((j>0)&&(vs->var_id[j-1]>vs->var_id[j]))
    {
      SWAP(vs->var_id[j],vs->var_id[j-1],s);
      j--;
    }
}

void InitVarSet(Tvariable_set *vs)
{
  vs->nvars=0;
  vs->maxvars=INIT_NUM_VARIABLES;
  NEW(vs->var_id,vs->maxvars,unsigned int);
}

void ResetVarSet(Tvariable_set *vs)
{
  vs->nvars=0;
}

void CopyVarSet(Tvariable_set *vs_dst,Tvariable_set *vs_orig)
{
  vs_dst->nvars=vs_orig->nvars;

  vs_dst->maxvars=vs_orig->maxvars;

  NEW(vs_dst->var_id,vs_dst->maxvars,unsigned int);

  memcpy(vs_dst->var_id,vs_orig->var_id,vs_dst->nvars*sizeof(unsigned int));
}

unsigned int CmpVarSet(Tvariable_set *vs1,Tvariable_set *vs2)
{
  unsigned int r;
  unsigned int i;

  if (vs1->nvars>vs2->nvars)
    r=2;
  else
    {
      if (vs2->nvars>vs1->nvars)
	r=1;
      else
	{
	  i=0;
	  while((i<vs1->nvars)&&
		(vs1->var_id[i]==vs2->var_id[i])) 
	    i++;
  
	  if (i==vs1->nvars)
	    r=0;
	  else
	    {
	      if (vs2->var_id[i]>vs1->var_id[i])
		r=2;
	      else
		r=1;
	    }
	}
    }
  return(r);
}

inline boolean EmptyVarSet(Tvariable_set *vs)
{
  return(vs->nvars==0);
}

unsigned int GetPlaceinSet(unsigned int id,Tvariable_set *vs)
{ 
  boolean found=FALSE;
  unsigned int i;

  i=0;
  while((i<vs->nvars)&&(id<=vs->var_id[i])&&(!found))
    {
      found=(id==vs->var_id[i]);
      if (!found) i++;
    }
    
  if (found)
    return(i);
  else
    return(NO_UINT); 
}

unsigned int MaxVarInSet(Tvariable_set *vs)
{
  if (vs->nvars==0)
    return(NO_UINT);
  else
    return(vs->var_id[vs->nvars-1]);
}

boolean VarIncluded(unsigned int id,Tvariable_set *vs)
{ 
  return((boolean)(GetPlaceinSet(id,vs)!=NO_UINT));
}

boolean Included(unsigned int id,Tvariable_set *vs)
{
  boolean found=FALSE;
  unsigned int i;

  i=0;
  while((i<vs->nvars)&&(id<=vs->var_id[i])&&(!found))
    {
      found=(id==vs->var_id[i]);
      if (!found) i++;
    }
  
  return(found);
}

/*True if they share a variable with the SAME exponent and function */
boolean VarSetIntersect(Tvariable_set *vs1,Tvariable_set *vs2)
{
  boolean found=FALSE;
  unsigned int i;
  
  i=0;
  while((i<vs1->nvars)&&(!found))
    {
      found=Included(vs1->var_id[i],vs2);
      if (!found) i++;
    }
  return(found);
}

/*result of v1+v2 : we want to keep all the variables and the maximum of the exponents
  in v1,v2 */
void UnionVarSet(Tvariable_set *vs_new,Tvariable_set *vs)
{
  unsigned int i,j,n,id;
  boolean found;

  for(i=0;i<vs_new->nvars;i++)
    {  
      /* Size of 'vs' changes as we add variables (and the order
         of variables in 'vs' change!!) */
      n=vs->nvars;

      id=vs_new->var_id[i];
      found=FALSE;
      j=0;
      while((j<n)&&(!found)&&(vs->var_id[j]<=id))
	{
	  found=(vs->var_id[j]==id);
	  if (!found)
	    j++;
	}

      if (!found)
	AddVariableInt(id,vs);
    }
}

/*If already in the set, we just increse the exponent. We implement: (var_set)*x^p */
void AddVariable2Set(unsigned int varid,Tvariable_set *vs)
{
  boolean found=FALSE;
  unsigned int i;

  i=0;
  while((i<vs->nvars)&&(varid<=vs->var_id[i])&&(!found))
    {
      found=(varid==vs->var_id[i]);
      if (!found) i++;
    }
  if (found)
    Error("Non-linear system");
  else   
    AddVariableInt(varid,vs);
}

void ProductVarSet(Tvariable_set *vs1,Tvariable_set *vs2,Tvariable_set *v_out)
{
  Tvariable_set *v1,*v2;
  unsigned int i;

  if ((vs1->nvars>vs2->nvars)||(vs1==v_out))
    {
      v1=vs1;
      v2=vs2;
    }
  else
    {
      v1=vs2;
      v2=vs1;
    }

  if (v1!=v_out)
    CopyVarSet(v_out,v1);
  for(i=0;i<v2->nvars;i++)
    AddVariable2Set(v2->var_id[i],v_out);
}

void RemoveVariableFromSet(unsigned int varid,Tvariable_set *vs)
{
  unsigned int i,k;

  k=0;
  for(i=0;i<vs->nvars;i++)
    {
      if (vs->var_id[i]!=varid)
	{
	  vs->var_id[k]=vs->var_id[i];
	  if (vs->var_id[k]>varid)
	    vs->var_id[k]--;
	  k++;
	}
    }
  vs->nvars=k;
}


double ReplaceVariableInVarSet(unsigned int varid,double ct,unsigned int newID,Tvariable_set *vs)
{
  double r;
  unsigned int i;
  Tvariable_set vTmp;

  InitVarSet(&vTmp);
  r=1.0;
  for(i=0;i<vs->nvars;i++)
    { 
      r*=ct;

      AddVariable2Set(vs->var_id[i]==varid?newID:vs->var_id[i],&vTmp);
    }
  DeleteVarSet(vs);
  CopyVarSet(vs,&vTmp);
  
  return(r);
}

inline unsigned int VariableSetSize(Tvariable_set *vs)
{
  return(vs->nvars);
}

inline unsigned int GetVariableN(unsigned int n,Tvariable_set *vs)
{
  #if (_DEBUG>0)
    if (n>=vs->nvars)
      Error("Requested an element not included in VarSet");
  #endif
  return(vs->var_id[n]);
}

inline unsigned int *GetVariables(Tvariable_set *vs)
{
  return(vs->var_id);
}

inline double EvaluateVarSet(double *varValues,Tvariable_set *vs)
{
  double f;
  unsigned int i,n;

  #if (_DEBUG>2)
    if (vs->nvars==0)
      Error("Should not evalute empty var sets");
  #endif

  f=1.0;
  for(i=0;i<vs->nvars;i++)
    {
      n=vs->var_id[i];
      f*=varValues[n];
    }

  return(f);
}

void PrintVarSet(FILE *f,char **varNames,Tvariable_set *vs)
{
  unsigned int i;
  
  if (vs->nvars>0)
    {
      for(i=0;i<vs->nvars;i++)
	{
	  if (i>0) 
	    fprintf(f,"*");
 
	  if (varNames!=NULL)
	    { 
	      char *n;

	      n=varNames[vs->var_id[i]];
	      PRINT_VARIABLE_NAME(f,n);
	    }
	  else
	    fprintf(f,"t_%u",vs->var_id[i]);
	}
    }
}

void DeleteVarSet(Tvariable_set *vs)
{
  free(vs->var_id);
}
