#include "variables.h"

#include "error.h"
#include "boolean.h"
#include "defines.h"

#include <stdlib.h>
#include <string.h>


/** 
   \file variables.c

   \brief Implementation of the functions operating on box Tvariables. 

   Implementation of the functions operating on box Tvariables. 

   \sa Tvariables, variables.h.
*/

/*
 * Init the set of variables.
 */
void InitVariables(Tvariables *vs)
{
  vs->max_variables=INIT_NUM_VARIABLES;
  NEW(vs->vs,vs->max_variables,Tvariable *);
  vs->n=0; /* total */
  vs->no=0; /* offset */ 
}

void CopyVariables(Tvariables *vs_dst,Tvariables *vs_src)
{
  unsigned int i;

  vs_dst->n=vs_src->n;
  vs_dst->no=vs_src->no;

  vs_dst->max_variables=vs_src->max_variables;
  NEW(vs_dst->vs,vs_dst->max_variables,Tvariable *);

  for(i=0;i<vs_src->n;i++)
    {
      NEW(vs_dst->vs[i],1,Tvariable);
      CopyVariable(vs_dst->vs[i],vs_src->vs[i]);
    }
}

/*
 * Returns the number of variables included in the set 'vs'
 */
unsigned int NVariables(Tvariables *vs)
{
  return(vs->n);
}

/*
 * Returns the number of offset variables included in the set 'vs'
 */
unsigned int NOffsetVariables(Tvariables *vs)
{
  return(vs->no);
}

/*
 * Add a new variable (pointed by 'v') to the set 'vs'.
 * If 'v' has the same name of an already existing variable in 'vs' and error is issued (and
 * the program stopped).
 * Returns the ID assigned to this variable (the position where it is inserted) and not its order
 * in the set (first, second,...)
 */
unsigned int AddVariable(Tvariable *v,Tvariables *vs)
{
  if (GetVariableWithName(GetVariableName(v),vs)!=NULL)
    Error("Duplicated variable");
  else
    {
      signed int k;
      
      /*look for a  free position*/
      if (vs->n==vs->max_variables)
	{
	  MEM_DUP(vs->vs,vs->max_variables,Tvariable *);
	}
      
      /* Let's put offset variables at the end of the list. In this way we can
         fix the offsets without changing the index of the non-offset variables */
      k=vs->n;
      while((k>0)&&(IsOffsetVariable(vs->vs[k-1])))
	{
	  vs->vs[k]=vs->vs[k-1];
	  k--;
	}
	  
      NEW(vs->vs[k],1,Tvariable);
      CopyVariable(vs->vs[k],v);
      vs->n++;
     
      if (IsOffsetVariable(v))
	vs->no++;
    }
  return(vs->n-1);
}

/*
 * Returns a pointer to the variable description of the variable with the given 'name'
 * If there is no variable with that name NULL is returned.
 */
Tvariable *GetVariableWithName(char *name,Tvariables *vs)
{
  unsigned int i;

  i=GetVariableID(name,vs);
  if (i==NO_UINT)
    return(NULL);
  else
    return(vs->vs[i]);
}

/*
 * Returns a pointer to the variable with n
 */ 
Tvariable *GetVariable(unsigned int n,Tvariables *vs)
{
  if (n>=vs->n)
    Error("Undefined variable referenced in function GetVariable");

  return(vs->vs[n]);    
}

/*
 * Returns the ID of variable with name 'name'.
 * returns -1 if there is no variable with this name
 */
unsigned int GetVariableID(char *name,Tvariables *vs)
{
  boolean found;
  unsigned int i;

  found=FALSE;
  i=0;
  while((!found)&&(i<vs->n))
    {
      if (strcmp(name,GetVariableName(vs->vs[i]))==0)
	found=TRUE;
      else
	i++;
    }
  
  if (found)
    return(i);
  else
    return(NO_UINT);
}

void GetVariableNames(char **varNames,Tvariables *vs)
{
  unsigned int i;
  
  for(i=0;i<vs->n;i++)
    varNames[i]=GetVariableName(vs->vs[i]);
}

char *VariableName(unsigned int i,Tvariables *vs)
{
  if (i<vs->n)
    return(GetVariableName(vs->vs[i]));
  else
    return(NULL);
}

void PointFromVariables(double **p,Tvariables *vs)
{
  unsigned int i;

  NEW(*p,vs->n,double);

  for(i=0;i<vs->n;i++) 
    (*p)[i]=IntervalCenter(GetVariableInterval(GetVariable(i,vs)));
}

/*
 * Prints the set of variables 'vs' on the file 'f'
 */
void PrintVariables(FILE *f,Tvariables *vs)
{
  unsigned int i;
  
  fprintf(f,"\n[VARS]\n");
 
  for(i=0;i<vs->n;i++)
    {
      if (!IsOffsetVariable(vs->vs[i]))
	{
	  fprintf(f,"   ");
	  PrintVariable(f,vs->vs[i]);
	  fprintf(f,"\n");
	  //fprintf(f," %% %u\n",i);
	}
    }
  
  fprintf(f,"\n[OFFSETS]\n");
 
  for(i=0;i<vs->n;i++)
    {
      if (IsOffsetVariable(vs->vs[i]))
	{
	  fprintf(f,"   ");
	  PrintVariable(f,vs->vs[i]);
	  fprintf(f," : ");
	  PRINT_VARIABLE_NAME(f,GetVariableName(vs->vs[ShiftedVariable(vs->vs[i])]));
	  fprintf(f,"\n");
	  //fprintf(f," %% %u\n",i);
	}
    }
}

/*
 * Deletes the set of variables 'vs' (recursively deleting all the included variables).
 */
void DeleteVariables(Tvariables *vs)
{
  unsigned int i;

  for(i=0;i<vs->n;i++)
    {	    
      DeleteVariable(vs->vs[i]);
      free(vs->vs[i]);
    }
  free(vs->vs);
}

